Package: SparseSignatures
Version: 2.20.0
Date: 2025-09-23
Title: SparseSignatures
Authors@R: c(
    person("Daniele", "Ramazzotti", role=c("aut"),email="daniele.ramazzotti@unimib.it",
                comment = c(ORCID = "0000-0002-6087-2666")),
    person("Avantika", "Lal", role=c("aut")),
    person("Keli", "Liu", role=c("ctb")),
    person("Luca", "De Sano", role=c("cre","aut"), email="luca.desano@gmail.com",
                comment = c(ORCID = "0000-0002-9618-3774")),
    person("Robert", "Tibshirani", role=c("ctb")),
    person("Arend", "Sidow", role=c("aut")))
Depends: R (>= 4.1.0), NMF
Imports: nnlasso, nnls, parallel, data.table, Biostrings,
        GenomicRanges, IRanges, BSgenome, GenomeInfoDb, ggplot2,
        gridExtra, reshape2, RhpcBLASctl
Suggests: BiocGenerics, BSgenome.Hsapiens.1000genomes.hs37d5,
        BiocStyle, testthat, knitr,
Name: An R package for the extraction of sparse mutational signatures
        from whole genome sequencing data
Description: 
    Point mutations occurring in a genome can be divided into 96 categories based on the base being mutated, the base it is mutated into and its two flanking bases. Therefore, for any patient, it is possible to represent all the point mutations occurring in that patient's tumor as a vector of length 96, where each element represents the count of mutations for a given category in the patient. A mutational signature represents the pattern of mutations produced by a mutagen or mutagenic process inside the cell. Each signature can also be represented by a vector of length 96, where each element represents the probability that this particular mutagenic process generates a mutation of the 96 above mentioned categories. In this R package, we provide a set of functions to extract and visualize the mutational signatures that best explain the mutation counts of a large number of patients.
Encoding: UTF-8
License: file LICENSE
URL: https://github.com/danro9685/SparseSignatures
BugReports: https://github.com/danro9685/SparseSignatures
biocViews: BiomedicalInformatics, SomaticMutation
RoxygenNote: 7.3.2
VignetteBuilder: knitr
git_url: https://git.bioconductor.org/packages/SparseSignatures
git_branch: RELEASE_3_22
git_last_commit: 9b8de11
git_last_commit_date: 2025-10-29
Repository: Bioconductor 3.22
Date/Publication: 2025-10-29
NeedsCompilation: no
Packaged: 2025-10-30 06:42:14 UTC; biocbuild
Author: Daniele Ramazzotti [aut] (ORCID:
    <https://orcid.org/0000-0002-6087-2666>),
  Avantika Lal [aut],
  Keli Liu [ctb],
  Luca De Sano [cre, aut] (ORCID:
    <https://orcid.org/0000-0002-9618-3774>),
  Robert Tibshirani [ctb],
  Arend Sidow [aut]
Maintainer: Luca De Sano <luca.desano@gmail.com>

# Generated by roxygen2: do not edit by hand

export(import.trinucleotides.counts)
export(lambdaRangeAlphaEvaluation)
export(lambdaRangeBetaEvaluation)
export(nmfLasso)
export(nmfLassoBootstrap)
export(nmfLassoCV)
export(patients.plot)
export(sigAssignmentCV)
export(sigAssignmentEvaluation)
export(sigAssignmentLasso)
export(signatures.plot)
export(startingBetaEstimation)
import(GenomeInfoDb)
import(GenomicRanges)
import(IRanges)
import(NMF)
import(RhpcBLASctl)
import(ggplot2)
import(gridExtra)
import(nnlasso)
import(nnls)
import(parallel)
importFrom(BSgenome,getSeq)
importFrom(Biostrings,DNAStringSet)
importFrom(Biostrings,complement)
importFrom(Biostrings,reverseComplement)
importFrom(Biostrings,subseq)
importFrom(data.table,":=")
importFrom(data.table,.N)
importFrom(data.table,as.data.table)
importFrom(data.table,data.table)
importFrom(data.table,dcast)
importFrom(reshape2,melt)

#' Perform the assessment of different nmfLasso solutions by cross validation for K (unknown) somatic mutational signatures given a set of observations x. The estimation can slow down because of 
#' memory usage and intensive computations, when a big number of cross validation repetitions is asked and when the grid search is performed for a lot of configurations. In this case, 
#' an advice may be to split the computation into multiple smaller sets.
#'
#' @examples
#' data(background)
#' data(patients)
#' res = nmfLassoCV(x=patients[1:100,],
#'      K=3:5,
#'      background_signature=background,
#'      nmf_runs=1,
#'      lambda_values_alpha=c(0.00),
#'      lambda_values_beta=c(0.00),
#'      cross_validation_repetitions=2,
#'      num_processes=NA,
#'      seed=12345)
#'
#' @title nmfLassoCV
#' @param x count matrix for a set of n patients and 96 trinucleotides.
#' @param K a range of numeric value (each of them greater than 1) indicating the number of signatures to be discovered.
#' @param starting_beta a list of starting beta value for each configuration of K. If it is NULL, starting betas are estimated by 
#' NMF.
#' @param background_signature background signature to be used. If not provided, a warning is thrown and an initial value for it is 
#' estimated by NMF. If beta is not NULL, this parameter is ignored.
#' @param normalize_counts if true, the input count matrix x is normalize such that the patients have the same number of mutation.
#' @param nmf_runs number of iteration (minimum 1) of NMF to be performed for a robust estimation of starting beta. If beta is not NULL, 
#' this parameter is ignored.
#' @param lambda_values_alpha value of LASSO to be used for alpha between 0 and 1. This value should be greater than 0. 1 is the value of LASSO 
#' that would shrink all the signatures to 0 within one step. The higher lambda_rate_alpha is, the sparser are the resulting exposures, 
#' but too large values may result in a reduced fit of the observed counts.
#' @param lambda_values_beta value of LASSO to be used for beta between 0 and 1. This value should be greater than 0. 1 is the value of LASSO 
#' that would shrink all the signatures to 0 within one step. The higher lambda_rate_beta is, the sparser are the resulting exposures, 
#' but too large values may result in a reduced fit of the observed counts.
#' @param cross_validation_entries Percentage of cells in the count matrix to be replaced by 0s during cross validation.
#' @param cross_validation_iterations For each configuration, the first time the signatures are discovered form a matrix with a 
#' percentage of values replaced by 0s. This may result in poor fit/results. Then, we perform predictions of these entries and replace them with 
#' such predicted values. This parameter is the number of restarts to be performed to improve this estimate and obtain more stable solutions.
#' @param cross_validation_repetitions Number of time cross-validation should be repeated. Higher values result in better estimate, but are computationally more expensive.
#' @param iterations Number of iterations to be performed. Each iteration corresponds to a first step where beta is fitted 
#' and a second step where alpha is fitted.
#' @param max_iterations_lasso Number of maximum iterations to be performed during the sparsification via Lasso.
#' @param num_processes Number of processes to be used during parallel execution. To execute in single process mode, 
#' this parameter needs to be set to either NA or NULL.
#' @param seed Seed for reproducibility.
#' @param verbose boolean; Shall I print all messages?
#' @param log_file log file where to print outputs when using parallel. If parallel execution is disabled, this parameter is ignored.
#' @return A list of 2 elements: grid_search_mse and and grid_search_loglik. Here, grid_search_mse reports the mean squared error for each configuration of performed 
#' cross validation; grid_search_loglik reports for each configuration the number of times the algorithm converged.
#' @export nmfLassoCV
#' @import NMF
#' @import nnlasso
#' @import nnls
#' @import RhpcBLASctl
#' @import parallel
#'
"nmfLassoCV" <- function( x, K = 3:10, starting_beta = NULL, background_signature = NULL, normalize_counts = TRUE, nmf_runs = 10, lambda_values_alpha = c(0.00, 0.01, 0.05, 0.10), lambda_values_beta = c(0.00, 0.01, 0.05, 0.10), cross_validation_entries = 0.01, cross_validation_iterations = 5, cross_validation_repetitions = 50, iterations = 30, max_iterations_lasso = 10000, num_processes = Inf, seed = NULL, verbose = TRUE, log_file = "" ) {
    
    # set the seed
    set.seed(seed)

    # check the input parameters
    if(min(K)<2) {
        if(verbose) {
            cat("The minimum value of K is 2...","\n")
        }
        K <- K[K>=2]
        if(length(K)==0) {
            K <- 2
            warning("No value of K greater than 2 is provided: setting K to 2...")
        }
    }
    if(nmf_runs<1) {
        if(verbose) {
            cat("The minimum value of nmf_runs is 1...","\n")
        }
        nmf_runs <- 1
    }
    x <- as.matrix(x)
    if(normalize_counts) {
        x <- (x/rowSums(x))*2500
    }
    lambda_values_alpha <- sort(unique(lambda_values_alpha))
    lambda_values_beta <- sort(unique(lambda_values_beta))

    # perform a grid search to estimate the best values of K and lambda
    if(verbose) {
        cat("Performing a grid search to estimate the best values of K and lambda with a total of",cross_validation_repetitions,"cross validation repetitions...","\n")
    }

    # setting up parallel execution
    parallel <- NULL
    close_parallel <- FALSE
    if(is.null(parallel)&&cross_validation_repetitions>1) {
        if(is.na(num_processes) || is.null(num_processes)) {
            parallel <- NULL
        }
        else if(num_processes==Inf) {
            cores <- as.integer((detectCores()-1))
            if(cores < 2) {
                parallel <- NULL
            }
            else {
                num_processes <- min(num_processes,cross_validation_repetitions)
                parallel <- makeCluster(num_processes,outfile=log_file)
                close_parallel <- TRUE
            }
        }
        else {
            num_processes <- min(num_processes,cross_validation_repetitions)
            parallel <- makeCluster(num_processes,outfile=log_file)
            close_parallel <- TRUE
        }
        
        if(verbose && !is.null(parallel)) {
            cat("Executing",num_processes,"processes via parallel...","\n")
        }
    }

    # estimate the background signature
    if(is.null(background_signature)&&is.null(starting_beta)) {
        warning("No background signature has been specified...")
        background_signature_beta <- basis(nmf(t(x),rank=1,nrun=nmf_runs))
        background_signature_beta <- t(background_signature_beta)
        background_signature_beta <- background_signature_beta / rowSums(background_signature_beta)
        background_signature <- as.numeric(background_signature_beta[1,])
    }
    
    # structure to save the starting values of beta for each K
    if(is.null(starting_beta)) {
        
        background_signature <- as.numeric(background_signature)
        background_signature <- background_signature / sum(background_signature)

        starting_beta <- array(list(),c(length(K),1))
        rownames(starting_beta) <- paste0(as.character(K),"_Signatures")
        colnames(starting_beta) <- "Value"

        # estimate the contribution of the background to the observed counts
        background_signature_beta <- t(background_signature)
        background_signature_alpha <- matrix(0,nrow=dim(x)[1],ncol=1)
        for(i in 1:dim(x)[1]) {
            blas_set_num_threads(1)
            background_signature_alpha[i,] <- nnls(t(background_signature_beta),as.vector(x[i,]))$x
            blas_set_num_threads(blas_get_num_procs()) # reset to default
        }
        curr_x <- x - (background_signature_alpha %*% background_signature_beta)
        curr_x[curr_x<0] <- 0
        if(length(which(rowSums(t(curr_x))==0))>0) {
            invalid_cols <- as.numeric(which(rowSums(t(curr_x))==0))
            for(inv_cols in invalid_cols) {
                curr_x[sample(1:length(curr_x[,inv_cols]),size=1),inv_cols] <- 1e-05
            }
        }

        # compute starting values for beta
        pos_k <- 0
        for(k in K) {
            # compute the initial values of beta
            if(verbose) {
                cat(paste0("Computing the initial values of beta by standard NMF for K equals to ",k,"..."),"\n")
            }
            pos_k <- pos_k + 1
            beta <- NULL
            beta <- basis(nmf(t(curr_x),rank=(k-1),nrun=nmf_runs))
            beta <- t(beta)
            beta <- rbind(background_signature,beta)
            
            # normalize and save beta for the current value of K
            rownames(beta) <- c("Background",paste0("S",1:(nrow(beta)-1)))
            colnames(beta) <- colnames(x)
            beta <- beta / rowSums(beta)
            starting_beta[[pos_k,1]] <- beta
        }

    }

    if(verbose) {
        cat("Starting cross validation with a total of",cross_validation_repetitions,"repetitions...","\n")
    }

    # structure to save the results of the grid search
    grid_search_mse <- array(list(),c(length(lambda_values_alpha),length(lambda_values_beta),length(K)),dimnames=list(paste0(as.character(lambda_values_alpha),"_Lambda_Alpha"),paste0(as.character(lambda_values_beta),"_Lambda_Beta"),paste0(as.character(K),"_Signatures")))
    grid_search_loglik <- array(list(),c(length(lambda_values_alpha),length(lambda_values_beta),length(K)),dimnames=list(paste0(as.character(lambda_values_alpha),"_Lambda_Alpha"),paste0(as.character(lambda_values_beta),"_Lambda_Beta"),paste0(as.character(K),"_Signatures")))

    # now starting cross validation
    if(is.null(parallel)) {

        # perform a total of cross_validation_repetitions repetitions of cross validation
        set.seed(round(runif(1)*100000))
        for(cv_repetitions in 1:cross_validation_repetitions) {

            if(verbose) {
                cat(paste0("Performing repetition ",cv_repetitions," out of ",cross_validation_repetitions,"..."),"\n")
            }

            # randomly set the cross validation entries for the current iteration
            valid_entries <- which(x>0,arr.ind=TRUE)
            cv_entries <- valid_entries[sample(1:nrow(valid_entries),size=round(nrow(valid_entries)*cross_validation_entries),replace=FALSE),]

            # consider all the values for K
            cont <- 0
            pos_k <- 0
            for(k in K) {
                # consider the first k signatures to be used for the current configuration
                pos_k <- pos_k + 1

                # consider all the values for lambda
                pos_l_alpha <- 0
                for(l_alpha in lambda_values_alpha) {
                    pos_l_alpha <- pos_l_alpha + 1

                    pos_l_beta <- 0
                    for(l_beta in lambda_values_beta) {
                        pos_l_beta <- pos_l_beta + 1

                        # repeat the estimation for a number of cross_validation_iterations
                        x_cv <- NULL
                        cont <- cont + 1
                        for(cv_iteration in 1:cross_validation_iterations) {
                    
                            if(verbose) {
                                cat(paste0("Performing cross validation iteration ",cv_iteration," out of ",cross_validation_iterations,"..."),"\n")
                            }

                            # set a percentage of cross_validation_entries entries to 0 in order to perform cross validation
                            if(cv_iteration==1) {
                                x_cv <- x
                                x_cv[cv_entries] <- 0
                            }
                            else {
                                if(!is.na(curr_iter_alpha[1])&&!is.na(curr_iter_beta[1])) {
                                    predicted_counts <- curr_iter_alpha %*% curr_iter_beta
                                    x_cv[cv_entries] <- predicted_counts[cv_entries]
                                }
                            }

                            # perform the inference
                            curr_results <- nmfLasso(x = x_cv, 
                                                     K = k, 
                                                     beta = starting_beta[[pos_k,1]], 
                                                     background_signature = background_signature, 
                                                     normalize_counts = normalize_counts, 
                                                     nmf_runs = 10, 
                                                     lambda_rate_alpha = l_alpha, 
                                                     lambda_rate_beta = l_beta, 
                                                     iterations = iterations, 
                                                     max_iterations_lasso = max_iterations_lasso, 
                                                     seed = round(runif(1)*100000), 
                                                     verbose = FALSE)

                            curr_iter_alpha <- curr_results[["alpha"]]
                            curr_iter_beta <- curr_results[["beta"]]
                            curr_iter_loglik <- curr_results[["loglik_progression"]]

                        }
                        
                        # save an estimate of mean squared error and stability of the solution
                        if(!is.na(curr_iter_alpha[1])&&!is.na(curr_iter_beta[1])) {
                            curr_predicted_counts <- round(curr_iter_alpha%*%curr_iter_beta)
                            curr_true_considered_counts <- as.vector(x[cv_entries])
                            curr_predicted_considered_counts <- as.vector(curr_predicted_counts[cv_entries])
                            error <- mean((curr_true_considered_counts-curr_predicted_considered_counts)^2)
                            grid_search_mse[pos_l_alpha,pos_l_beta,pos_k][[1]] <- c(grid_search_mse[pos_l_alpha,pos_l_beta,pos_k][[1]],error)
                        }
                        if((!is.na(curr_iter_loglik)[1])&&(curr_iter_loglik[1]<curr_iter_loglik[length(curr_iter_loglik)])) {
                            grid_search_loglik[pos_l_alpha,pos_l_beta,pos_k][[1]] <- c(grid_search_loglik[pos_l_alpha,pos_l_beta,pos_k][[1]],"increasing")
                        }
                        else if((!is.na(curr_iter_loglik)[1])&&(curr_iter_loglik[1]>=curr_iter_loglik[length(curr_iter_loglik)])) {
                            grid_search_loglik[pos_l_alpha,pos_l_beta,pos_k][[1]] <- c(grid_search_loglik[pos_l_alpha,pos_l_beta,pos_k][[1]],"decreasing")
                        }
                        
                        if(verbose) {
                            cat("Progress",paste0(round((cont/(length(K)*length(lambda_values_alpha)*length(lambda_values_beta)))*100,digits=3),"%..."),"\n")
                        }

                    }

                }

            }

        }

        # save the results
        results <- list(grid_search_mse=grid_search_mse,grid_search_loglik=grid_search_loglik)

    }
    else {

        # perform the inference
        res_clusterEvalQ <- clusterEvalQ(parallel,library("nnls"))
        res_clusterEvalQ <- clusterEvalQ(parallel,library("RhpcBLASctl"))
        res_clusterEvalQ <- clusterEvalQ(parallel,library("nnlasso"))
        clusterExport(parallel,varlist=c("x","K","starting_beta","background_signature","normalize_counts","cross_validation_entries"),envir=environment())
        clusterExport(parallel,varlist=c("lambda_values_alpha","lambda_values_beta","cross_validation_iterations","iterations","max_iterations_lasso"),envir=environment())
        clusterExport(parallel,varlist=c("grid_search_mse","grid_search_loglik","cross_validation_repetitions","verbose"),envir=environment())
        clusterExport(parallel,c('nmfLasso','nmfLassoDecomposition','basis','nmf'),envir=environment())
        clusterSetRNGStream(parallel,iseed=round(runif(1)*100000))
        curr_results <- parLapply(parallel,1:cross_validation_repetitions,function(cv_rep) {

            if(verbose) {
                cat(paste0("Performing repetition ",cv_rep," out of ",cross_validation_repetitions,"..."),"\n")
            }

            # randomly set the cross validation entries for the current iteration
            valid_entries <- which(x>0,arr.ind=TRUE)
            cv_entries <- valid_entries[sample(1:nrow(valid_entries),size=round(nrow(valid_entries)*cross_validation_entries),replace=FALSE),]

            # consider all the values for K
            cont <- 0
            pos_k <- 0
            for(k in K) {
                # consider the first k signatures to be used for the current configuration
                pos_k <- pos_k + 1

                # consider all the values for lambda
                pos_l_alpha <- 0
                for(l_alpha in lambda_values_alpha) {
                    pos_l_alpha <- pos_l_alpha + 1

                    pos_l_beta <- 0
                    for(l_beta in lambda_values_beta) {
                        pos_l_beta <- pos_l_beta + 1

                        # repeat the estimation for a number of cross_validation_iterations
                        x_cv <- NULL
                        cont <- cont + 1
                        for(cv_iteration in 1:cross_validation_iterations) {
                    
                            # set a percentage of cross_validation_entries entries to 0 in order to perform cross validation
                            if(cv_iteration==1) {
                                x_cv <- x
                                x_cv[cv_entries] <- 0
                            }
                            else {
                                if(!is.na(curr_iter_alpha[1])&&!is.na(curr_iter_beta[1])) {
                                    predicted_counts <- curr_iter_alpha %*% curr_iter_beta
                                    x_cv[cv_entries] <- predicted_counts[cv_entries]
                                }
                            }

                            # perform the inference
                            curr_results <- nmfLasso(x = x_cv, 
                                                     K = k, 
                                                     beta = starting_beta[[pos_k,1]], 
                                                     background_signature = background_signature, 
                                                     normalize_counts = normalize_counts, 
                                                     nmf_runs = 10, 
                                                     lambda_rate_alpha = l_alpha, 
                                                     lambda_rate_beta = l_beta, 
                                                     iterations = iterations, 
                                                     max_iterations_lasso = max_iterations_lasso, 
                                                     seed = round(runif(1)*100000), 
                                                     verbose = FALSE)

                            curr_iter_alpha <- curr_results[["alpha"]]
                            curr_iter_beta <- curr_results[["beta"]]
                            curr_iter_loglik <- curr_results[["loglik_progression"]]

                        }
                        
                        # save an estimate of mean squared error and stability of the solution
                        if(!is.na(curr_iter_alpha[1])&&!is.na(curr_iter_beta[1])) {
                            curr_predicted_counts <- round(curr_iter_alpha%*%curr_iter_beta)
                            curr_true_considered_counts <- as.vector(x[cv_entries])
                            curr_predicted_considered_counts <- as.vector(curr_predicted_counts[cv_entries])
                            error <- mean((curr_true_considered_counts-curr_predicted_considered_counts)^2)
                            grid_search_mse[pos_l_alpha,pos_l_beta,pos_k][[1]] <- c(grid_search_mse[pos_l_alpha,pos_l_beta,pos_k][[1]],error)
                        }
                        if((!is.na(curr_iter_loglik)[1])&&(curr_iter_loglik[1]<curr_iter_loglik[length(curr_iter_loglik)])) {
                            grid_search_loglik[pos_l_alpha,pos_l_beta,pos_k][[1]] <- c(grid_search_loglik[pos_l_alpha,pos_l_beta,pos_k][[1]],"increasing")
                        }
                        else if((!is.na(curr_iter_loglik)[1])&&(curr_iter_loglik[1]>=curr_iter_loglik[length(curr_iter_loglik)])) {
                            grid_search_loglik[pos_l_alpha,pos_l_beta,pos_k][[1]] <- c(grid_search_loglik[pos_l_alpha,pos_l_beta,pos_k][[1]],"decreasing")
                        }

                    }

                }

            }

            # save the results
            curr_results <- list(grid_search_mse=grid_search_mse,grid_search_loglik=grid_search_loglik)

            return(curr_results)

        })

        # save the results
        results <- list(grid_search_mse=grid_search_mse,grid_search_loglik=grid_search_loglik)
        for(par_res in 1:length(curr_results)) {
            curr_par_res <- curr_results[[par_res]]
            curr_par_res_grid_search_mse <- curr_par_res$grid_search_mse
            curr_par_res_grid_search_loglik <- curr_par_res$grid_search_loglik
            for(a in 1:dim(curr_par_res_grid_search_mse)[1]) {
                for(b in 1:dim(curr_par_res_grid_search_mse)[2]) {
                    for(c in 1:dim(curr_par_res_grid_search_mse)[3]) {
                        if(!is.null(unlist(curr_par_res_grid_search_mse[a,b,c]))) {
                            results$grid_search_mse[[a,b,c]] <- c(unlist(results$grid_search_mse[a,b,c]),unlist(curr_par_res_grid_search_mse[a,b,c]))
                            results$grid_search_loglik[[a,b,c]] <- c(unlist(results$grid_search_loglik[a,b,c]),unlist(curr_par_res_grid_search_loglik[a,b,c]))
                        }
                    }
                }
            }
        }

    }
    
    # close parallel
    if(close_parallel) {
        stopCluster(parallel)
    }
    
    return(results)
    
}

#' Perform the evaluation of different nmfLasso solutions by bootstrap for K (unknown) somatic mutational signatures given a set of observations x. The estimation can slow down because of 
#' memory usage and intensive computations, when a big number of bootstrap repetitions is asked and when the analysis is performed for a big range of signatures (K). In this case, 
#' an advice may be to split the computation into multiple smaller sets.
#'
#' @examples
#' data(background)
#' data(patients)
#' res = nmfLassoBootstrap(x=patients[1:100,],
#'      K=3:5,
#'      background_signature=background,
#'      nmf_runs=1,
#'      bootstrap_repetitions=2,
#'      num_processes=NA,
#'      seed=12345)
#'
#' @title nmfLassoBootstrap
#' @param x count matrix for a set of n patients and 96 trinucleotides.
#' @param K a range of numeric value (each of them greater than 1) indicating the number of signatures to be discovered.
#' @param starting_beta a list of starting beta value for each configuration of K. If it is NULL, starting betas are estimated by 
#' NMF.
#' @param background_signature background signature to be used. If not provided, a warning is thrown and an initial value for it is 
#' estimated by NMF. If beta is not NULL, this parameter is ignored.
#' @param normalize_counts if true, the input count matrix x is normalize such that the patients have the same number of mutation.
#' @param nmf_runs number of iteration (minimum 1) of NMF to be performed for a robust estimation of starting beta. If beta is not NULL, 
#' this parameter is ignored.
#' @param bootstrap_repetitions Number of time bootstrap should be repeated. Higher values result in better estimate, but are computationally more expensive.
#' @param iterations Number of iterations to be performed. Each iteration corresponds to a first step where beta is fitted 
#' and a second step where alpha is fitted.
#' @param max_iterations_lasso Number of maximum iterations to be performed during the sparsification via Lasso.
#' @param num_processes Number of processes to be used during parallel execution. To execute in single process mode, 
#' this parameter needs to be set to either NA or NULL.
#' @param seed Seed for reproducibility.
#' @param verbose boolean; Shall I print all messages?
#' @param log_file log file where to print outputs when using parallel. If parallel execution is disabled, this parameter is ignored.
#' @return A list of 3 elements: stability, RSS and evar. Here, stability reports the estimared cosine similarity for alpha and beta at each bootstrap 
#' repetition; RSS reports for each configuration the estimated residual sum of squares; finally, evar reports the explained variance.
#' @export nmfLassoBootstrap
#' @import NMF
#' @import nnlasso
#' @import nnls
#' @import RhpcBLASctl
#' @import parallel
#'
"nmfLassoBootstrap" <- function( x, K = 3:10, starting_beta = NULL, background_signature = NULL, normalize_counts = TRUE, nmf_runs = 10, bootstrap_repetitions = 50, iterations = 30, max_iterations_lasso = 10000, num_processes = Inf, seed = NULL, verbose = TRUE, log_file = "" ) {
    
    # set the seed
    set.seed(seed)

    # check the input parameters
    if(min(K)<2) {
        if(verbose) {
            cat("The minimum value of K is 2...","\n")
        }
        K <- K[K>=2]
        if(length(K)==0) {
            K <- 2
            warning("No value of K greater than 2 is provided: setting K to 2...")
        }
    }
    if(nmf_runs<1) {
        if(verbose) {
            cat("The minimum value of nmf_runs is 1...","\n")
        }
        nmf_runs <- 1
    }
    x <- as.matrix(x)
    if(normalize_counts) {
        x <- (x/rowSums(x))*2500
    }

    # no Lasso is considered here to speed up computations, i.e., this is an heuristic/approximated result
    lambda_values_alpha <- 0.00
    lambda_values_beta <- 0.00

    # perform a grid search to estimate the best values of K and lambda
    if(verbose) {
        cat("Performing a total of",bootstrap_repetitions,"bootstrap repetitions to assess nmfLasso solutions for different K ranks...","\n")
    }

    # setting up parallel execution
    parallel <- NULL
    close_parallel <- FALSE
    if(is.null(parallel)&&bootstrap_repetitions>1) {
        if(is.na(num_processes) || is.null(num_processes)) {
            parallel <- NULL
        }
        else if(num_processes==Inf) {
            cores <- as.integer((detectCores()-1))
            if(cores < 2) {
                parallel <- NULL
            }
            else {
                num_processes <- min(num_processes,bootstrap_repetitions)
                parallel <- makeCluster(num_processes,outfile=log_file)
                close_parallel <- TRUE
            }
        }
        else {
            num_processes <- min(num_processes,bootstrap_repetitions)
            parallel <- makeCluster(num_processes,outfile=log_file)
            close_parallel <- TRUE
        }
        
        if(verbose && !is.null(parallel)) {
            cat("Executing",num_processes,"processes via parallel...","\n")
        }
    }

    # estimate the background signature
    if(is.null(background_signature)&&is.null(starting_beta)) {
        warning("No background signature has been specified...")
        background_signature_beta <- basis(nmf(t(x),rank=1,nrun=nmf_runs))
        background_signature_beta <- t(background_signature_beta)
        background_signature_beta <- background_signature_beta / rowSums(background_signature_beta)
        background_signature <- as.numeric(background_signature_beta[1,])
    }
    
    # structure to save the starting values of beta for each K
    if(is.null(starting_beta)) {
        
        background_signature <- as.numeric(background_signature)
        background_signature <- background_signature / sum(background_signature)

        starting_beta <- array(list(),c(length(K),1))
        rownames(starting_beta) <- paste0(as.character(K),"_Signatures")
        colnames(starting_beta) <- "Value"

        # estimate the contribution of the background to the observed counts
        background_signature_beta <- t(background_signature)
        background_signature_alpha <- matrix(0,nrow=dim(x)[1],ncol=1)
        for(i in 1:dim(x)[1]) {
            blas_set_num_threads(1)
            background_signature_alpha[i,] <- nnls(t(background_signature_beta),as.vector(x[i,]))$x
            blas_set_num_threads(blas_get_num_procs()) # reset to default
        }
        curr_x <- x - (background_signature_alpha %*% background_signature_beta)
        curr_x[curr_x<0] <- 0
        if(length(which(rowSums(t(curr_x))==0))>0) {
            invalid_cols <- as.numeric(which(rowSums(t(curr_x))==0))
            for(inv_cols in invalid_cols) {
                curr_x[sample(1:length(curr_x[,inv_cols]),size=1),inv_cols] <- 1e-05
            }
        }

        # compute starting values for beta
        pos_k <- 0
        for(k in K) {
            # compute the initial values of beta
            if(verbose) {
                cat(paste0("Computing the initial values of beta by standard NMF for K equals to ",k,"..."),"\n")
            }
            pos_k <- pos_k + 1
            beta <- NULL
            beta <- basis(nmf(t(curr_x),rank=(k-1),nrun=nmf_runs))
            beta <- t(beta)
            beta <- rbind(background_signature,beta)
            
            # normalize and save beta for the current value of K
            rownames(beta) <- c("Background",paste0("S",1:(nrow(beta)-1)))
            colnames(beta) <- colnames(x)
            beta <- beta / rowSums(beta)
            starting_beta[[pos_k,1]] <- beta
        }

    }

    if(verbose) {
        cat("Starting bootstrap for a total of",bootstrap_repetitions,"repetitions...","\n")
    }

    # structure to save the results of the grid search
    bootstrap_rss <- array(list(),c(length(K),2))
    rownames(bootstrap_rss) <- paste0(as.character(K),"_Signatures")
    colnames(bootstrap_rss) <- c("RSS bootstrapped data","RSS randomized data")
    bootstrap_evar <- bootstrap_rss
    colnames(bootstrap_evar) <- c("Explained Variance bootstrapped data","Explained Variance randomized data")
    bootstrap_stability <- array(list(),c(length(K),4))
    rownames(bootstrap_stability) <- paste0(as.character(K),"_Signatures")
    colnames(bootstrap_stability) <- c("Similarity alpha bootstrapped data","Similarity beta bootstrapped data","Similarity alpha randomized data","Similarity beta randomized data")
    starting_values_bootstrap = bootstrap_stability

    # generate a set of randomized data from x
    set.seed(round(runif(1)*100000))
    randomized_x <- x
    for(i in 1:ncol(randomized_x)) {
        randomized_x[,i] <- randomized_x[sample(1:dim(randomized_x)[1],size=dim(randomized_x)[1],replace=FALSE),i]
    }
    for(j in 1:nrow(randomized_x)) {
        randomized_x[j,] <- randomized_x[j,sample(1:dim(randomized_x)[2],size=dim(randomized_x)[2],replace=FALSE)]
    }

    # compute starting values of alpha and beta in order to estimate stability of the considered solutions (and ranks)
    pos_k <- 0
    for(k in K) {

        pos_k <- pos_k + 1

        is.valid <- FALSE
        while(!is.valid) {
            curr_results <- nmfLasso(x = x, 
                                     K = k, 
                                     beta = starting_beta[[pos_k,1]], 
                                     background_signature = background_signature, 
                                     normalize_counts = normalize_counts, 
                                     nmf_runs = 10, 
                                     lambda_rate_alpha = lambda_values_alpha, 
                                     lambda_rate_beta = lambda_values_beta, 
                                     iterations = iterations, 
                                     max_iterations_lasso = max_iterations_lasso, 
                                     seed = round(runif(1)*100000), 
                                     verbose = FALSE)
            if(!is.na(curr_results$best_loglik)) {
                is.valid <- TRUE
            }
        }

        starting_values_bootstrap[[pos_k,1]] <- curr_results$alpha
        starting_values_bootstrap[[pos_k,2]] <- curr_results$beta

        is.valid <- FALSE
        while(!is.valid) {
            curr_results <- nmfLasso(x = randomized_x, 
                                     K = k, 
                                     beta = starting_beta[[pos_k,1]], 
                                     background_signature = background_signature, 
                                     normalize_counts = normalize_counts, 
                                     nmf_runs = 10, 
                                     lambda_rate_alpha = lambda_values_alpha, 
                                     lambda_rate_beta = lambda_values_beta, 
                                     iterations = iterations, 
                                     max_iterations_lasso = max_iterations_lasso, 
                                     seed = round(runif(1)*100000), 
                                     verbose = FALSE)
            if(!is.na(curr_results$best_loglik)) {
                is.valid <- TRUE
            }
        }

        starting_values_bootstrap[[pos_k,3]] <- curr_results$alpha
        starting_values_bootstrap[[pos_k,4]] <- curr_results$beta

    }

    # now starting cross validation
    if(is.null(parallel)) {

        # perform a total of bootstrap_repetitions repetitions of bootstrap
        for(nboot in 1:bootstrap_repetitions) {

            if(verbose) {
                cat(paste0("Performing repetition ",nboot," out of ",bootstrap_repetitions,"..."),"\n")
            }

            # generate bootstrapped dataset from data
            bootstrapped_data <- x
            for(i in 1:nrow(x)) {
                num_mutations <- sum(x[i,])
                mut_distribution <- as.numeric(x[i,]/sum(x[i,]))
                sampling <- sample(1:96,size=num_mutations,replace=TRUE,prob=mut_distribution)
                bootstrapped_data[i,] <- 0
                for(j in sampling) {
                    bootstrapped_data[i,j] <- bootstrapped_data[i,j] + 1
                }
            }
            
            # generate randomized data from the bootstrapped data
            randomized_data <- bootstrapped_data
            for(i in 1:ncol(randomized_data)) {
                randomized_data[,i] <- randomized_data[sample(1:dim(randomized_data)[1],size=dim(randomized_data)[1],replace=FALSE),i]
            }
            for(j in 1:nrow(randomized_data)) {
                randomized_data[j,] <- randomized_data[j,sample(1:dim(randomized_data)[2],size=dim(randomized_data)[2],replace=FALSE)]
            }
    
            # perform inference on bootstrapped and randomized data
            pos_k <- 0
            for(k in K) {
                
                pos_k <- pos_k + 1

                # perform the analysis for bootstrapped data
                res <- nmfLasso(x=bootstrapped_data,K=k,beta=starting_beta[[pos_k,1]],lambda_rate_alpha=lambda_values_alpha,lambda_rate_beta=lambda_values_beta,verbose=FALSE)

                if(!is.na(res$best_loglik)) {

                    # compute cosine similarity of alpha for bootstrapped data
                    similarity <- NULL
                    for(i in 1:ncol(res$alpha)) {
                        curr_similarity <- ((sum(starting_values_bootstrap[[pos_k,1]][,i]*res$alpha[,i]))/sqrt(sum(starting_values_bootstrap[[pos_k,1]][,i]^2)*sum(res$alpha[,i]^2)))
                        similarity <- c(similarity,curr_similarity)
                    }
                    bootstrap_stability[[pos_k,"Similarity alpha bootstrapped data"]] <- c(unlist(bootstrap_stability[[pos_k,"Similarity alpha bootstrapped data"]]),mean(similarity))

                    # compute cosine similarity of beta for bootstrapped data
                    similarity <- NULL
                    for(i in 1:nrow(res$beta)) {
                        curr_similarity <- ((sum(starting_values_bootstrap[[pos_k,2]][i,]*res$beta[i,]))/sqrt(sum(starting_values_bootstrap[[pos_k,2]][i,]^2)*sum(res$beta[i,]^2)))
                        similarity <- c(similarity,curr_similarity)
                    }
                    bootstrap_stability[[pos_k,"Similarity beta bootstrapped data"]] <- c(unlist(bootstrap_stability[[pos_k,"Similarity beta bootstrapped data"]]),mean(similarity))

                    # compute RSS for bootstrapped data
                    predictions <- res$alpha%*%res$beta
                    mse <- NULL
                    for(i in 1:nrow(predictions)) {
                        for(j in 1:ncol(predictions)) {
                            mse <- c(mse,((bootstrapped_data[i,j]-predictions[i,j])^2))
                        }
                    }
                    mse <- sum(mse)
                    bootstrap_rss[[pos_k,"RSS bootstrapped data"]] <- c(unlist(bootstrap_rss[[pos_k,"RSS bootstrapped data"]]),mse)

                    # compute Explained Variance for bootstrapped data
                    evar <- (1 - (mse/sum(bootstrapped_data^2)))
                    bootstrap_evar[[pos_k,"Explained Variance bootstrapped data"]] <- c(unlist(bootstrap_evar[[pos_k,"Explained Variance bootstrapped data"]]),evar)

                }
                else {
                    bootstrap_stability[[pos_k,"Similarity alpha bootstrapped data"]] <- c(unlist(bootstrap_stability[[pos_k,"Similarity alpha bootstrapped data"]]),NA)
                    bootstrap_stability[[pos_k,"Similarity beta bootstrapped data"]] <- c(unlist(bootstrap_stability[[pos_k,"Similarity beta bootstrapped data"]]),NA)
                    bootstrap_rss[[pos_k,"RSS bootstrapped data"]] <- c(unlist(bootstrap_rss[[pos_k,"RSS bootstrapped data"]]),NA)
                    bootstrap_evar[[pos_k,"Explained Variance bootstrapped data"]] <- c(unlist(bootstrap_evar[[pos_k,"Explained Variance bootstrapped data"]]),NA)
                }

                # perform the analysis for randomized data
                res <- nmfLasso(x=randomized_data,K=k,beta=starting_beta[[pos_k,1]],lambda_rate_alpha=lambda_values_alpha,lambda_rate_beta=lambda_values_beta,verbose=FALSE)

                if(!is.na(res$best_loglik)) {

                    # compute cosine similarity of alpha for randomized data
                    similarity <- NULL
                    for(i in 1:ncol(res$alpha)) {
                        curr_similarity <- ((sum(starting_values_bootstrap[[pos_k,3]][,i]*res$alpha[,i]))/sqrt(sum(starting_values_bootstrap[[pos_k,3]][,i]^2)*sum(res$alpha[,i]^2)))
                        similarity <- c(similarity,curr_similarity)
                    }
                    bootstrap_stability[[pos_k,"Similarity alpha randomized data"]] <- c(unlist(bootstrap_stability[[pos_k,"Similarity alpha randomized data"]]),mean(similarity))

                    # compute cosine similarity of beta for randomized data
                    similarity <- NULL
                    for(i in 1:nrow(res$beta)) {
                        curr_similarity <- ((sum(starting_values_bootstrap[[pos_k,4]][i,]*res$beta[i,]))/sqrt(sum(starting_values_bootstrap[[pos_k,4]][i,]^2)*sum(res$beta[i,]^2)))
                        similarity <- c(similarity,curr_similarity)
                    }
                    bootstrap_stability[[pos_k,"Similarity beta randomized data"]] <- c(unlist(bootstrap_stability[[pos_k,"Similarity beta randomized data"]]),mean(similarity))

                    # compute RSS for randomized data
                    predictions <- res$alpha%*%res$beta
                    mse <- NULL
                    for(i in 1:nrow(predictions)) {
                        for(j in 1:ncol(predictions)) {
                            mse <- c(mse,((randomized_data[i,j]-predictions[i,j])^2))
                        }
                    }
                    mse <- sum(mse)
                    bootstrap_rss[[pos_k,"RSS randomized data"]] <- c(unlist(bootstrap_rss[[pos_k,"RSS randomized data"]]),mse)

                    # compute Explained Variance for randomized data
                    evar <- (1 - (mse/sum(randomized_data^2)))
                    bootstrap_evar[[pos_k,"Explained Variance randomized data"]] <- c(unlist(bootstrap_evar[[pos_k,"Explained Variance randomized data"]]),evar)

                }
                else {
                    bootstrap_stability[[pos_k,"Similarity alpha randomized data"]] <- c(unlist(bootstrap_stability[[pos_k,"Similarity alpha randomized data"]]),NA)
                    bootstrap_stability[[pos_k,"Similarity beta randomized data"]] <- c(unlist(bootstrap_stability[[pos_k,"Similarity beta randomized data"]]),NA)
                    bootstrap_rss[[pos_k,"RSS randomized data"]] <- c(unlist(bootstrap_rss[[pos_k,"RSS randomized data"]]),NA)
                    bootstrap_evar[[pos_k,"Explained Variance randomized data"]] <- c(unlist(bootstrap_evar[[pos_k,"Explained Variance randomized data"]]),NA)
                }
                
            }

            if(verbose) {
                cat("Progress",paste0(round((nboot/bootstrap_repetitions)*100,digits=3),"%..."),"\n")
            }

        }

        # save the results
        results <- list(stability=bootstrap_stability,RSS=bootstrap_rss,evar=bootstrap_evar)

    }
    else {

        # perform the inference
        res_clusterEvalQ <- clusterEvalQ(parallel,library("nnls"))
        res_clusterEvalQ <- clusterEvalQ(parallel,library("RhpcBLASctl"))
        res_clusterEvalQ <- clusterEvalQ(parallel,library("nnlasso"))
        clusterExport(parallel,varlist=c("x","K","starting_beta","background_signature","normalize_counts"),envir=environment())
        clusterExport(parallel,varlist=c("lambda_values_alpha","lambda_values_beta","iterations","max_iterations_lasso"),envir=environment())
        clusterExport(parallel,varlist=c("bootstrap_stability","bootstrap_rss","bootstrap_evar","bootstrap_repetitions","verbose"),envir=environment())
        clusterExport(parallel,c('nmfLasso','nmfLassoDecomposition','basis','nmf'),envir=environment())
        clusterSetRNGStream(parallel,iseed=round(runif(1)*100000))
        curr_results <- parLapply(parallel,1:bootstrap_repetitions,function(nboot) {

            if(verbose) {
                cat(paste0("Performing repetition ",nboot," out of ",bootstrap_repetitions,"..."),"\n")
            }

            # generate bootstrapped dataset from data
            bootstrapped_data <- x
            for(i in 1:nrow(x)) {
                num_mutations <- sum(x[i,])
                mut_distribution <- as.numeric(x[i,]/sum(x[i,]))
                sampling <- sample(1:96,size=num_mutations,replace=TRUE,prob=mut_distribution)
                bootstrapped_data[i,] <- 0
                for(j in sampling) {
                    bootstrapped_data[i,j] <- bootstrapped_data[i,j] + 1
                }
            }
            
            # generate randomized data from the bootstrapped data
            randomized_data <- bootstrapped_data
            for(i in 1:ncol(randomized_data)) {
                randomized_data[,i] <- randomized_data[sample(1:dim(randomized_data)[1],size=dim(randomized_data)[1],replace=FALSE),i]
            }
            for(j in 1:nrow(randomized_data)) {
                randomized_data[j,] <- randomized_data[j,sample(1:dim(randomized_data)[2],size=dim(randomized_data)[2],replace=FALSE)]
            }
    
            # perform inference on bootstrapped and randomized data
            pos_k <- 0
            for(k in K) {
                
                pos_k <- pos_k + 1

                # perform the analysis for bootstrapped data
                res <- nmfLasso(x=bootstrapped_data,K=k,beta=starting_beta[[pos_k,1]],lambda_rate_alpha=lambda_values_alpha,lambda_rate_beta=lambda_values_beta,verbose=FALSE)

                if(!is.na(res$best_loglik)) {

                    # compute cosine similarity of alpha for bootstrapped data
                    similarity <- NULL
                    for(i in 1:ncol(res$alpha)) {
                        curr_similarity <- ((sum(starting_values_bootstrap[[pos_k,1]][,i]*res$alpha[,i]))/sqrt(sum(starting_values_bootstrap[[pos_k,1]][,i]^2)*sum(res$alpha[,i]^2)))
                        similarity <- c(similarity,curr_similarity)
                    }
                    bootstrap_stability[[pos_k,"Similarity alpha bootstrapped data"]] <- c(unlist(bootstrap_stability[[pos_k,"Similarity alpha bootstrapped data"]]),mean(similarity))

                    # compute cosine similarity of beta for bootstrapped data
                    similarity <- NULL
                    for(i in 1:nrow(res$beta)) {
                        curr_similarity <- ((sum(starting_values_bootstrap[[pos_k,2]][i,]*res$beta[i,]))/sqrt(sum(starting_values_bootstrap[[pos_k,2]][i,]^2)*sum(res$beta[i,]^2)))
                        similarity <- c(similarity,curr_similarity)
                    }
                    bootstrap_stability[[pos_k,"Similarity beta bootstrapped data"]] <- c(unlist(bootstrap_stability[[pos_k,"Similarity beta bootstrapped data"]]),mean(similarity))

                    # compute RSS for bootstrapped data
                    predictions <- res$alpha%*%res$beta
                    mse <- NULL
                    for(i in 1:nrow(predictions)) {
                        for(j in 1:ncol(predictions)) {
                            mse <- c(mse,((bootstrapped_data[i,j]-predictions[i,j])^2))
                        }
                    }
                    mse <- sum(mse)
                    bootstrap_rss[[pos_k,"RSS bootstrapped data"]] <- c(unlist(bootstrap_rss[[pos_k,"RSS bootstrapped data"]]),mse)

                    # compute Explained Variance for bootstrapped data
                    evar <- (1 - (mse/sum(bootstrapped_data^2)))
                    bootstrap_evar[[pos_k,"Explained Variance bootstrapped data"]] <- c(unlist(bootstrap_evar[[pos_k,"Explained Variance bootstrapped data"]]),evar)

                }
                else {
                    bootstrap_stability[[pos_k,"Similarity alpha bootstrapped data"]] <- c(unlist(bootstrap_stability[[pos_k,"Similarity alpha bootstrapped data"]]),NA)
                    bootstrap_stability[[pos_k,"Similarity beta bootstrapped data"]] <- c(unlist(bootstrap_stability[[pos_k,"Similarity beta bootstrapped data"]]),NA)
                    bootstrap_rss[[pos_k,"RSS bootstrapped data"]] <- c(unlist(bootstrap_rss[[pos_k,"RSS bootstrapped data"]]),NA)
                    bootstrap_evar[[pos_k,"Explained Variance bootstrapped data"]] <- c(unlist(bootstrap_evar[[pos_k,"Explained Variance bootstrapped data"]]),NA)
                }

                # perform the analysis for randomized data
                res <- nmfLasso(x=randomized_data,K=k,beta=starting_beta[[pos_k,1]],lambda_rate_alpha=lambda_values_alpha,lambda_rate_beta=lambda_values_beta,verbose=FALSE)

                if(!is.na(res$best_loglik)) {

                    # compute cosine similarity of alpha for randomized data
                    similarity <- NULL
                    for(i in 1:ncol(res$alpha)) {
                        curr_similarity <- ((sum(starting_values_bootstrap[[pos_k,3]][,i]*res$alpha[,i]))/sqrt(sum(starting_values_bootstrap[[pos_k,3]][,i]^2)*sum(res$alpha[,i]^2)))
                        similarity <- c(similarity,curr_similarity)
                    }
                    bootstrap_stability[[pos_k,"Similarity alpha randomized data"]] <- c(unlist(bootstrap_stability[[pos_k,"Similarity alpha randomized data"]]),mean(similarity))

                    # compute cosine similarity of beta for randomized data
                    similarity <- NULL
                    for(i in 1:nrow(res$beta)) {
                        curr_similarity <- ((sum(starting_values_bootstrap[[pos_k,4]][i,]*res$beta[i,]))/sqrt(sum(starting_values_bootstrap[[pos_k,4]][i,]^2)*sum(res$beta[i,]^2)))
                        similarity <- c(similarity,curr_similarity)
                    }
                    bootstrap_stability[[pos_k,"Similarity beta randomized data"]] <- c(unlist(bootstrap_stability[[pos_k,"Similarity beta randomized data"]]),mean(similarity))

                    # compute RSS for randomized data
                    predictions <- res$alpha%*%res$beta
                    mse <- NULL
                    for(i in 1:nrow(predictions)) {
                        for(j in 1:ncol(predictions)) {
                            mse <- c(mse,((randomized_data[i,j]-predictions[i,j])^2))
                        }
                    }
                    mse <- sum(mse)
                    bootstrap_rss[[pos_k,"RSS randomized data"]] <- c(unlist(bootstrap_rss[[pos_k,"RSS randomized data"]]),mse)

                    # compute Explained Variance for randomized data
                    evar <- (1 - (mse/sum(randomized_data^2)))
                    bootstrap_evar[[pos_k,"Explained Variance randomized data"]] <- c(unlist(bootstrap_evar[[pos_k,"Explained Variance randomized data"]]),evar)

                }
                else {
                    bootstrap_stability[[pos_k,"Similarity alpha randomized data"]] <- c(unlist(bootstrap_stability[[pos_k,"Similarity alpha randomized data"]]),NA)
                    bootstrap_stability[[pos_k,"Similarity beta randomized data"]] <- c(unlist(bootstrap_stability[[pos_k,"Similarity beta randomized data"]]),NA)
                    bootstrap_rss[[pos_k,"RSS randomized data"]] <- c(unlist(bootstrap_rss[[pos_k,"RSS randomized data"]]),NA)
                    bootstrap_evar[[pos_k,"Explained Variance randomized data"]] <- c(unlist(bootstrap_evar[[pos_k,"Explained Variance randomized data"]]),NA)
                }
                
            }

            # save the results
            curr_results <- list(stability=bootstrap_stability,RSS=bootstrap_rss,evar=bootstrap_evar)

            return(curr_results)

        })

        # save the results
        results <- list(stability=bootstrap_stability,RSS=bootstrap_rss,evar=bootstrap_evar)
        for(par_res in 1:length(curr_results)) {
            curr_par_res <- curr_results[[par_res]]
            curr_par_res_stability <- curr_par_res$stability
            curr_par_res_rss <- curr_par_res$RSS
            curr_par_res_evar <- curr_par_res$evar
            for(a in 1:dim(curr_par_res_stability)[1]) {
                for(b in 1:dim(curr_par_res_stability)[2]) {
                    results$stability[[a,b]] <- c(unlist(results$stability[a,b]),unlist(curr_par_res_stability[a,b]))
                }
            }
            for(a in 1:dim(curr_par_res_rss)[1]) {
                for(b in 1:dim(curr_par_res_rss)[2]) {
                    results$RSS[[a,b]] <- c(unlist(results$RSS[a,b]),unlist(curr_par_res_rss[a,b]))
                    results$evar[[a,b]] <- c(unlist(results$evar[a,b]),unlist(curr_par_res_evar[a,b]))
                }
            }
        }

    }
    
    # close parallel
    if(close_parallel) {
        stopCluster(parallel)
    }
    
    return(results)
    
}

#' Perform a robust estimation of the starting betas for the nmfLasso method
#'
#' @examples
#' data(background)
#' data(patients)
#' res = startingBetaEstimation(x=patients[1:100,],
#'      K=3:5,
#'      background_signature=background,
#'      nmf_runs=1,
#'      seed=12345)
#'
#' @title startingBetaEstimation
#' @param x count matrix for a set of n patients and 96 trinucleotides.
#' @param K numeric value (minimum 2) indicating the number of signatures to be discovered.
#' @param background_signature background signature to be used. If not provided, a warning is thrown and an initial value for it is 
#' estimated by NMF. If beta is not NULL, this parameter is ignored.
#' @param normalize_counts if true, the input count matrix x is normalize such that the patients have the same number of mutation.
#' @param nmf_runs number of iteration (minimum 1) of NMF to be performed for a robust estimation of starting beta. If beta is not NULL, 
#' this parameter is ignored.
#' @param seed Seed for reproducibility.
#' @param verbose boolean; Shall I print all messages?
#' @return A list containing the starting beta values for each configuration of K.
#' @export startingBetaEstimation
#' @import NMF
#' @import nnls
#' @import RhpcBLASctl
#'
"startingBetaEstimation" <- function( x, K = 3:10, background_signature = NULL, normalize_counts = TRUE, nmf_runs = 10, seed = NULL, verbose = TRUE ) {
    
    # set the seed
    set.seed(seed)

    # check the input parameters
    if(min(K)<2) {
        if(verbose) {
            cat("The minimum value of K is 2...","\n")
        }
        K <- K[K>=2]
        if(length(K)==0) {
            K <- 2
            warning("No value of K greater than 2 is provided: setting K to 2...")
        }
    }
    if(nmf_runs<1) {
        if(verbose) {
            cat("The minimum value of nmf_runs is 1...","\n")
        }
        nmf_runs <- 1
    }
    x <- as.matrix(x)
    if(normalize_counts) {
        x <- (x/rowSums(x))*2500
    }

    # estimate the background signature
    if(is.null(background_signature)) {
        warning("No background signature has been specified...")
        background_signature_beta <- basis(nmf(t(x),rank=1,nrun=nmf_runs))
        background_signature_beta <- t(background_signature_beta)
        background_signature_beta <- background_signature_beta / rowSums(background_signature_beta)
        background_signature <- as.numeric(background_signature_beta[1,])
    }
    background_signature <- as.numeric(background_signature)
    background_signature <- background_signature / sum(background_signature)

    # estimate the contribution of the background to the observed counts
    background_signature_beta <- t(background_signature)
    background_signature_alpha <- matrix(0,nrow=dim(x)[1],ncol=1)
    for(i in 1:dim(x)[1]) {
        blas_set_num_threads(1)
        background_signature_alpha[i,] <- nnls(t(background_signature_beta),as.vector(x[i,]))$x
        blas_set_num_threads(blas_get_num_procs()) # reset to default
    }
    curr_x <- x - (background_signature_alpha %*% background_signature_beta)
    curr_x[curr_x<0] <- 0
    if(length(which(rowSums(t(curr_x))==0))>0) {
        invalid_cols <- as.numeric(which(rowSums(t(curr_x))==0))
        for(inv_cols in invalid_cols) {
            curr_x[sample(1:length(curr_x[,inv_cols]),size=1),inv_cols] <- 1e-05
        }
    }


    # perform a robust estimation of the starting beta for the nmfLasso method
    if(verbose) {
        cat("Performing a robust estimation of the starting betas for the nmfLasso method...","\n")
    }

    # structure to save the starting values of beta for each K
    starting_beta <- array(list(),c(length(K),1))
    rownames(starting_beta) <- paste0(as.character(K),"_signatures")
    colnames(starting_beta) <- "Value"

    # consider all the values of K
    pos_k <- 0
    for(k in K) {
    
        pos_k <- pos_k + 1
        beta <- NULL

        # compute the initial values of beta
        if(verbose) {
            cat("Computing the initial values of beta by standard NMF...","\n")
        }
        beta <- basis(nmf(t(curr_x),rank=(k-1),nrun=nmf_runs))
        beta <- t(beta)
        beta <- rbind(background_signature,beta)
        
        # normalize and save beta for the current value of K
        beta <- beta / rowSums(beta)
        rownames(beta) <- c("Background",paste0("S",1:(nrow(beta)-1)))
        colnames(beta) <- colnames(x)
        starting_beta[[pos_k,1]] <- beta

        if(verbose) {
            cat("Progress",paste0(round((pos_k/length(K))*100,digits=3),"%..."),"\n")
        }

    }

    return(starting_beta)
    
}

#' Estimate the range of lambda values for alpha to be considered in the signature inference. Note that too small values of lambda 
#' result in dense exposures, but too large values lead to bad fit of the counts.
#'
#' @examples
#' data(background)
#' data(patients)
#' res = lambdaRangeAlphaEvaluation(x=patients[1:100,],
#'      K=5,background_signature=background,
#'      nmf_runs=1,lambda_values=c(0.01,0.05),
#'      num_processes=NA,
#'      seed=12345)
#'
#' @title lambdaRangeAlphaEvaluation
#' @param x count matrix for a set of n patients and 96 trinucleotides.
#' @param K numeric value (minimum 2) indicating the number of signatures to be discovered.
#' @param beta starting beta for the estimation. If it is NULL, starting beta is estimated by NMF.
#' @param background_signature background signature to be used. If not provided, a warning is thrown and an initial value for it is 
#' estimated by NMF. If beta is not NULL, this parameter is ignored.
#' @param normalize_counts if true, the input count matrix x is normalize such that the patients have the same number of mutation.
#' @param nmf_runs number of iteration (minimum 1) of NMF to be performed for a robust estimation of starting beta. If beta is not NULL, 
#' this parameter is ignored.
#' @param lambda_values value of LASSO to be used for alpha between 0 and 1. This value should be greater than 0. 1 is the value of LASSO 
#' that would shrink all the signatures to 0 within one step. The higher lambda_values is, the sparser are the resulting exposures, 
#' but too large values may result in a reduced fit of the observed counts.
#' @param iterations Number of iterations to be performed. Each iteration corresponds to a first step where beta is fitted 
#' and a second step where alpha is fitted.
#' @param max_iterations_lasso Number of maximum iterations to be performed during the sparsification via Lasso.
#' @param num_processes Number of processes to be used during parallel execution. To execute in single process mode, 
#' this parameter needs to be set to either NA or NULL.
#' @param seed Seed for reproducibility.
#' @param verbose boolean; Shall I print all messages?
#' @param log_file log file where to print outputs when using parallel. If parallel execution is disabled, this parameter is ignored.
#' @return A list corresponding to results of the function nmfLasso for each value of lambda to be tested. This function allows 
#' to test a good range of lambda values for alpha to be considered. One should keep in mind that too small values generate dense solution, 
#' while too high ones leads to poor fit. This behavior is resampled in the values of loglik_progression, which should be increasing: 
#' too small values of lambda results in unstable log-likelihood through the iterations, while too large values make log-likelihood 
#' drop. 
#' @export lambdaRangeAlphaEvaluation
#' @import NMF
#' @import nnlasso
#' @import nnls
#' @import RhpcBLASctl
#' @import parallel
#'
"lambdaRangeAlphaEvaluation" <- function( x, K = 5, beta = NULL, background_signature = NULL, normalize_counts = TRUE, nmf_runs = 10, lambda_values = c(0.01, 0.05, 0.10, 0.20), iterations = 30, max_iterations_lasso = 10000, num_processes = Inf, seed = NULL, verbose = TRUE, log_file = "" ) {
    
    # set the seed
    set.seed(seed)

    # check the input parameters
    if(K<2) {
        if(verbose) {
            cat("The minimum value of K is 2...","\n")
        }
        K <- 2
    }
    if(nmf_runs<1) {
        if(verbose) {
            cat("The minimum value of nmf_runs is 1...","\n")
        }
        nmf_runs <- 1
    }
    x <- as.matrix(x)
    if(normalize_counts) {
        x_not_normalized <- x
        x <- (x/rowSums(x))*2500
    }
    lambda_values <- sort(unique(lambda_values))

    # compute the initial values of beta
    if(is.null(beta)) {

        if(verbose) {
            cat("Computing the initial values of beta by standard NMF...","\n")
        }
        
        # add a signature to beta (leading to K signatures in total) to explicitly model background noise
        if(is.null(background_signature)) {
            warning("No background signature has been specified...")
            background_signature_beta <- basis(nmf(t(x),rank=1,nrun=nmf_runs))
            background_signature_beta <- t(background_signature_beta)
            background_signature_beta <- background_signature_beta / rowSums(background_signature_beta)
            background_signature <- as.numeric(background_signature_beta[1,])
        }
        else {
            background_signature <- as.numeric(background_signature)
            background_signature <- background_signature / sum(background_signature)
            background_signature_beta <- t(background_signature)
        }

        background_signature_alpha <- matrix(0,nrow=dim(x)[1],ncol=1)
        for(i in 1:dim(x)[1]) {
            blas_set_num_threads(1)
            background_signature_alpha[i,] <- nnls(t(background_signature_beta),as.vector(x[i,]))$x
            blas_set_num_threads(blas_get_num_procs()) # reset to default
        }
        curr_x <- x - (background_signature_alpha %*% background_signature_beta)
        curr_x[curr_x<0] <- 0
        if(length(which(rowSums(t(curr_x))==0))>0) {
            invalid_cols <- as.numeric(which(rowSums(t(curr_x))==0))
            for(inv_cols in invalid_cols) {
                curr_x[sample(1:length(curr_x[,inv_cols]),size=1),inv_cols] <- 1e-05
            }
        }
        beta <- basis(nmf(t(curr_x),rank=(K-1),nrun=nmf_runs))
        beta <- t(beta)
        beta <- rbind(background_signature,beta)

    }
    beta <- beta / rowSums(beta)
    rownames(beta) <- c("Background",paste0("S",1:(nrow(beta)-1)))
    colnames(beta) <- colnames(x)
    
    # setting up parallel execution
    parallel <- NULL
    close_parallel <- FALSE
    if(is.null(parallel)&&length(lambda_values)>1) {
        if(is.na(num_processes) || is.null(num_processes)) {
            parallel <- NULL
        }
        else if(num_processes==Inf) {
            cores <- as.integer((detectCores()-1))
            if(cores < 2) {
                parallel <- NULL
            }
            else {
                num_processes <- min(num_processes,length(lambda_values))
                parallel <- makeCluster(num_processes,outfile=log_file)
                close_parallel <- TRUE
            }
        }
        else {
            num_processes <- min(num_processes,length(lambda_values))
            parallel <- makeCluster(num_processes,outfile=log_file)
            close_parallel <- TRUE
        }
        
        if(verbose && !is.null(parallel)) {
            cat("Executing",num_processes,"processes via parallel...","\n")
        }
    }
    
    # structure to save the estimated signatures
    lambda_results <- array(list(),c(length(K),length(lambda_values)))
    rownames(lambda_results) <- paste0(as.character(K),"_signatures")
    colnames(lambda_results) <- paste0(as.character(lambda_values),"_lambda")
    
    if(verbose) {
        cat("Performing estimation of lambda range for alpha...","\n")
    }
    
    # perform signature discovery for all the values of lambda
    if(is.null(parallel)) {
        set.seed(round(runif(1)*100000))
        cont <- 0
        for(l in lambda_values) {

            # perform the inference
            curr_results <- nmfLasso(x = x, 
                                     K = K, 
                                     beta = beta, 
                                     normalize_counts = normalize_counts, 
                                     lambda_rate_alpha = l, 
                                     lambda_rate_beta = 0.0, 
                                     iterations = iterations, 
                                     max_iterations_lasso = max_iterations_lasso, 
                                     seed = round(runif(1)*100000), 
                                     verbose = FALSE)
                                     
            # save the results for the current configuration
            cont <- cont + 1

            # rescale alpha to the original magnitude
            if(normalize_counts) {
                curr_results$starting_alpha <- curr_results$starting_alpha * (rowSums(x_not_normalized)/2500)
                curr_results$alpha <- curr_results$alpha * (rowSums(x_not_normalized)/2500)
            }

            lambda_results[[1,cont]] <- curr_results
                
            if(verbose) {
                cat("Progress",paste0(round((cont/(length(K)*length(lambda_values)))*100,digits=3),"%..."),"\n")
            }
            
        }
    }
    else {
        res_clusterEvalQ <- clusterEvalQ(parallel,library("nnls"))
        res_clusterEvalQ <- clusterEvalQ(parallel,library("RhpcBLASctl"))
        res_clusterEvalQ <- clusterEvalQ(parallel,library("nnlasso"))
        clusterExport(parallel,varlist=c("x","K","beta","normalize_counts","iterations","max_iterations_lasso"),envir=environment())
        clusterExport(parallel,c('nmfLasso','nmfLassoDecomposition','basis','nmf'),envir=environment())
        clusterSetRNGStream(parallel,iseed=round(runif(1)*100000))
        lambda_results_res <- parLapply(parallel,lambda_values,function(l) {

            # perform the inference
            curr_results <- nmfLasso(x = x, 
                                     K = K, 
                                     beta = beta, 
                                     normalize_counts = normalize_counts, 
                                     lambda_rate_alpha = l, 
                                     lambda_rate_beta = 0.0, 
                                     iterations = iterations, 
                                     max_iterations_lasso = max_iterations_lasso, 
                                     seed = round(runif(1)*100000), 
                                     verbose = FALSE)

            return(curr_results)

        })
        for(i in 1:ncol(lambda_results)) {
            # rescale alpha to the original magnitude
            if(normalize_counts) {
                lambda_results_res[[i]]$starting_alpha <- lambda_results_res[[i]]$starting_alpha * (rowSums(x_not_normalized)/2500)
                lambda_results_res[[i]]$alpha <- lambda_results_res[[i]]$alpha * (rowSums(x_not_normalized)/2500)
            }
            lambda_results[[1,i]] <- lambda_results_res[[i]]
        }
    }
    
    # close parallel
    if(close_parallel) {
        stopCluster(parallel)
    }
    
    return(lambda_results)
    
}

#' Estimate the range of lambda values for beta to be considered in the signature inference. Note that too small values of lambda 
#' result in dense signatures, but too large values lead to bad fit of the counts.
#'
#' @examples
#' data(background)
#' data(patients)
#' res = lambdaRangeBetaEvaluation(x=patients[1:100,],
#'      K=5,
#'      background_signature=background,
#'      nmf_runs=1,
#'      lambda_values=c(0.01,0.05),
#'      num_processes=NA,
#'      seed=12345)
#'
#' @title lambdaRangeBetaEvaluation
#' @param x count matrix for a set of n patients and 96 trinucleotides.
#' @param K numeric value (minimum 2) indicating the number of signatures to be discovered.
#' @param beta starting beta for the estimation. If it is NULL, starting beta is estimated by NMF.
#' @param background_signature background signature to be used. If not provided, a warning is thrown and an initial value for it is 
#' estimated by NMF. If beta is not NULL, this parameter is ignored.
#' @param normalize_counts if true, the input count matrix x is normalize such that the patients have the same number of mutation.
#' @param nmf_runs number of iteration (minimum 1) of NMF to be performed for a robust estimation of starting beta. If beta is not NULL, 
#' this parameter is ignored.
#' @param lambda_values value of LASSO to be used for beta between 0 and 1. This value should be greater than 0. 1 is the value of LASSO 
#' that would shrink all the signatures to 0 within one step. The higher lambda_values is, the sparser are the resulting signatures, 
#' but too large values may result in a reduced fit of the observed counts.
#' @param iterations Number of iterations to be performed. Each iteration corresponds to a first step where beta is fitted 
#' and a second step where alpha is fitted.
#' @param max_iterations_lasso Number of maximum iterations to be performed during the sparsification via Lasso.
#' @param num_processes Number of processes to be used during parallel execution. To execute in single process mode, 
#' this parameter needs to be set to either NA or NULL.
#' @param seed Seed for reproducibility.
#' @param verbose boolean; Shall I print all messages?
#' @param log_file log file where to print outputs when using parallel. If parallel execution is disabled, this parameter is ignored.
#' @return A list corresponding to results of the function nmfLasso for each value of lambda to be tested. This function allows 
#' to test a good range of lambda values for beta to be considered. One should keep in mind that too small values generate dense solution, 
#' while too high ones leads to poor fit. This behavior is resampled in the values of loglik_progression, which should be increasing: 
#' too small values of lambda results in unstable log-likelihood through the iterations, while too large values make log-likelihood 
#' drop. 
#' @export lambdaRangeBetaEvaluation
#' @import NMF
#' @import nnlasso
#' @import nnls
#' @import RhpcBLASctl
#' @import parallel
#'
"lambdaRangeBetaEvaluation" <- function( x, K = 5, beta = NULL, background_signature = NULL, normalize_counts = TRUE, nmf_runs = 10, lambda_values = c(0.01, 0.05, 0.10, 0.20), iterations = 30, max_iterations_lasso = 10000, num_processes = Inf, seed = NULL, verbose = TRUE, log_file = "" ) {
    
    # set the seed
    set.seed(seed)

    # check the input parameters
    if(K<2) {
        if(verbose) {
            cat("The minimum value of K is 2...","\n")
        }
        K <- 2
    }
    if(nmf_runs<1) {
        if(verbose) {
            cat("The minimum value of nmf_runs is 1...","\n")
        }
        nmf_runs <- 1
    }
    x <- as.matrix(x)
    if(normalize_counts) {
        x_not_normalized <- x
        x <- (x/rowSums(x))*2500
    }
    lambda_values <- sort(unique(lambda_values))

    # compute the initial values of beta
    if(is.null(beta)) {

        if(verbose) {
            cat("Computing the initial values of beta by standard NMF...","\n")
        }
        
        # add a signature to beta (leading to K signatures in total) to explicitly model background noise
        if(is.null(background_signature)) {
            warning("No background signature has been specified...")
            background_signature_beta <- basis(nmf(t(x),rank=1,nrun=nmf_runs))
            background_signature_beta <- t(background_signature_beta)
            background_signature_beta <- background_signature_beta / rowSums(background_signature_beta)
            background_signature <- as.numeric(background_signature_beta[1,])
        }
        else {
            background_signature <- as.numeric(background_signature)
            background_signature <- background_signature / sum(background_signature)
            background_signature_beta <- t(background_signature)
        }

        background_signature_alpha <- matrix(0,nrow=dim(x)[1],ncol=1)
        for(i in 1:dim(x)[1]) {
            blas_set_num_threads(1)
            background_signature_alpha[i,] <- nnls(t(background_signature_beta),as.vector(x[i,]))$x
            blas_set_num_threads(blas_get_num_procs()) # reset to default
        }
        curr_x <- x - (background_signature_alpha %*% background_signature_beta)
        curr_x[curr_x<0] <- 0
        if(length(which(rowSums(t(curr_x))==0))>0) {
            invalid_cols <- as.numeric(which(rowSums(t(curr_x))==0))
            for(inv_cols in invalid_cols) {
                curr_x[sample(1:length(curr_x[,inv_cols]),size=1),inv_cols] <- 1e-05
            }
        }
        beta <- basis(nmf(t(curr_x),rank=(K-1),nrun=nmf_runs))
        beta <- t(beta)
        beta <- rbind(background_signature,beta)

    }
    beta <- beta / rowSums(beta)
    rownames(beta) <- c("Background",paste0("S",1:(nrow(beta)-1)))
    colnames(beta) <- colnames(x)
    
    # setting up parallel execution
    parallel <- NULL
    close_parallel <- FALSE
    if(is.null(parallel)&&length(lambda_values)>1) {
        if(is.na(num_processes) || is.null(num_processes)) {
            parallel <- NULL
        }
        else if(num_processes==Inf) {
            cores <- as.integer((detectCores()-1))
            if(cores < 2) {
                parallel <- NULL
            }
            else {
                num_processes <- min(num_processes,length(lambda_values))
                parallel <- makeCluster(num_processes,outfile=log_file)
                close_parallel <- TRUE
            }
        }
        else {
            num_processes <- min(num_processes,length(lambda_values))
            parallel <- makeCluster(num_processes,outfile=log_file)
            close_parallel <- TRUE
        }
        
        if(verbose && !is.null(parallel)) {
            cat("Executing",num_processes,"processes via parallel...","\n")
        }
    }
    
    # structure to save the estimated signatures
    lambda_results <- array(list(),c(length(K),length(lambda_values)))
    rownames(lambda_results) <- paste0(as.character(K),"_signatures")
    colnames(lambda_results) <- paste0(as.character(lambda_values),"_lambda")
    
    if(verbose) {
        cat("Performing estimation of lambda range for beta...","\n")
    }
    
    # perform signature discovery for all the values of lambda
    if(is.null(parallel)) {
        set.seed(round(runif(1)*100000))
        cont <- 0
        for(l in lambda_values) {

            # perform the inference
            curr_results <- nmfLasso(x = x, 
                                     K = K, 
                                     beta = beta, 
                                     normalize_counts = normalize_counts, 
                                     lambda_rate_alpha = 0.0, 
                                     lambda_rate_beta = l, 
                                     iterations = iterations, 
                                     max_iterations_lasso = max_iterations_lasso, 
                                     seed = round(runif(1)*100000), 
                                     verbose = FALSE)
                                     
            # save the results for the current configuration
            cont <- cont + 1

            # rescale alpha to the original magnitude
            if(normalize_counts) {
                curr_results$starting_alpha <- curr_results$starting_alpha * (rowSums(x_not_normalized)/2500)
                curr_results$alpha <- curr_results$alpha * (rowSums(x_not_normalized)/2500)
            }

            lambda_results[[1,cont]] <- curr_results
                
            if(verbose) {
                cat("Progress",paste0(round((cont/(length(K)*length(lambda_values)))*100,digits=3),"%..."),"\n")
            }
            
        }
    }
    else {
        res_clusterEvalQ <- clusterEvalQ(parallel,library("nnls"))
        res_clusterEvalQ <- clusterEvalQ(parallel,library("RhpcBLASctl"))
        res_clusterEvalQ <- clusterEvalQ(parallel,library("nnlasso"))
        clusterExport(parallel,varlist=c("x","K","beta","normalize_counts","iterations","max_iterations_lasso"),envir=environment())
        clusterExport(parallel,c('nmfLasso','nmfLassoDecomposition','basis','nmf'),envir=environment())
        clusterSetRNGStream(parallel,iseed=round(runif(1)*100000))
        lambda_results_res <- parLapply(parallel,lambda_values,function(l) {

            # perform the inference
            curr_results <- nmfLasso(x = x, 
                                     K = K, 
                                     beta = beta, 
                                     normalize_counts = normalize_counts, 
                                     lambda_rate_alpha = 0.0, 
                                     lambda_rate_beta = l, 
                                     iterations = iterations, 
                                     max_iterations_lasso = max_iterations_lasso, 
                                     seed = round(runif(1)*100000), 
                                     verbose = FALSE)

            return(curr_results)

        })
        for(i in 1:ncol(lambda_results)) {
            # rescale alpha to the original magnitude
            if(normalize_counts) {
                lambda_results_res[[i]]$starting_alpha <- lambda_results_res[[i]]$starting_alpha * (rowSums(x_not_normalized)/2500)
                lambda_results_res[[i]]$alpha <- lambda_results_res[[i]]$alpha * (rowSums(x_not_normalized)/2500)
            }
            lambda_results[[1,i]] <- lambda_results_res[[i]]
        }
    }
    
    # close parallel
    if(close_parallel) {
        stopCluster(parallel)
    }
    
    return(lambda_results)
    
}

#' Perform the discovery of K somatic mutational signatures given a set of observed counts x.
#'
#' @examples
#' data(patients)
#' data(starting_betas_example)
#' beta = starting_betas_example[["5_signatures","Value"]]
#' res = nmfLasso(x=patients[1:100,],
#'      K=5,
#'      beta=beta,
#'      lambda_rate_alpha=0.05,
#'      lambda_rate_beta=0.05,
#'      iterations=5,
#'      seed=12345)
#'
#' @title nmfLasso
#' @param x count matrix for a set of n patients and 96 trinucleotides.
#' @param K numeric value (minimum 2) indicating the number of signatures to be discovered.
#' @param beta starting beta for the estimation. If it is NULL, starting beta is estimated by NMF.
#' @param background_signature background signature to be used. If not provided, a warning is thrown and an initial value for it is 
#' estimated by NMF. If beta is not NULL, this parameter is ignored.
#' @param normalize_counts if true, the input count matrix x is normalize such that the patients have the same number of mutation.
#' @param nmf_runs number of iteration (minimum 1) of NMF to be performed for a robust estimation of starting beta. If beta is not NULL, 
#' this parameter is ignored.
#' @param lambda_rate_alpha value of LASSO to be used for alpha between 0 and 1. This value should be greater than 0. 1 is the value of LASSO 
#' that would shrink all the exposure values to 0 within one step. The higher lambda_rate_alpha is, the sparser are the resulting exposure values, 
#' but too large values may result in a reduced fit of the observed counts.
#' @param lambda_rate_beta value of LASSO to be used for beta between 0 and 1. This value should be greater than 0. 1 is the value of LASSO 
#' that would shrink all the signatures to 0 within one step. The higher lambda_rate_beta is, the sparser are the resulting signatures, 
#' but too large values may result in a reduced fit of the observed counts.
#' @param iterations Number of iterations to be performed. Each iteration corresponds to a first step where beta is fitted 
#' and a second step where alpha is fitted.
#' @param max_iterations_lasso Number of maximum iterations to be performed during the sparsification via Lasso.
#' @param seed Seed for reproducibility.
#' @param verbose boolean; Shall I print all messages?
#' @return A list with the discovered signatures. It includes 6 elements: 
#'              alpha: matrix of the discovered exposure values
#'              beta: matrix of the discovered signatures
#'              starting_alpha: initial alpha on which the method has been applied
#'              starting_beta: initial beta on which the method has been applied
#'              loglik_progression: log-likelihood values during the iterations. This values should be increasing, if not the selected value of lambda is too high
#'              best_loglik: log-likelihood of the best signatures configuration
#' @export nmfLasso
#' @import NMF
#' @import nnls
#' @import RhpcBLASctl
#' @import nnlasso
#'
"nmfLasso" <- function( x, K, beta = NULL, background_signature = NULL, normalize_counts = TRUE, nmf_runs = 10, lambda_rate_alpha = 0.05, lambda_rate_beta = 0.05, iterations = 30, max_iterations_lasso = 10000, seed = NULL, verbose = TRUE ) {
    
    # set the seed
    set.seed(seed)

    # check the input parameters
    if(K<2) {
        if(verbose) {
            cat("The minimum value of K is 2...","\n")
        }
        K <- 2
    }
    if(nmf_runs<1) {
        if(verbose) {
            cat("The minimum value of nmf_runs is 1...","\n")
        }
        nmf_runs <- 1
    }
    if(lambda_rate_alpha<0) {
        if(verbose) {
            cat("The minimum value of lambda_rate_alpha is 0...","\n")
        }
        lambda_rate_alpha <- 0
    }
    if(lambda_rate_alpha>1) {
        if(verbose) {
            cat("The maximum value of lambda_rate_alpha is 1...","\n")
        }
        lambda_rate_alpha <- 1
    }
    if(lambda_rate_beta<0) {
        if(verbose) {
            cat("The minimum value of lambda_rate_beta is 0...","\n")
        }
        lambda_rate_beta <- 0
    }
    if(lambda_rate_beta>1) {
        if(verbose) {
            cat("The maximum value of lambda_rate_beta is 1...","\n")
        }
        lambda_rate_beta <- 1
    }
    x <- as.matrix(x)
    if(normalize_counts) {
        x_not_normalized <- x
        x <- (x/rowSums(x))*2500
    }

    # compute the initial values of beta
    if(is.null(beta)) {

        if(verbose) {
            cat("Computing the initial values of beta by standard NMF...","\n")
        }
        
        # add a signature to beta (leading to K signatures in total) to explicitly model background noise
        if(is.null(background_signature)) {
            warning("No background signature has been specified...")
            background_signature_beta <- basis(nmf(t(x),rank=1,nrun=nmf_runs))
            background_signature_beta <- t(background_signature_beta)
            background_signature_beta <- background_signature_beta / rowSums(background_signature_beta)
            background_signature <- as.numeric(background_signature_beta[1,])
        }
        else {
            background_signature <- as.numeric(background_signature)
            background_signature <- background_signature / sum(background_signature)
            background_signature_beta <- t(background_signature)
        }

        background_signature_alpha <- matrix(0,nrow=dim(x)[1],ncol=1)
        for(i in 1:dim(x)[1]) {
            blas_set_num_threads(1)
            background_signature_alpha[i,] <- nnls(t(background_signature_beta),as.vector(x[i,]))$x
            blas_set_num_threads(blas_get_num_procs()) # reset to default
        }
        curr_x <- x - (background_signature_alpha %*% background_signature_beta)
        curr_x[curr_x<0] <- 0
        if(length(which(rowSums(t(curr_x))==0))>0) {
            invalid_cols <- as.numeric(which(rowSums(t(curr_x))==0))
            for(inv_cols in invalid_cols) {
                curr_x[sample(1:length(curr_x[,inv_cols]),size=1),inv_cols] <- 1e-05
            }
        }
        beta <- basis(nmf(t(curr_x),rank=(K-1),nrun=nmf_runs))
        beta <- t(beta)
        beta <- rbind(background_signature,beta)

    }
    beta <- beta / rowSums(beta)
    rownames(beta) <- c("Background",paste0("S",1:(nrow(beta)-1)))
    colnames(beta) <- colnames(x)
    
    if(verbose) {
        cat("Performing the discovery of the signatures by NMF with Lasso...","\n")
    }
    
    # perform the discovery of the signatures
    results = tryCatch({
        results = nmfLassoDecomposition(x,beta,lambda_rate_alpha,lambda_rate_beta,iterations,max_iterations_lasso,round(runif(1)*100000),verbose)
        # rescale alpha to the original magnitude
        if(normalize_counts) {
            results$starting_alpha <- results$starting_alpha * (rowSums(x_not_normalized)/2500)
            results$alpha <- results$alpha * (rowSums(x_not_normalized)/2500)
        }
        return(results)
    }, error = function(e) {
        warning("Lasso did not converge, you should try a lower value of lambda! Current settings: K = ",K,", lambda_rate_alpha = ",lambda_rate_alpha,", lambda_rate_beta = ",lambda_rate_beta,"...")
        return(list(alpha=NA,beta=NA,starting_alpha=NA,starting_beta=NA,loglik_progression=rep(NA,iterations),best_loglik=NA))
    })
    
    return(results)
    
}

# perform de novo discovery of somatic mutational signatures using NMF with Lasso to ensure sparsity
"nmfLassoDecomposition" <- function( x, starting_beta, lambda_rate_alpha = 0.05, lambda_rate_beta = 0.05, iterations = 30, max_iterations_lasso = 10000, seed = NULL, verbose = TRUE ) {
    
    # set the seed
    set.seed(seed)
    
    # n is the number of observations in x, i.e., the patients
    n <- dim(x)[1]
    
    # J is the number of trinucleotides, i.e., 96 categories
    J <- dim(x)[2]
    
    # K is the number of signatures to be called
    K <- dim(starting_beta)[1]
    
    # initialize beta
    starting_beta <- starting_beta / rowSums(starting_beta)
    rownames(starting_beta) <- c("Background",paste0("S",1:(nrow(starting_beta)-1)))
    colnames(starting_beta) <- colnames(x)
    beta <- starting_beta
    
    # initialize alpha
    alpha <- matrix(0,nrow=n,ncol=K)
    rownames(alpha) <- 1:nrow(alpha)
    colnames(alpha) <- rownames(beta)
    for(i in 1:n) {
        blas_set_num_threads(1)
        alpha[i,] <- nnls(t(beta),as.vector(x[i,]))$x
        blas_set_num_threads(blas_get_num_procs()) # reset to default
    }
    starting_alpha <- alpha

    # rescale the two matrices of a factor of 50 for numerical reasons
    beta <- beta * 50
    for(i in 1:n) {
        blas_set_num_threads(1)
        alpha[i,] <- nnls(t(beta),as.vector(x[i,]))$x
        blas_set_num_threads(blas_get_num_procs()) # reset to default
    }
    
    # structure where to save the log-likelihood at each iteration
    loglik <- rep(NA,iterations)

    # structure where to save the lambda values for alpha and beta
    lambda_values_alpha <- rep(NA,n) # sparsity over the patients (reduce the number of signatures per patient)
    lambda_values_beta <- rep(NA,J) # sparsity over the signatures (produce sparse signatures)

    if(verbose) {
        cat("Performing a total of",iterations,"iterations...","\n")
    }

    # start the learning procedure
    best_loglik <- NA
    best_alpha <- NA
    best_beta <- NA
    for(i in 1:iterations) {
        
        # initialize the value of the log-likelihood for the current iteration
        loglik[i] <- 0

        # configuration 1 (turn off sparsity): repeat a 2 step algorithm iteratively, where 
        # first beta is estimated by Non-Negative Linear Least Squares and, then, alpha is also estimated by Non-Negative Linear Least Squares 
        if(lambda_rate_alpha==0&&lambda_rate_beta==0) {

            # update beta by Non-Negative Linear Least Squares
            for(k in 1:J) {
                 # the first signature represents the background model, thus it is not changed
                blas_set_num_threads(1)
                beta[2:K,k] <- nnls(alpha[,2:K,drop=FALSE],as.vector(x[,k]-(alpha[,1]*beta[1,k])))$x
                blas_set_num_threads(blas_get_num_procs()) # reset to default
            }

            # update alpha by Non-Negative Linear Least Squares
            for(j in 1:n) {
                blas_set_num_threads(1)
                alpha[j,] <- nnls(t(beta),as.vector(x[j,]))$x
                blas_set_num_threads(blas_get_num_procs()) # reset to default
            }

            # update the log-likelihood for the current iteration
            loglik[i] <- -sum((x - alpha %*% beta)^2)

        }

        # configuration 2 (sparsity only on beta): repeat a 2 step algorithm iteratively, where 
        # first beta is estimated by Non-Negative Lasso and, then, alpha is estimated by Non-Negative Linear Least Squares 
        if(lambda_rate_alpha==0&&lambda_rate_beta>0) {
            
            # update beta by Non-Negative Lasso
            for(k in 1:J) {
                
                # compute independently for each trinucleotide the difference between the observed counts, i.e., x, 
                # and the counts predicted by considering only the first signature (i.e., which represents the background model) 
                target <- x[,k] - (alpha[,1] * beta[1,k])
                
                # estimate beta for the remaining signatues by Non-Negative Lasso to ensure sparsity
                if(is.na(lambda_values_beta[k])) {
                    max_lambda_value <- max(abs(t(alpha[,2:K,drop=FALSE]) %*% target))
                    lambda_values_beta[k] <- max_lambda_value * lambda_rate_beta
                }
                beta[2:K,k] <- as.vector(nnlasso(x = alpha[,2:K,drop=FALSE], 
                                                y = target, 
                                                family = "normal", 
                                                lambda = lambda_values_beta[k], 
                                                intercept = FALSE, 
                                                normalize = FALSE, 
                                                tol = 1e-05, 
                                                maxiter = max_iterations_lasso, 
                                                path = FALSE)$coef[2,])
                
            }

            # update alpha by Non-Negative Linear Least Squares
            for(j in 1:n) {
                blas_set_num_threads(1)
                alpha[j,] <- nnls(t(beta),as.vector(x[j,]))$x
                blas_set_num_threads(blas_get_num_procs()) # reset to default
            }
            
            # update the log-likelihood for the current iteration
            for(k in 1:J) {
                curr_loglik <- -sum((x[,k] - alpha %*% beta[,k])^2) - (lambda_values_beta[k] * sum(beta[2:K,k]))
                loglik[i] <- loglik[i] + curr_loglik
            }

        }

        # configuration 3 (sparsity only on alpha): repeat a 2 step algorithm iteratively, where 
        # first beta is estimated by Non-Negative Linear Least Squares and, then, alpha is estimated by Non-Negative Lasso 
        if(lambda_rate_alpha>0&&lambda_rate_beta==0) {

            # update beta by Non-Negative Linear Least Squares
            for(k in 1:J) {
                # the first signature represents the background model, thus it is not changed
                blas_set_num_threads(1)
                beta[2:K,k] <- nnls(alpha[,2:K,drop=FALSE],as.vector(x[,k]-(alpha[,1]*beta[1,k])))$x
                blas_set_num_threads(blas_get_num_procs()) # reset to default
            }

            # update alpha by Non-Negative Lasso
            for(j in 1:n) {

                # compute independently for each patient the counts to be approximted
                target <- x[j,]
                
                # estimate alpha by Non-Negative Lasso to ensure sparsity
                if(is.na(lambda_values_alpha[j])) {
                    max_lambda_value <- max(abs(beta %*% target))
                    lambda_values_alpha[j] <- max_lambda_value * lambda_rate_alpha
                }
                alpha[j,] <- as.vector(nnlasso(x = t(beta), 
                                                y = target, 
                                                family = "normal", 
                                                lambda = lambda_values_alpha[j], 
                                                intercept = FALSE, 
                                                normalize = FALSE, 
                                                tol = 1e-05, 
                                                maxiter = max_iterations_lasso, 
                                                path = FALSE)$coef[2,])
                
            }
            
            # update the log-likelihood for the current iteration
            for(j in 1:n) {
                curr_loglik <- -sum((x[j,] - alpha[j,] %*% beta)^2) - (lambda_values_alpha[j] * sum(alpha[j,]))
                loglik[i] <- loglik[i] + curr_loglik
            }

        }

        # configuration 4 (sparsity on both alpha and beta): repeat a 2 step algorithm iteratively, where 
        # first beta is estimated by Non-Negative Lasso and, then, also alpha is estimated by Non-Negative Lasso 
        if(lambda_rate_alpha>0&&lambda_rate_beta>0) {
            
            # update beta by Non-Negative Lasso
            for(k in 1:J) {
                
                # compute independently for each trinucleotide the difference between the observed counts, i.e., x, 
                # and the counts predicted by considering only the first signature (i.e., which represents the background model) 
                target <- x[,k] - (alpha[,1] * beta[1,k])
                
                # estimate beta for the remaining signatues by Non-Negative Lasso to ensure sparsity
                if(is.na(lambda_values_beta[k])) {
                    max_lambda_value <- max(abs(t(alpha[,2:K,drop=FALSE]) %*% target))
                    lambda_values_beta[k] <- max_lambda_value * lambda_rate_beta
                }
                beta[2:K,k] <- as.vector(nnlasso(x = alpha[,2:K,drop=FALSE], 
                                                y = target, 
                                                family = "normal", 
                                                lambda = lambda_values_beta[k], 
                                                intercept = FALSE, 
                                                normalize = FALSE, 
                                                tol = 1e-05, 
                                                maxiter = max_iterations_lasso, 
                                                path = FALSE)$coef[2,])
                
            }
            
            # update alpha by Non-Negative Lasso
            for(j in 1:n) {

                # compute independently for each patient the counts to be approximted
                target <- x[j,]
                
                # estimate alpha by Non-Negative Lasso to ensure sparsity
                if(is.na(lambda_values_alpha[j])) {
                    max_lambda_value <- max(abs(beta %*% target))
                    lambda_values_alpha[j] <- max_lambda_value * lambda_rate_alpha
                }
                alpha[j,] <- as.vector(nnlasso(x = t(beta), 
                                                y = target, 
                                                family = "normal", 
                                                lambda = lambda_values_alpha[j], 
                                                intercept = FALSE, 
                                                normalize = FALSE, 
                                                tol = 1e-05, 
                                                maxiter = max_iterations_lasso, 
                                                path = FALSE)$coef[2,])
                
            }
            
            # update the log-likelihood for the current iteration
            for(k in 1:J) {
                for(j in 1:n) {
                    curr_loglik <- -((x[j,k] - alpha[j,] %*% beta[,k])^2) - (lambda_values_alpha[j] * sum(alpha[j,])) - (lambda_values_beta[k] * sum(beta[2:K,k]))
                    loglik[i] <- loglik[i] + curr_loglik
                }
            }

        }
        
        # save the results at maximum log-likelihood
        if(is.na(best_loglik)||loglik[i]>best_loglik) {
            best_alpha <- alpha
            best_beta <- beta
            best_loglik <- loglik[i]
        }
        
        if(verbose) {
            cat("Progress",paste0((i/iterations)*100,"%..."),"\n")
        }

    }
    alpha <- best_alpha
    beta <- best_beta
    
    # check if the likelihood is increasing
    if(length(loglik)>1) {
        cont <- 1
        for(i in 2:length(loglik)) {
            if(loglik[i]>loglik[(i-1)]) {
                cont <- cont + 1
            }
        }
        if(cont/iterations<0.5) {
            warning("The likelihood is not increasing, you should try a lower value of lambda! Current settings: K = ",K,", lambda_rate_alpha = ",lambda_rate_alpha,", lambda_rate_beta = ",lambda_rate_beta,"...")
        }
    }
    
    # normalize the rate of the signatures to sum to 1
    beta <- beta / rowSums(beta)
        
    # final computation of alpha using the normalized version of beta
    if(lambda_rate_alpha>0) {

        # update alpha by Non-Negative Lasso
        for(j in 1:n) {

            # compute independently for each patient the counts to be approximted
            target <- x[j,]
            
            # estimate alpha by Non-Negative Lasso to ensure sparsity
            alpha[j,] <- as.vector(nnlasso(x = t(beta), 
                                            y = target, 
                                            family = "normal", 
                                            lambda = (max(abs(beta %*% target)) * lambda_rate_alpha), 
                                            intercept = FALSE, 
                                            normalize = FALSE, 
                                            tol = 1e-05, 
                                            maxiter = max_iterations_lasso, 
                                            path = FALSE)$coef[2,])
            
        }
        
    }
    else {

        # update alpha by Non-Negative Linear Least Squares
        for(j in 1:n) {
            blas_set_num_threads(1)
            alpha[j,] <- nnls(t(beta),as.vector(x[j,]))$x
            blas_set_num_threads(blas_get_num_procs()) # reset to default
        }

    }
    
    # save the results
    results <- list(alpha=alpha,beta=beta,starting_alpha=starting_alpha,starting_beta=starting_beta,loglik_progression=loglik,best_loglik=best_loglik)
    
    return(results)
    
}

# Perform the discovery by cross validation of K (unknown) somatic mutational signatures given a set of observations x. The estimation can slow down because of 
# memory usage, when I high number of cross validation repetitions is asked and when the grid search is performed for a lot of configurations. In this case, 
# we advice to split the computation into multiple smaller sets. 
# @title nmf.LassoCV
# @param x count matrix.
# @param K a range of numeric value (each of them greater than 1) indicating the number of signatures to be discovered.
# @param starting_beta a list of starting beta value for each configuration of K. If it is NULL, starting betas are estimated by 
# NMF.
# @param background_signature background signature to be used. If not provided, a warning is thrown.
# @param nmf_runs number of iteration of NMF to be performed for a robust estimation of starting beta. If beta is not NULL, 
# this parameter is ignored.
# @param lambda_values range of values of LASSO to be used between 0 and 1. This value should be greater than 0. 1 is the value of 
# LASSO 
# that would shrink all the signatures to 0 within one step. The higher lambda_rate is, the sparser are the resulting signatures, 
# but too large values result in a poor fit of the counts.
# @param cross_validation_entries Percentage of cells in the count matrix to be replaced by 0s.
# @param cross_validation_iterations For each configuration, the first time the signatures are discovered form a matrix with a 
# ercentage of values replaced by 0s. This may result in a poor. This parameter is the number of restarts to be performed to 
# improve this estimate.
# @param cross_validation_repetitions Number of time cross-validation should be repeated. Higher values result in better estimate, but are computationally expensive.
# @param iterations Number of iterations to be performed. Each iteration correspond to a first step where the counts are fitted 
# and a second step where sparsity is enhanced.
# @param max_iterations_lasso Number of maximum iterations to be performed during the sparsification.
# @param num_processes Number of processes to be used during parallel execution. If executing in single process mode, 
# this is ignored.
# @param seed Seed for reproducibility.
# @param verbose boolean; Shall I print all messages?
# @return A list corresponding with 3 elements: grid_search, starting_beta and mean_squared_error. Here, grid_search provides all the results of the executions within 
# the grid search; starting_beta is the set of initial values of beta used for each configuration and mean_squared_error is the mean squared error between the 
# observed counts and the predicted ones for each configuration.
# @export nmf.LassoCV
# @import NMF
# @import nnlasso
# @import nnls
# @import parallel
#
"nmf.LassoCV" <- function( x, K = 3:10, starting_beta = NULL, background_signature = NULL, nmf_runs = 10, lambda_values = c(0.10, 0.20, 0.30), cross_validation_entries = 0.05, cross_validation_iterations = 5, cross_validation_repetitions = 10, iterations = 20, max_iterations_lasso = 10000, num_processes = Inf, seed = NULL, verbose = TRUE ) {

    .Deprecated("nmfLassoCV")

#    # set the seed
#    set.seed(seed)
#
#    nmf_method <- "nmf_standard"
#    
#    # perform a grid search to estimate the best values of K and lambda
#    if(verbose) {
#        cat("Performing a grid search to estimate the best values of K and lambda with a total of",cross_validation_repetitions,"cross validation repetitions...","\n")
#    }
#    
#    # setting up parallel execution
#    if(is.na(num_processes) || is.null(num_processes)) {
#        parallel <- NULL
#    }
#    else if(num_processes==Inf) {
#        cores <- as.integer((detectCores()-1))
#        if(cores < 2) {
#            parallel <- NULL
#        }
#        else {
#            num_processes <- cores
#            parallel <- makeCluster(num_processes,outfile="")
#            clusterSetRNGStream(parallel,iseed=round(runif(1)*100000))
#        }
#    }
#    else {
#        parallel <- makeCluster(num_processes,outfile="")
#        clusterSetRNGStream(parallel,iseed=round(runif(1)*100000))
#    }
#    
#    if(verbose && !is.null(parallel)) {
#        cat("Executing",num_processes,"processes via parallel...","\n")
#    }
#
#    if(verbose) {
#        cat("Starting cross validation with a total of",cross_validation_repetitions,"repetitions...","\n")
#    }
#
#    # now starting cross validations
#    results <- list()
#    for(cv_repetitions in 1:cross_validation_repetitions) {
#
#        if(verbose) {
#            cat(paste0("Performing repetition ",cv_repetitions," out of ",cross_validation_repetitions,"..."),"\n")
#        }
#    
#        # structure to save the results of the grid search for all the cross_validation_iterations
#        grid_search_iterations <- list()
#
#        # repeat the estimation for a number of cross_validation_iterations
#        for(cv_iteration in 1:cross_validation_iterations) {
#                    
#            if(verbose) {
#                cat(paste0("Performing cross validation iteration ",cv_iteration," out of ",cross_validation_iterations,"..."),"\n")
#            }
#
#            # set a percentage of cross_validation_entries entries to 0 in order to perform cross validation
#            if(cv_iteration==1) {
#                x_cv <- x
#                valid_entries <- which(x_cv>0,arr.ind=TRUE)
#                cv_entries <- valid_entries[sample(1:nrow(valid_entries),size=round(nrow(valid_entries)*cross_validation_entries),replace=FALSE),]
#                x_cv[cv_entries] <- 0
#            }
#            
#            # structure to save the results of the grid search
#            grid_search <- array(list(),c(length(K),length(lambda_values)))
#            rownames(grid_search) <- paste0(as.character(K),"_signatures")
#            colnames(grid_search) <- paste0(as.character(lambda_values),"_lambda")
#            
#            # structure to save the starting values of beta for each K
#            if(is.null(starting_beta)) {
#                starting_beta <- array(list(),c(length(K),1))
#                rownames(starting_beta) <- paste0(as.character(K),"_signatures")
#                colnames(starting_beta) <- "Value"
#            }
#
#            # consider all the values for K
#            cont <- 0
#            pos_k <- 0
#            for(k in K) {
#                    
#                # get the first k signatures to be used for the current configuration
#                pos_k <- pos_k + 1
#                if(is.null(starting_beta[[pos_k,1]])) {
#                    if(nmf_method=="nmf_lasso") {
#                        
#                        if(verbose) {
#                            cat("Computing the initial values of beta by NMF with Lasso...","\n")
#                        }
#                        
#                        # add a signature to beta (leading to K+1 signatures in total) to explicitly model noise
#                        if(is.null(background_signature)) {
#                            warning("No background signature has been specified...")
#                        }
#                        
#                        # compute the starting points nmf_runs times
#                        beta_estimation <- list()
#                        beta_mse <- NULL
#                        for(i in 1:nmf_runs) {
#                            
#                            # set the initial random values for beta
#                            if(is.null(background_signature)) {
#                                curr_beta <- matrix(0,nrow=k,ncol=dim(x)[2])
#                                for(i in 1:k) {
#                                    curr_beta[i,] <- runif(dim(x)[2])
#                                }
#                                colnames(curr_beta) <- colnames(x)
#                            }
#                            else {
#                                curr_beta <- matrix(0,nrow=(k+1),ncol=dim(x)[2])
#                                curr_beta[1,] <- background_signature
#                                for(i in 2:(k+1)) {
#                                    curr_beta[i,] <- runif(dim(x)[2])
#                                }
#                                rownames(curr_beta) <- c("background_signature",rep("",k))
#                                colnames(curr_beta) <- colnames(x)
#                            }
#                            
#                            # compute the starting beta given these initial values
#                            curr_starting_beta_estimation = tryCatch({
#                                res <- nmfLassoDecomposition(x,curr_beta,lambda_rate=0.01,iterations=20,max_iterations_lasso=10000,parallel=parallel,verbose=FALSE)
#                                mse <- sum((x-round(res$alpha%*%res$curr_beta))^2)/nrow(x)
#                                list(curr_beta=res$curr_beta,mse=mse)
#                            }, error = function(e) {
#                                list(curr_beta=NA,mse=NA)
#                            })
#                            
#                            # save the results at the current step if not NA
#                            if(!is.na(curr_starting_beta_estimation$mse)) {
#                                beta_estimation[[(length(beta_estimation)+1)]] <- curr_starting_beta_estimation$curr_beta
#                                beta_mse <- c(beta_mse,curr_starting_beta_estimation$mse)
#                            }
#                            
#                        }
#                        # estimate the best starting betas
#                        if(is.null(beta_mse)) {
#                            stopCluster(parallel)
#                            stop("Something went wrong while estimating the starting beta values, you may try again or consider to use nmf_standard initialization...")
#                        }
#                        else {
#                            curr_beta <- beta_estimation[[which.min(beta_mse)]]
#                        }
#                        
#                    }
#                    else if(nmf_method=="nmf_standard") {
#                        
#                        if(verbose) {
#                            cat("Computing the initial values of beta by standard NMF...","\n")
#                        }
#                        curr_beta <- basis(nmf(t(x),rank=k,nrun=nmf_runs))
#                        curr_beta <- t(curr_beta)
#                        
#                        # add a signature to beta (leading to K+1 signatures in total) to explicitly model noise
#                        if(is.null(background_signature)) {
#                            warning("No background signature has been specified...")
#                        }
#                        else {
#                            curr_beta <- rbind(background_signature,curr_beta)
#                        }
#                        
#                    }
#                    curr_beta <- curr_beta / rowSums(curr_beta)
#                    starting_beta[[pos_k,1]] <- curr_beta
#                }
#                else {
#                    curr_beta <- starting_beta[[pos_k,1]]
#                }
#                
#                # consider all the values for lambda
#                pos_l <- 0
#                for(l in lambda_values) {
#                    
#                    # set the predicted values for the cross validation entries
#                    pos_l <- pos_l + 1
#                    if(cv_iteration>1 && !is.na(grid_search_iterations[[(cv_iteration-1)]][[pos_k,pos_l]])) {
#                        best_alpha <- grid_search_iterations[[(cv_iteration-1)]][[pos_k,pos_l]][["alpha"]]
#                        best_beta <- grid_search_iterations[[(cv_iteration-1)]][[pos_k,pos_l]][["beta"]]
#                        predicted_counts <- best_alpha %*% best_beta
#                        x_cv[cv_entries] <- predicted_counts[cv_entries]
#                    }
#                    
#                    # perform the inference
#                    curr_results <- nmf.LassoK(x = x_cv, 
#                                             K = k, 
#                                             beta = curr_beta, 
#                                             background_signature = background_signature, 
#                                             nmf_runs = 10, 
#                                             lambda_rate = l, 
#                                             iterations = iterations, 
#                                             max_iterations_lasso = max_iterations_lasso, 
#                                             num_processes = NULL, 
#                                             parallel = parallel, 
#                                             seed = round(runif(1)*100000), 
#                                             verbose = FALSE)
#                    
#                    # save the results for the current configuration
#                    grid_search[[pos_k,pos_l]] <- curr_results
#                    
#                    if(verbose) {
#                        cont <- cont + 1
#                        cat("Progress",paste0(round((cont/(length(K)*length(lambda_values)))*100,digits=3),"%..."),"\n")
#                    }
#                    
#                }
#                
#            }
#            
#            # save the results for the current iteration
#            grid_search_iterations[[cv_iteration]] <- grid_search
#
#        }
#        
#        if(verbose) {
#            cat("Evaluating the results of the cross validation in terms of mean squared error...","\n")
#        }
#
#        # structure to save the mean squared errors for all the cross_validation_iterations
#        mean_squared_error_iterations <- list()
#
#        # repeat the estimation for a number of cross_validation_iterations
#        for(cv_iteration in 1:cross_validation_iterations) {
#        
#            # structure to save the mean squared errors
#            mean_squared_error <- array(NA,c(length(K),length(lambda_values)))
#            rownames(mean_squared_error) <- paste0(as.character(K),"_signatures")
#            colnames(mean_squared_error) <- paste0(as.character(lambda_values),"_lambda")
#            
#            # assess the results of the cross validation by mean squared error
#            J <- dim(x)[2]
#            pos_k <- 0
#            for(k in K) {
#                pos_k <- pos_k + 1
#                pos_l <- 0
#                for(l in lambda_values) {
#                    
#                    # consider the current configuration
#                    pos_l <- pos_l + 1
#                    curr_alpha <- grid_search_iterations[[cv_iteration]][[pos_k,pos_l]][["alpha"]]
#                    curr_beta <- grid_search_iterations[[cv_iteration]][[pos_k,pos_l]][["beta"]]
#                    
#                    # compute the mean squared error
#                    if(!is.na(curr_alpha)&&!is.na(curr_beta)) {
#                        curr_predicted_counts = round(curr_alpha%*%curr_beta)
#                        curr_true_considered_counts = as.vector(x[cv_entries])
#                        curr_predicted_considered_counts = as.vector(curr_predicted_counts[cv_entries])
#                        error = mean((curr_true_considered_counts-curr_predicted_considered_counts)^2)
#                    }
#                    else {
#                        error <- NA
#                    }
#
#                    mean_squared_error[pos_k,pos_l] <- error
#                    
#                }
#            }
#
#            # save the results for the current iteration
#            mean_squared_error_iterations[[cv_iteration]] <- mean_squared_error
#
#        }
#        
#        if(verbose) {
#            cat("Estimating the best configuration...","\n")
#        }
#
#        # save the results
#        curr_results <- list(grid_search=grid_search_iterations,starting_beta=starting_beta,mean_squared_error=mean_squared_error_iterations)
#        results[[cv_repetitions]] <- curr_results
#
#    }
#    
#    # close parallel
#    if(!is.null(parallel)) {
#        stopCluster(parallel)
#    }
#    
#    return(results)
#    

}

# Perform a robust estimation of the starting beta for the nmfLasso method 
# @title starting.betas.estimation
# @param x count matrix.
# @param K range of numeric values (each of them greater than 1) indicating the number of signatures to be discovered.
# @param background_signature background signature to be used. If not provided, a warning is thrown.
# @param nmf_runs number of iteration of NMF to be performed for a robust estimation of starting beta. If beta is not NULL, 
# this parameter is ignored.
# @param num_processes Number of processes to be used during parallel execution. If executing in single process mode, 
# this is ignored.
# @param seed Seed for reproducibility.
# @param verbose boolean; Shall I print all messages?
# @return A list of starting beta values for each configuration of K.
# @export starting.betas.estimation
# @importFrom NMF basis nmf
# @import nnls
# @import parallel
#
"starting.betas.estimation" <- function( x, K = 3:10, background_signature = NULL, nmf_runs = 10, num_processes = Inf, seed = NULL, verbose = TRUE ) {

    .Deprecated("startingBetaEstimation")

#    # set the seed
#    set.seed(seed)
#
#    nmf_method <- "nmf_standard"
#    
#    # setting up parallel execution
#    close_parallel <- FALSE
#    if(nmf_method=="nmf_lasso") {
#        if(is.na(num_processes) || is.null(num_processes)) {
#            parallel <- NULL
#        }
#        else if(num_processes==Inf) {
#            cores <- as.integer((detectCores()-1))
#            if(cores < 2) {
#                parallel <- NULL
#            }
#            else {
#                num_processes <- cores
#                parallel <- makeCluster(num_processes,outfile="")
#                clusterSetRNGStream(parallel,iseed<-round(runif(1)*100000))
#                close_parallel <- TRUE
#            }
#        }
#        else {
#            parallel <- makeCluster(num_processes,outfile="")
#            clusterSetRNGStream(parallel,iseed=round(runif(1)*100000))
#            close_parallel <- TRUE
#        }
#        
#        if(verbose && !is.null(parallel)) {
#            cat("Executing",num_processes,"processes via parallel...","\n")
#        }
#    }
#    
#    # perform a robust estimation of the starting beta for the nmfLasso method
#    if(verbose) {
#        cat("Performing a robust estimation of the starting betas for the nmfLasso method...","\n")
#    }
#
#    # structure to save the starting values of beta for each K
#    starting_beta <- array(list(),c(length(K),1))
#    rownames(starting_beta) <- paste0(as.character(K),"_signatures")
#    colnames(starting_beta) <- "Value"
#
#    # consider all the values for K
#    pos_k <- 0
#    for(k in K) {
#    
#        # compute the initial values of beta
#        pos_k <- pos_k + 1
#        beta <- NULL
#        if(is.null(beta)) {
#            
#            if(nmf_method=="nmf_lasso") {
#                
#                if(verbose) {
#                    cat("Computing the initial values of beta by NMF with Lasso...","\n")
#                }
#                
#                # add a signature to beta (leading to K+1 signatures in total) to explicitly model noise
#                if(is.null(background_signature)) {
#                    warning("No background signature has been specified...")
#                }
#                
#                # compute the starting points nmf_runs times
#                beta_estimation <- list()
#                beta_mse <- NULL
#                for(i in 1:nmf_runs) {
#                    
#                    # set the initial random values for beta
#                    if(is.null(background_signature)) {
#                        beta <- matrix(0,nrow=k,ncol=dim(x)[2])
#                        for(i in 1:k) {
#                            beta[i,] <- runif(dim(x)[2])
#                        }
#                        colnames(beta) <- colnames(x)
#                    }
#                    else {
#                        beta <- matrix(0,nrow=(k+1),ncol=dim(x)[2])
#                        beta[1,] <- background_signature
#                        for(i in 2:(k+1)) {
#                            beta[i,] <- runif(dim(x)[2])
#                        }
#                        rownames(beta) <- c("background_signature",rep("",k))
#                        colnames(beta) <- colnames(x)
#                    }
#                    
#                    # compute the starting beta given these initial values
#                    curr_starting_beta_estimation = tryCatch({
#                        res <- nmfLassoDecomposition(x,beta,lambda_rate=0.01,iterations=20,max_iterations_lasso=10000,parallel=parallel,verbose=FALSE)
#                        mse <- sum((x-round(res$alpha%*%res$beta))^2)/nrow(x)
#                        list(beta=res$beta,mse=mse)
#                    }, error = function(e) {
#                        list(beta=NA,mse=NA)
#                    })
#                    
#                    # save the results at the current step if not NA
#                    if(!is.na(curr_starting_beta_estimation$mse)) {
#                        beta_estimation[[(length(beta_estimation)+1)]] <- curr_starting_beta_estimation$beta
#                        beta_mse <- c(beta_mse,curr_starting_beta_estimation$mse)
#                    }
#                    
#                }
#                # estimate the best starting betas
#                if(is.null(beta_mse)) {
#                    if(close_parallel) {
#                        stopCluster(parallel)
#                    }
#                    stop("Something went wrong while estimating the starting beta values, you may try again or consider to use nmf_standard initialization...")
#                }
#                else {
#                    beta <- beta_estimation[[which.min(beta_mse)]]
#                }
#                
#            }
#            else if(nmf_method=="nmf_standard") {
#                
#                if(verbose) {
#                    cat("Computing the initial values of beta by standard NMF...","\n")
#                }
#                beta <- basis(nmf(t(x),rank=k,nrun=nmf_runs))
#                beta <- t(beta)
#                
#                # add a signature to beta (leading to K+1 signatures in total) to explicitly model noise
#                if(is.null(background_signature)) {
#                    warning("No background signature has been specified...")
#                }
#                else {
#                    beta <- rbind(background_signature,beta)
#                }
#                
#            }
#
#        }
#        beta <- beta / rowSums(beta)
#        starting_beta[[pos_k,1]] <- beta
#
#        if(verbose) {
#            cat("Progress",paste0(round((pos_k/length(K))*100,digits=3),"%..."),"\n")
#        }
#
#    }
#    
#    # close parallel
#    if(close_parallel) {
#        stopCluster(parallel)
#    }
#
#    return(starting_beta)
#    

}

# Estimate the range of lambda values to be considered in the signature inference. Note that too small values of lambda 
# result in dense signatures, but too large values lead to bad fit of the counts.
# @title evaluate.lambda.range
# @param x count matrix.
# @param K numeric value (greater than 1) indicating the number of signatures to be discovered.
# @param beta starting beta for the estimation. If it is NULL, starting beta is estimated by NMF.
# @param background_signature background signature to be used. If not provided, a warning is thrown.
# @param nmf_runs number of iteration of NMF to be performed for a robust estimation of starting beta. If beta is not NULL, 
# this parameter is ignored.
# @param lambda_values range of values of LASSO to be used between 0 and 1. This value should be greater than 0. 1 is the value of LASSO 
# that would shrink all the signatures to 0 within one step. The higher lambda_rate is, the sparser are the resulting signatures, 
# but too large values result in a poor fit of the counts.
# @param iterations Number of iterations to be performed. Each iteration correspond to a first step where the counts are fitted 
# and a second step where sparsity is enhanced.
# @param max_iterations_lasso Number of maximum iterations to be performed during the sparsification.
# @param num_processes Number of processes to be used during parallel execution. If executing in single process mode, 
# this is ignored.
# @param seed Seed for reproducibility.
# @param verbose boolean; Shall I print all messages?
# @return A list corresponding to results of the function nmf.LassoK for each value of lambda to be tested. This function allows 
# to test a good range of lambda values to be considered. One should keep in mind that too small values generate dense solution, 
# while too high ones leads to poor fit. This behavior is resampled in the values of loglik_progression, which should be increasing: 
# too small values of lamda results in unstable log-likelihood through the iterations, while too large values make log-likelihood 
# drop. 
# @export evaluate.lambda.range
# @import NMF
# @import nnlasso
# @import nnls
# @import parallel
#
"evaluate.lambda.range" <- function( x, K = 6, beta = NULL, background_signature = NULL, nmf_runs = 10, lambda_values = c(0.10, 0.20, 0.30, 0.40, 0.50), iterations = 20, max_iterations_lasso = 10000, num_processes = Inf, seed = NULL, verbose = TRUE ) {

    .Deprecated("lambdaRangeBetaEvaluation")

#    # set the seed
#    set.seed(seed)
#
#    nmf_method <- "nmf_standard"
#    
#    # setting up parallel execution
#    parallel <- NULL
#    close_parallel <- FALSE
#    if(is.null(parallel)) {
#        if(is.na(num_processes) || is.null(num_processes)) {
#            parallel <- NULL
#        }
#        else if(num_processes==Inf) {
#            cores <- as.integer((detectCores()-1))
#            if(cores < 2) {
#                parallel <- NULL
#            }
#            else {
#                num_processes <- cores
#                parallel <- makeCluster(num_processes,outfile="")
#                clusterSetRNGStream(parallel,iseed=round(runif(1)*100000))
#                close_parallel <- TRUE
#            }
#        }
#        else {
#            parallel <- makeCluster(num_processes,outfile="")
#            clusterSetRNGStream(parallel,iseed=round(runif(1)*100000))
#            close_parallel <- TRUE
#        }
#        
#        if(verbose && !is.null(parallel)) {
#            cat("Executing",num_processes,"processes via parallel...","\n")
#        }
#    }
#    
#    # compute the initial values of beta
#    if(is.null(beta)) {
#        if(nmf_method=="nmf_lasso") {
#            
#            if(verbose) {
#                cat("Computing the initial values of beta by NMF with Lasso...","\n")
#            }
#            
#            # add a signature to beta (leading to K+1 signatures in total) to explicitly model noise
#            if(is.null(background_signature)) {
#                warning("No background signature has been specified...")
#            }
#            
#            # compute the starting points nmf_runs times
#            beta_estimation <- list()
#            beta_mse <- NULL
#            for(i in 1:nmf_runs) {
#                
#                # set the initial random values for beta
#                if(is.null(background_signature)) {
#                    beta <- matrix(0,nrow=K,ncol=dim(x)[2])
#                    for(i in 1:K) {
#                        beta[i,] <- runif(dim(x)[2])
#                    }
#                    colnames(beta) <- colnames(x)
#                }
#                else {
#                    beta <- matrix(0,nrow=(K+1),ncol=dim(x)[2])
#                    beta[1,] <- background_signature
#                    for(i in 2:(K+1)) {
#                        beta[i,] <- runif(dim(x)[2])
#                    }
#                    rownames(beta) <- c("background_signature",rep("",K))
#                    colnames(beta) <- colnames(x)
#                }
#                
#                # compute the starting beta given these initial values
#                curr_starting_beta_estimation = tryCatch({
#                    res <- nmfLassoDecomposition(x,beta,lambda_rate=0.01,iterations=20,max_iterations_lasso=10000,parallel=parallel,verbose=FALSE)
#                    mse <- sum((x-round(res$alpha%*%res$beta))^2)/nrow(x)
#                    list(beta=res$beta,mse=mse)
#                }, error = function(e) {
#                    list(beta=NA,mse=NA)
#                })
#                
#                # save the results at the current step if not NA
#                if(!is.na(curr_starting_beta_estimation$mse)) {
#                    beta_estimation[[(length(beta_estimation)+1)]] <- curr_starting_beta_estimation$beta
#                    beta_mse <- c(beta_mse,curr_starting_beta_estimation$mse)
#                }
#                
#            }
#            # estimate the best starting betas
#            if(is.null(beta_mse)) {
#                if(close_parallel) {
#                    stopCluster(parallel)
#                }
#                stop("Something went wrong while estimating the starting beta values, you may try again or consider to use nmf_standard initialization...")
#            }
#            else {
#                beta <- beta_estimation[[which.min(beta_mse)]]
#            }
#            
#        }
#        else if(nmf_method=="nmf_standard") {
#            
#            if(verbose) {
#                cat("Computing the initial values of beta by standard NMF...","\n")
#            }
#            beta <- basis(nmf(t(x),rank=K,nrun=nmf_runs))
#            beta <- t(beta)
#            
#            # add a signature to beta (leading to K+1 signatures in total) to explicitly model noise
#            if(is.null(background_signature)) {
#                warning("No background signature has been specified...")
#            }
#            else {
#                beta <- rbind(background_signature,beta)
#            }
#            
#        }
#    }
#    
#    # structure to save the estimated signatures
#    lambda_results <- array(list(),c(length(K),length(lambda_values)))
#    rownames(lambda_results) <- paste0(as.character(K),"_signatures")
#    colnames(lambda_results) <- paste0(as.character(lambda_values),"_lambda")
#    
#    # perform signature discovery for all the values of lambda
#    cont <- 0
#    for(l in lambda_values) {
#            
#        # perform the inference
#        curr_results <- nmf.LassoK(x = x, 
#                                 K = K, 
#                                 beta = beta, 
#                                 background_signature = background_signature, 
#                                 nmf_runs = 10, 
#                                 lambda_rate = l, 
#                                 iterations = iterations, 
#                                 max_iterations_lasso = max_iterations_lasso, 
#                                 num_processes = NULL, 
#                                 parallel = parallel, 
#                                 seed = round(runif(1)*100000), 
#                                 verbose = FALSE)
#                                 
#        # save the results for the current configuration
#        cont <- cont + 1
#        lambda_results[[1,cont]] <- curr_results
#            
#        if(verbose) {
#            cat("Progress",paste0(round((cont/(length(K)*length(lambda_values)))*100,digits=3),"%..."),"\n")
#        }
#        
#    }
#    
#    # close parallel
#    if(close_parallel) {
#        stopCluster(parallel)
#    }
#    
#    # save the results
#    results <- lambda_results
#    
#    return(results)
#    

}

# Perform the discovery of K somatic mutational signatures given a set of observed counts x.
#
### @examples
### data(starting_betas_example)
### beta = starting_betas_example[["5_signatures","Value"]]
### res = nmf.LassoK(x=patients,K=5,beta=beta,background=background,lambda_rate=0.10,iterations=5,num_processes=NA)
#
# @title nmf.LassoK
# @param x count matrix.
# @param K numeric value (greater than 1) indicating the number of signatures to be discovered.
# @param beta starting beta for the estimation. If it is NULL, starting beta is estimated by NMF.
# @param background_signature background signature to be used. If not provided, a warning is thrown.
# @param nmf_runs number of iteration of NMF to be performed for a robust estimation of starting beta. If beta is not NULL, 
# this parameter is ignored.
# @param lambda_rate value of LASSO to be used between 0 and 1. This value should be greater than 0. 1 is the value of LASSO 
# that would shrink all the signatures to 0 within one step. The higher lambda_rate is, the sparser are the resulting signatures, 
# but too large values result in a poor fit of the counts.
# @param iterations Number of iterations to be performed. Each iteration correspond to a first step where the counts are fitted 
# and a second step where sparsity is enhanced.
# @param max_iterations_lasso Number of maximum iterations to be performed during the sparsification.
# @param num_processes Number of processes to be used during parallel execution. If executing in single process mode, 
# this is ignored.
# @param parallel Cluster object for parallel execution.
# @param seed Seed for reproducibility.
# @param verbose boolean; Shall I print all messages?
# @return A list with the discovered signatures. It includes 5 elements: 
#              alpha: matrix of the discovered alpha values
#              beta: matrix of the discovered signatures
#              starting_beta: initial signatures on which the method has been apploid
#              best_loglik: log-likelihood of the best signatures configuration
#              loglik_progression: log-likelihood values during the iterations. This values should be increasing, if not the selected value of lambda is too high
# @export nmf.LassoK
# @import NMF
# @import nnlasso
# @import nnls
# @import parallel
#
"nmf.LassoK" <- function( x, K, beta = NULL, background_signature = NULL, nmf_runs = 10, lambda_rate = 0.20, iterations = 20, max_iterations_lasso = 10000, num_processes = Inf, parallel = NULL, seed = NULL, verbose = TRUE ) {

    .Deprecated("nmfLasso")

#    # set the seed
#    set.seed(seed)
#
#    nmf_method <- "nmf_standard"
#    
#    # setting up parallel execution
#    close_parallel <- FALSE
#    if(is.null(parallel)) {
#        if(is.na(num_processes) || is.null(num_processes)) {
#            parallel <- NULL
#        }
#        else if(num_processes==Inf) {
#            cores <- as.integer((detectCores()-1))
#            if(cores < 2) {
#                parallel <- NULL
#            }
#            else {
#                num_processes <- cores
#                parallel <- makeCluster(num_processes,outfile="")
#                clusterSetRNGStream(parallel,iseed=round(runif(1)*100000))
#                close_parallel <- TRUE
#            }
#        }
#        else {
#            parallel <- makeCluster(num_processes,outfile="")
#            clusterSetRNGStream(parallel,iseed=round(runif(1)*100000))
#            close_parallel <- TRUE
#        }
#        
#        if(verbose && !is.null(parallel)) {
#            cat("Executing",num_processes,"processes via parallel...","\n")
#        }
#    }
#    
#    # compute the initial values of beta
#    if(is.null(beta)) {
#        if(nmf_method=="nmf_lasso") {
#            
#            if(verbose) {
#                cat("Computing the initial values of beta by NMF with Lasso...","\n")
#            }
#            
#            # add a signature to beta (leading to K+1 signatures in total) to explicitly model noise
#            if(is.null(background_signature)) {
#                warning("No background signature has been specified...")
#            }
#            
#            # compute the starting points nmf_runs times
#            beta_estimation <- list()
#            beta_mse <- NULL
#            for(i in 1:nmf_runs) {
#                
#                # set the initial random values for beta
#                if(is.null(background_signature)) {
#                    beta <- matrix(0,nrow=K,ncol=dim(x)[2])
#                    for(i in 1:K) {
#                        beta[i,] <- runif(dim(x)[2])
#                    }
#                    colnames(beta) <- colnames(x)
#                }
#                else {
#                    beta <- matrix(0,nrow=(K+1),ncol=dim(x)[2])
#                    beta[1,] <- background_signature
#                    for(i in 2:(K+1)) {
#                        beta[i,] <- runif(dim(x)[2])
#                    }
#                    rownames(beta) <- c("background_signature",rep("",K))
#                    colnames(beta) <- colnames(x)
#                }
#                
#                # compute the starting beta given these initial values
#                curr_starting_beta_estimation = tryCatch({
#                    res <- nmfLassoDecomposition(x,beta,lambda_rate=0.01,iterations=20,max_iterations_lasso=10000,parallel=parallel,verbose=FALSE)
#                    mse <- sum((x-round(res$alpha%*%res$beta))^2)/nrow(x)
#                    list(beta=res$beta,mse=mse)
#                }, error = function(e) {
#                    list(beta=NA,mse=NA)
#                })
#                
#                # save the results at the current step if not NA
#                if(!is.na(curr_starting_beta_estimation$mse)) {
#                    beta_estimation[[(length(beta_estimation)+1)]] <- curr_starting_beta_estimation$beta
#                    beta_mse <- c(beta_mse,curr_starting_beta_estimation$mse)
#                }
#                
#            }
#            # estimate the best starting betas
#            if(is.null(beta_mse)) {
#                if(close_parallel) {
#                    stopCluster(parallel)
#                }
#                stop("Something went wrong while estimating the starting beta values, you may try again or consider to use nmf_standard initialization...")
#            }
#            else {
#                beta <- beta_estimation[[which.min(beta_mse)]]
#            }
#            
#        }
#        else if(nmf_method=="nmf_standard") {
#            
#            if(verbose) {
#                cat("Computing the initial values of beta by standard NMF...","\n")
#            }
#            beta <- basis(nmf(t(x),rank=K,nrun=nmf_runs))
#            beta <- t(beta)
#            
#            # add a signature to beta (leading to K+1 signatures in total) to explicitly model noise
#            if(is.null(background_signature)) {
#                warning("No background signature has been specified...")
#            }
#            else {
#                beta <- rbind(background_signature,beta)
#            }
#            
#        }
#    }
#    
#    if(verbose) {
#        cat("Performing the discovery of the signatures by NMF with Lasso...","\n")
#    }
#    
#    # perform the discovery of the signatures
#    results = tryCatch({
#        nmfLassoDecomposition(x,beta,lambda_rate,iterations,max_iterations_lasso,parallel,verbose)
#    }, error = function(e) {
#        warning("Lasso did not converge, you should try a lower value of lambda! Current settings: K = ",K,", lambda_rate = ",lambda_rate,"...")
#        list(alpha=NA,beta=NA,starting_beta=NA,best_loglik=NA,loglik_progression=rep(NA,iterations))
#    })
#    
#    # close parallel
#    if(close_parallel) {
#        stopCluster(parallel)
#    }
#    
#    return(results)

}

# perform de novo discovery of somatic mutational signatures using NMF with Lasso to ensure sparsity
#"nmfLassoDecomposition" <- function( x, beta, lambda_rate = 0.20, iterations = 20, max_iterations_lasso = 10000, parallel = NULL, verbose = TRUE ) {
#    
#    # n is the number of observations in x, i.e., the patients
#    n <- dim(x)[1]
#    
#    # J is the number of trinucleotides, i.e., 96 categories
#    J <- dim(x)[2]
#    
#    # K is the number of signatures to be called
#    K <- dim(beta)[1]
#    
#    # initialize alpha
#    alpha <- matrix(0,nrow=n,ncol=K)
#    rownames(alpha) <- 1:nrow(alpha)
#    colnames(alpha) <- rownames(beta)
#    
#    # starting beta
#    starting_beta <- beta / rowSums(beta)
#    beta <- starting_beta * 1000
#    
#    # structure where to save the log-likelihood at each iteration 
#    loglik <- rep(NA,iterations)
#    
#    # structure where to save the lambda values for each J
#    lambda_values <- rep(NA,J)
#    
#    if(verbose) {
#        cat("Performing a total of",iterations,"iterations...","\n")
#    }
#    
#    # repeat a 2 step algorithm iteratively, where first alpha is estimated by Non-Negative Linear Least Squares 
#    # and, then, beta is estimated by Non-Negative Lasso
#    best_loglik <- -Inf
#    best_alpha <- NA
#    best_beta <- NA
#    for(i in 1:iterations) {
#        
#        # initialize the value of the log-likelihood for the current iteration
#        loglik[i] <- 0
#        
#        # update alpha independently for each patient by Non-Negative Linear Least Squares
#        if(is.null(parallel)) {
#            for(j in 1:n) {
#                alpha[j,] <- nnls(t(beta),as.vector(x[j,]))$x
#            }
#        }
#        else {
#            
#            # compute alpha in parallel
#            j <- 1:n
#            res_clusterEvalQ <- clusterEvalQ(parallel,library("nnls"))
#            clusterExport(parallel,varlist=c("x","beta"),envir=environment())
#            alpha_res <- parLapply(parallel,j,function(j) {
#                return(nnls(t(beta),as.vector(x[j,]))$x)
#            })
#            
#            # reduce the results
#            for(j in 1:n) {
#                alpha[j,] <- alpha_res[[j]]
#            }
#            
#        }
#        
#        # update beta by Non-Negative Lasso
#        if(is.null(parallel)) {
#            
#            for(k in 1:J) {
#                
#                # compute independently for each trinucleotide the error between the observed counts, i.e., x, 
#                # and the ones predicted by the first signature (i.e., which represents the noise model)
#                error <- x[,k] - alpha[,1] * beta[1,k]
#                
#                # estimate beta for the remaining signatues by Non-Negative Lasso to ensure sparsity
#                if(is.na(lambda_values[k])) {
#                    max_lambda_value <- max(abs(t(alpha[,2:K]) %*% error))
#                    lambda_values[k] <- max_lambda_value * lambda_rate
#                }
#                beta[2:K,k] <- as.vector(nnlasso(x = alpha[,2:K], 
#                                                y = error, 
#                                                family = "normal", 
#                                                lambda = lambda_values[k], 
#                                                intercept = FALSE, 
#                                                normalize = FALSE, 
#                                                maxiter = max_iterations_lasso, 
#                                                path = FALSE)$coef[2,])
#                
#                # update the log-likelihood for the current iteration
#                curr_loglik <- -sum((x[,k] - alpha %*% beta[,k])^2) - lambda_values[k] * sum(beta[2:K,k])
#                loglik[i] <- loglik[i] + curr_loglik
#                
#            }
#            
#        }
#        else {
#            
#            # if this is the first iteration, compute the values of lambda sequentially
#            if(is.na(lambda_values[1])) {
#                
#                for(k in 1:J) {
#                    error <- x[,k] - alpha[,1] * beta[1,k]
#                    max_lambda_value <- max(abs(t(alpha[,2:K]) %*% error))
#                    lambda_values[k] <- max_lambda_value * lambda_rate
#                }
#                
#            }
#            
#            # perform the computations of beta in parallel
#            k <- 1:J
#            res_clusterEvalQ <- clusterEvalQ(parallel,library("nnlasso"))
#            clusterExport(parallel,varlist=c("x","alpha","beta","lambda_values","K","max_iterations_lasso"),envir<-environment())
#            beta_res <- parLapply(parallel,1:J,function(k) {
#                
#                # compute independently for each trinucleotide the error between the observed counts, i.e., x, 
#                # and the ones predicted by the first signature (i.e., which represents the noise model)
#                error <- x[,k] - alpha[,1] * beta[1,k]
#                
#                # estimate beta for the remaining signatues by Non-Negative Lasso to ensure sparsity
#                beta[2:K,k] <- as.vector(nnlasso(x = alpha[,2:K], 
#                                                y = error, 
#                                                family = "normal", 
#                                                lambda = lambda_values[k], 
#                                                intercept = FALSE, 
#                                                normalize = FALSE, 
#                                                maxiter = max_iterations_lasso, 
#                                                path = FALSE)$coef[2,])
#                
#                # compute the log-likelihood for the current iteration
#                curr_loglik <- -sum((x[,k] - alpha %*% beta[,k])^2) - lambda_values[k] * sum(beta[2:K,k])
#                
#                return(list(beta=beta[2:K,k],loglik=curr_loglik))
#                
#            })
#            
#            # reduce the results
#            for(k in 1:J) {
#                
#                beta[2:K,k] <- beta_res[[k]][["beta"]]
#                loglik[i] <- loglik[i] + beta_res[[k]][["loglik"]]
#                
#            }
#            
#        }
#        
#        # save the grid_search at maximum log-likelihood
#        if(loglik[i]>best_loglik) {
#            best_loglik <- loglik[i]
#            best_alpha <- alpha
#            best_beta <- beta
#        }
#        
#        if(verbose) {
#            cat("Progress",paste0((i/iterations)*100,"%..."),"\n")
#        }
#        
#    }
#    alpha <- best_alpha
#    beta <- best_beta
#    
#    # check if the likelihood is increasing
#    if(length(loglik)>1) {
#        cont <- 1
#        for(i in 2:length(loglik)) {
#            if(loglik[i]>loglik[(i-1)]) {
#                cont <- cont + 1
#            }
#        }
#        if(cont/iterations<0.5) {
#            warning("The likelihood is not increasing, you should try a lower value of lambda! Current settings: K = ",K,", lambda_rate = ",lambda_rate,"...")
#        }
#    }
#    
#    # normalize the rate of the signatures to sum to 1
#    beta <- beta / rowSums(beta)
#        
#    # final computation of alpha by Non-Negative Linear Least Squares using the normalized version o the best beta
#    if(is.null(parallel)) {
#        for(j in 1:n) {
#            alpha[j,] <- nnls(t(beta),as.vector(x[j,]))$x
#        }
#    }
#    else {
#        
#        # compute alpha in parallel
#        j <- 1:n
#        res_clusterEvalQ <- clusterEvalQ(parallel,library("nnls"))
#        clusterExport(parallel,varlist=c("x","beta"),envir=environment())
#        alpha_res <- parLapply(parallel,j,function(j) {
#            return(nnls(t(beta),as.vector(x[j,]))$x)
#        })
#        
#        # reduce the results
#        for(j in 1:n) {
#            alpha[j,] <- alpha_res[[j]]
#        }
#        
#    }
#    
#    # save the results
#    results <- list(alpha=alpha,beta=beta,starting_beta=starting_beta,best_loglik=best_loglik,loglik_progression=loglik)
#    
#    return(results)
#    
#}

#' Perform the assessment of different sigAssignmentLasso solutions by cross validation for a given set of somatic mutational signatures and observations x. The estimation can slow down because of 
#' memory usage and intensive computations, when a big number of cross validation repetitions is asked and when the grid search is performed for a lot of configurations. In this case, 
#' an advice may be to split the computation into multiple smaller sets.
#'
#' @examples
#' data(patients)
#' data(starting_betas_example)
#' beta = starting_betas_example[["5_signatures","Value"]]
#' res = sigAssignmentCV(x=patients[1:100,],
#'      beta=beta,
#'      lambda_values_alpha=c(0.00),
#'      cross_validation_repetitions=2,
#'      num_processes=NA,
#'      seed=12345)
#'
#' @title sigAssignmentCV
#' @param x count matrix for a set of n patients and 96 trinucleotides.
#' @param beta beta to be fixed during the estimation of alpha.
#' @param normalize_counts if true, the input count matrix x is normalize such that the patients have the same number of mutation.
#' @param lambda_values_alpha value of LASSO to be used for alpha between 0 and 1. This value should be greater than 0. 1 is the value of LASSO 
#' that would shrink all the signatures to 0 within one step. The higher lambda_rate_alpha is, the sparser are the resulting exposures, 
#' but too large values may result in a reduced fit of the observed counts.
#' @param cross_validation_entries Percentage of cells in the count matrix to be replaced by 0s during cross validation.
#' @param cross_validation_iterations For each configuration, the first time the signatures are discovered form a matrix with a 
#' percentage of values replaced by 0s. This may result in poor fit/results. Then, we perform predictions of these entries and replace them with 
#' such predicted values. This parameter is the number of restarts to be performed to improve this estimate and obtain more stable solutions.
#' @param cross_validation_repetitions Number of time cross-validation should be repeated. Higher values result in better estimate, but are computationally more expensive.
#' @param max_iterations_lasso Number of maximum iterations to be performed during the sparsification via Lasso.
#' @param num_processes Number of processes to be used during parallel execution. To execute in single process mode, 
#' this parameter needs to be set to either NA or NULL.
#' @param seed Seed for reproducibility.
#' @param verbose boolean; Shall I print all messages?
#' @param log_file log file where to print outputs when using parallel. If parallel execution is disabled, this parameter is ignored.
#' @return A list of 2 elements: grid_search_mse and and grid_search_loglik. Here, grid_search_mse reports the mean squared error for each configuration of performed 
#' cross validation; grid_search_loglik reports for each configuration the number of times the algorithm converged.
#' @export sigAssignmentCV
#' @import nnlasso
#' @import nnls
#' @import RhpcBLASctl
#' @import parallel
#'
"sigAssignmentCV" <- function( x, beta, normalize_counts = TRUE, lambda_values_alpha = c(0.00, 0.01, 0.05, 0.10), cross_validation_entries = 0.01, cross_validation_iterations = 5, cross_validation_repetitions = 50, max_iterations_lasso = 10000, num_processes = Inf, seed = NULL, verbose = TRUE, log_file = "" ) {
    
    # set the seed
    set.seed(seed)

    # check the input parameters
    x <- as.matrix(x)
    if(normalize_counts) {
        x <- (x/rowSums(x))*2500
    }
    lambda_values_alpha <- sort(unique(lambda_values_alpha))

    # initialize beta
    beta <- beta / rowSums(beta)
    rownames(beta) <- c("Background",paste0("S",1:(nrow(beta)-1)))
    colnames(beta) <- colnames(x)

    # perform a grid search to estimate the best values of K and lambda
    if(verbose) {
        cat("Performing a search to estimate the best values of lambda with a total of",cross_validation_repetitions,"cross validation repetitions...","\n")
    }

    # setting up parallel execution
    parallel <- NULL
    close_parallel <- FALSE
    if(is.null(parallel)&&cross_validation_repetitions>1) {
        if(is.na(num_processes) || is.null(num_processes)) {
            parallel <- NULL
        }
        else if(num_processes==Inf) {
            cores <- as.integer((detectCores()-1))
            if(cores < 2) {
                parallel <- NULL
            }
            else {
                num_processes <- min(num_processes,cross_validation_repetitions)
                parallel <- makeCluster(num_processes,outfile=log_file)
                close_parallel <- TRUE
            }
        }
        else {
            num_processes <- min(num_processes,cross_validation_repetitions)
            parallel <- makeCluster(num_processes,outfile=log_file)
            close_parallel <- TRUE
        }
        
        if(verbose && !is.null(parallel)) {
            cat("Executing",num_processes,"processes via parallel...","\n")
        }
    }

    # structure to save the results of the grid search
    grid_search_mse <- array(list(),c(length(lambda_values_alpha),1,1),dimnames=list(paste0(as.character(lambda_values_alpha),"_Lambda_Alpha"),"Fixed_Lambda_Beta",paste0(as.character(dim(beta)[1]),"_Signatures")))
    grid_search_loglik <- array(list(),c(length(lambda_values_alpha),1,1),dimnames=list(paste0(as.character(lambda_values_alpha),"_Lambda_Alpha"),"Fixed_Lambda_Beta",paste0(as.character(dim(beta)[1]),"_Signatures")))

    # now starting cross validation
    if(is.null(parallel)) {

        # perform a total of cross_validation_repetitions repetitions of cross validation
        set.seed(round(runif(1)*100000))
        for(cv_repetitions in 1:cross_validation_repetitions) {

            if(verbose) {
                cat(paste0("Performing repetition ",cv_repetitions," out of ",cross_validation_repetitions,"..."),"\n")
            }

            # randomly set the cross validation entries for the current iteration
            valid_entries <- which(x>0,arr.ind=TRUE)
            cv_entries <- valid_entries[sample(1:nrow(valid_entries),size=round(nrow(valid_entries)*cross_validation_entries),replace=FALSE),]

            # consider all the values for K
            cont <- 0
            pos_k <- 0
            for(k in 1:1) {
                # consider the first k signatures to be used for the current configuration
                pos_k <- pos_k + 1

                # consider all the values for lambda
                pos_l_alpha <- 0
                for(l_alpha in lambda_values_alpha) {
                    pos_l_alpha <- pos_l_alpha + 1

                    pos_l_beta <- 0
                    for(l_beta in 1:1) {
                        pos_l_beta <- pos_l_beta + 1

                        # repeat the estimation for a number of cross_validation_iterations
                        x_cv <- NULL
                        cont <- cont + 1
                        for(cv_iteration in 1:cross_validation_iterations) {
                    
                            if(verbose) {
                                cat(paste0("Performing cross validation iteration ",cv_iteration," out of ",cross_validation_iterations,"..."),"\n")
                            }

                            # set a percentage of cross_validation_entries entries to 0 in order to perform cross validation
                            if(cv_iteration==1) {
                                x_cv <- x
                                x_cv[cv_entries] <- 0
                            }
                            else {
                                if(!is.na(curr_iter_alpha[1])&&!is.na(curr_iter_beta[1])) {
                                    predicted_counts <- curr_iter_alpha %*% curr_iter_beta
                                    x_cv[cv_entries] <- predicted_counts[cv_entries]
                                }
                            }

                            # perform the inference
                            curr_results <- sigAssignmentLasso(x = x_cv, 
                                                                beta = beta, 
                                                                normalize_counts = normalize_counts, 
                                                                lambda_rate_alpha = l_alpha, 
                                                                max_iterations_lasso = max_iterations_lasso, 
                                                                seed = round(runif(1)*100000), 
                                                                verbose = FALSE)

                            curr_iter_alpha <- curr_results[["alpha"]]
                            curr_iter_beta <- curr_results[["beta"]]

                        }
                        
                        # save an estimate of mean squared error and stability of the solution
                        if(!is.na(curr_iter_alpha[1])&&!is.na(curr_iter_beta[1])) {
                            curr_predicted_counts <- round(curr_iter_alpha%*%curr_iter_beta)
                            curr_true_considered_counts <- as.vector(x[cv_entries])
                            curr_predicted_considered_counts <- as.vector(curr_predicted_counts[cv_entries])
                            error <- mean((curr_true_considered_counts-curr_predicted_considered_counts)^2)
                            grid_search_mse[pos_l_alpha,pos_l_beta,pos_k][[1]] <- c(grid_search_mse[pos_l_alpha,pos_l_beta,pos_k][[1]],error)
                            grid_search_loglik[pos_l_alpha,pos_l_beta,pos_k][[1]] <- c(grid_search_loglik[pos_l_alpha,pos_l_beta,pos_k][[1]],"increasing")
                        }
                        else {
                            grid_search_loglik[pos_l_alpha,pos_l_beta,pos_k][[1]] <- c(grid_search_loglik[pos_l_alpha,pos_l_beta,pos_k][[1]],"decreasing")
                        }
                        
                        if(verbose) {
                            cat("Progress",paste0(round((cont/(1*length(lambda_values_alpha)*1))*100,digits=3),"%..."),"\n")
                        }

                    }

                }

            }

        }

        # save the results
        results <- list(grid_search_mse=grid_search_mse,grid_search_loglik=grid_search_loglik)

    }
    else {

        # perform the inference
        res_clusterEvalQ <- clusterEvalQ(parallel,library("nnls"))
        res_clusterEvalQ <- clusterEvalQ(parallel,library("RhpcBLASctl"))
        res_clusterEvalQ <- clusterEvalQ(parallel,library("nnlasso"))
        clusterExport(parallel,varlist=c("x","beta","normalize_counts","cross_validation_entries"),envir=environment())
        clusterExport(parallel,varlist=c("lambda_values_alpha","cross_validation_iterations","max_iterations_lasso"),envir=environment())
        clusterExport(parallel,varlist=c("grid_search_mse","grid_search_loglik","cross_validation_repetitions","verbose"),envir=environment())
        clusterExport(parallel,c('sigAssignmentLasso','nmfLassoDeconstruction'),envir=environment())
        clusterSetRNGStream(parallel,iseed=round(runif(1)*100000))
        curr_results <- parLapply(parallel,1:cross_validation_repetitions,function(cv_rep) {

            if(verbose) {
                cat(paste0("Performing repetition ",cv_rep," out of ",cross_validation_repetitions,"..."),"\n")
            }

            # randomly set the cross validation entries for the current iteration
            valid_entries <- which(x>0,arr.ind=TRUE)
            cv_entries <- valid_entries[sample(1:nrow(valid_entries),size=round(nrow(valid_entries)*cross_validation_entries),replace=FALSE),]

            # consider all the values for K
            cont <- 0
            pos_k <- 0
            for(k in 1:1) {
                # consider the first k signatures to be used for the current configuration
                pos_k <- pos_k + 1

                # consider all the values for lambda
                pos_l_alpha <- 0
                for(l_alpha in lambda_values_alpha) {
                    pos_l_alpha <- pos_l_alpha + 1

                    pos_l_beta <- 0
                    for(l_beta in 1:1) {
                        pos_l_beta <- pos_l_beta + 1

                        # repeat the estimation for a number of cross_validation_iterations
                        x_cv <- NULL
                        cont <- cont + 1
                        for(cv_iteration in 1:cross_validation_iterations) {
                    
                            # set a percentage of cross_validation_entries entries to 0 in order to perform cross validation
                            if(cv_iteration==1) {
                                x_cv <- x
                                x_cv[cv_entries] <- 0
                            }
                            else {
                                if(!is.na(curr_iter_alpha[1])&&!is.na(curr_iter_beta[1])) {
                                    predicted_counts <- curr_iter_alpha %*% curr_iter_beta
                                    x_cv[cv_entries] <- predicted_counts[cv_entries]
                                }
                            }

                            # perform the inference
                            curr_results <- sigAssignmentLasso(x = x_cv, 
                                                                beta = beta, 
                                                                normalize_counts = normalize_counts, 
                                                                lambda_rate_alpha = l_alpha, 
                                                                max_iterations_lasso = max_iterations_lasso, 
                                                                seed = round(runif(1)*100000), 
                                                                verbose = FALSE)

                            curr_iter_alpha <- curr_results[["alpha"]]
                            curr_iter_beta <- curr_results[["beta"]]

                        }
                        
                        # save an estimate of mean squared error and stability of the solution
                        if(!is.na(curr_iter_alpha[1])&&!is.na(curr_iter_beta[1])) {
                            curr_predicted_counts <- round(curr_iter_alpha%*%curr_iter_beta)
                            curr_true_considered_counts <- as.vector(x[cv_entries])
                            curr_predicted_considered_counts <- as.vector(curr_predicted_counts[cv_entries])
                            error <- mean((curr_true_considered_counts-curr_predicted_considered_counts)^2)
                            grid_search_mse[pos_l_alpha,pos_l_beta,pos_k][[1]] <- c(grid_search_mse[pos_l_alpha,pos_l_beta,pos_k][[1]],error)
                            grid_search_loglik[pos_l_alpha,pos_l_beta,pos_k][[1]] <- c(grid_search_loglik[pos_l_alpha,pos_l_beta,pos_k][[1]],"increasing")
                        }
                        else {
                            grid_search_loglik[pos_l_alpha,pos_l_beta,pos_k][[1]] <- c(grid_search_loglik[pos_l_alpha,pos_l_beta,pos_k][[1]],"decreasing")
                        }

                    }

                }

            }

            # save the results
            curr_results <- list(grid_search_mse=grid_search_mse,grid_search_loglik=grid_search_loglik)

            return(curr_results)

        })

        # save the results
        results <- list(grid_search_mse=grid_search_mse,grid_search_loglik=grid_search_loglik)
        for(par_res in 1:length(curr_results)) {
            curr_par_res <- curr_results[[par_res]]
            curr_par_res_grid_search_mse <- curr_par_res$grid_search_mse
            curr_par_res_grid_search_loglik <- curr_par_res$grid_search_loglik
            for(a in 1:dim(curr_par_res_grid_search_mse)[1]) {
                for(b in 1:dim(curr_par_res_grid_search_mse)[2]) {
                    for(c in 1:dim(curr_par_res_grid_search_mse)[3]) {
                        if(!is.null(unlist(curr_par_res_grid_search_mse[a,b,c]))) {
                            results$grid_search_mse[[a,b,c]] <- c(unlist(results$grid_search_mse[a,b,c]),unlist(curr_par_res_grid_search_mse[a,b,c]))
                            results$grid_search_loglik[[a,b,c]] <- c(unlist(results$grid_search_loglik[a,b,c]),unlist(curr_par_res_grid_search_loglik[a,b,c]))
                        }
                    }
                }
            }
        }

    }
    
    # close parallel
    if(close_parallel) {
        stopCluster(parallel)
    }
    
    return(results)
    
}

#' Estimate the range of lambda values for alpha to be considered in the signature assignment. Note that too small values of lambda 
#' result in dense exposures, but too large values lead to bad fit of the counts.
#'
#' @examples
#' data(patients)
#' data(starting_betas_example)
#' beta = starting_betas_example[["5_signatures","Value"]]
#' res = sigAssignmentEvaluation(x=patients[1:100,],
#'      beta=beta,
#'      lambda_values=c(0.01,0.05),
#'      num_processes=NA,
#'      seed=12345)
#'
#' @title sigAssignmentEvaluation
#' @param x count matrix for a set of n patients and 96 trinucleotides.
#' @param beta beta to be fixed during the estimation of alpha.
#' @param normalize_counts if true, the input count matrix x is normalize such that the patients have the same number of mutation.
#' @param lambda_values value of LASSO to be used for alpha between 0 and 1. This value should be greater than 0. 1 is the value of LASSO 
#' that would shrink all the signatures to 0 within one step. The higher lambda_values is, the sparser are the resulting exposures, 
#' but too large values may result in a reduced fit of the observed counts.
#' @param max_iterations_lasso Number of maximum iterations to be performed during the sparsification via Lasso.
#' @param num_processes Number of processes to be used during parallel execution. To execute in single process mode, 
#' this parameter needs to be set to either NA or NULL.
#' @param seed Seed for reproducibility.
#' @param verbose boolean; Shall I print all messages?
#' @param log_file log file where to print outputs when using parallel. If parallel execution is disabled, this parameter is ignored.
#' @return A list corresponding to results of the function sigAssignmentLasso for each value of lambda to be tested. This function allows 
#' to test a good range of lambda values for alpha to be considered. One should keep in mind that too small values generate dense solution, 
#' while too high ones leads to poor fit. This behavior is resampled in the values of loglik_progression, which should be increasing: 
#' too small values of lambda results in unstable log-likelihood through the iterations, while too large values make log-likelihood 
#' drop. 
#' @export sigAssignmentEvaluation
#' @import nnlasso
#' @import nnls
#' @import RhpcBLASctl
#' @import parallel
#'
"sigAssignmentEvaluation" <- function( x, beta, normalize_counts = TRUE, lambda_values = c(0.01, 0.05, 0.10, 0.20), max_iterations_lasso = 10000, num_processes = Inf, seed = NULL, verbose = TRUE, log_file = "" ) {
    
    # set the seed
    set.seed(seed)

    # check the input parameters
    x <- as.matrix(x)
    if(normalize_counts) {
        x_not_normalized <- x
        x <- (x/rowSums(x))*2500
    }
    lambda_values <- sort(unique(lambda_values))

    # initialize beta
    beta <- beta / rowSums(beta)
    rownames(beta) <- c("Background",paste0("S",1:(nrow(beta)-1)))
    colnames(beta) <- colnames(x)
    
    # setting up parallel execution
    parallel <- NULL
    close_parallel <- FALSE
    if(is.null(parallel)&&length(lambda_values)>1) {
        if(is.na(num_processes) || is.null(num_processes)) {
            parallel <- NULL
        }
        else if(num_processes==Inf) {
            cores <- as.integer((detectCores()-1))
            if(cores < 2) {
                parallel <- NULL
            }
            else {
                num_processes <- min(num_processes,length(lambda_values))
                parallel <- makeCluster(num_processes,outfile=log_file)
                close_parallel <- TRUE
            }
        }
        else {
            num_processes <- min(num_processes,length(lambda_values))
            parallel <- makeCluster(num_processes,outfile=log_file)
            close_parallel <- TRUE
        }
        
        if(verbose && !is.null(parallel)) {
            cat("Executing",num_processes,"processes via parallel...","\n")
        }
    }
    
    # structure to save the estimated signatures
    lambda_results <- array(list(),c(1,length(lambda_values)))
    rownames(lambda_results) <- paste0(as.character(dim(beta)[1]),"_signatures")
    colnames(lambda_results) <- paste0(as.character(lambda_values),"_lambda")
    
    if(verbose) {
        cat("Performing estimation of lambda range for alpha...","\n")
    }
    
    # perform signature discovery for all the values of lambda
    if(is.null(parallel)) {
        set.seed(round(runif(1)*100000))
        cont <- 0
        for(l in lambda_values) {

            # perform the inference
            curr_results <- sigAssignmentLasso(x = x, 
                                                beta = beta, 
                                                normalize_counts = normalize_counts, 
                                                lambda_rate_alpha = l, 
                                                max_iterations_lasso = max_iterations_lasso, 
                                                seed = round(runif(1)*100000), 
                                                verbose = FALSE)
                                     
            # save the results for the current configuration
            cont <- cont + 1

            # rescale alpha to the original magnitude
            if(normalize_counts) {
                curr_results$alpha <- curr_results$alpha * (rowSums(x_not_normalized)/2500)
            }

            lambda_results[[1,cont]] <- curr_results
                
            if(verbose) {
                cat("Progress",paste0(round((cont/(1*length(lambda_values)))*100,digits=3),"%..."),"\n")
            }
            
        }
    }
    else {
        res_clusterEvalQ <- clusterEvalQ(parallel,library("nnls"))
        res_clusterEvalQ <- clusterEvalQ(parallel,library("RhpcBLASctl"))
        res_clusterEvalQ <- clusterEvalQ(parallel,library("nnlasso"))
        clusterExport(parallel,varlist=c("x","beta","normalize_counts","max_iterations_lasso"),envir=environment())
        clusterExport(parallel,c('sigAssignmentLasso','nmfLassoDeconstruction'),envir=environment())
        clusterSetRNGStream(parallel,iseed=round(runif(1)*100000))
        lambda_results_res <- parLapply(parallel,lambda_values,function(l) {

            # perform the inference
            curr_results <- sigAssignmentLasso(x = x, 
                                                beta = beta, 
                                                normalize_counts = normalize_counts, 
                                                lambda_rate_alpha = l, 
                                                max_iterations_lasso = max_iterations_lasso, 
                                                seed = round(runif(1)*100000), 
                                                verbose = FALSE)

            return(curr_results)

        })
        for(i in 1:ncol(lambda_results)) {
            # rescale alpha to the original magnitude
            if(normalize_counts) {
                lambda_results_res[[i]]$alpha <- lambda_results_res[[i]]$alpha * (rowSums(x_not_normalized)/2500)
            }
            lambda_results[[1,i]] <- lambda_results_res[[i]]
        }
    }
    
    # close parallel
    if(close_parallel) {
        stopCluster(parallel)
    }
    
    return(lambda_results)
    
}

#' Perform the assignment of somatic mutational signatures to patients given a set of observed counts x and signatures beta.
#'
#' @examples
#' data(patients)
#' data(starting_betas_example)
#' beta = starting_betas_example[["5_signatures","Value"]]
#' res = sigAssignmentLasso(x=patients[1:100,],beta=beta,lambda_rate_alpha=0.05,seed=12345)
#'
#' @title sigAssignmentLasso
#' @param x count matrix for a set of n patients and 96 trinucleotides.
#' @param beta beta to be fixed during the estimation of alpha.
#' @param normalize_counts if true, the input count matrix x is normalize such that the patients have the same number of mutation.
#' @param lambda_rate_alpha value of LASSO to be used for alpha between 0 and 1. This value should be greater than 0. 1 is the value of LASSO 
#' that would shrink all the exposure values to 0 within one step. The higher lambda_rate_alpha is, the sparser are the resulting exposure values, 
#' but too large values may result in a reduced fit of the observed counts.
#' @param max_iterations_lasso Number of maximum iterations to be performed during the sparsification via Lasso.
#' @param seed Seed for reproducibility.
#' @param verbose boolean; Shall I print all messages?
#' @return A list with the discovered signatures and their assignment to patients. It includes 2 elements: 
#'              alpha: matrix of the assigned exposure values
#'              beta: matrix of the discovered signatures
#' @export sigAssignmentLasso
#' @import nnls
#' @import RhpcBLASctl
#' @import nnlasso
#'
"sigAssignmentLasso" <- function( x, beta, normalize_counts = TRUE, lambda_rate_alpha = 0.05, max_iterations_lasso = 10000, seed = NULL, verbose = TRUE ) {
    
    # set the seed
    set.seed(seed)

    # check the input parameters
    if(lambda_rate_alpha<0) {
        if(verbose) {
            cat("The minimum value of lambda_rate_alpha is 0...","\n")
        }
        lambda_rate_alpha <- 0
    }
    if(lambda_rate_alpha>1) {
        if(verbose) {
            cat("The maximum value of lambda_rate_alpha is 1...","\n")
        }
        lambda_rate_alpha <- 1
    }
    x <- as.matrix(x)
    if(normalize_counts) {
        x_not_normalized <- x
        x <- (x/rowSums(x))*2500
    }

    # initialize beta
    beta <- beta / rowSums(beta)
    rownames(beta) <- c("Background",paste0("S",1:(nrow(beta)-1)))
    colnames(beta) <- colnames(x)
    
    if(verbose) {
        cat("Performing the assignment of signatures to patients with Lasso...","\n")
    }
    
    # perform the assignment of signatures to samples
    results = tryCatch({
        results = nmfLassoDeconstruction(x,beta,lambda_rate_alpha,max_iterations_lasso,round(runif(1)*100000))
        # rescale alpha to the original magnitude
        if(normalize_counts) {
            results$alpha <- results$alpha * (rowSums(x_not_normalized)/2500)
        }
        return(results)
    }, error = function(e) {
        warning("Lasso did not converge, you should try a lower value of lambda! Current settings: lambda_rate_alpha = ",lambda_rate_alpha,"...")
        return(list(alpha=NA,beta=NA))
    })
    
    return(results)
    
}

# perform assignment of somatic mutational signatures to samples using Lasso to ensure sparsity
"nmfLassoDeconstruction" <- function( x, beta, lambda_rate_alpha = 0.05, max_iterations_lasso = 10000, seed = NULL ) {
    
    # set the seed
    set.seed(seed)
    
    # n is the number of observations in x, i.e., the patients
    n <- dim(x)[1]
    
    # K is the number of signatures to be assigned
    K <- dim(beta)[1]
    
    # initialize beta
    beta <- beta / rowSums(beta)
    rownames(beta) <- c("Background",paste0("S",1:(nrow(beta)-1)))
    colnames(beta) <- colnames(x)
    
    # initialize alpha
    alpha <- matrix(0,nrow=n,ncol=K)
    rownames(alpha) <- 1:nrow(alpha)
    colnames(alpha) <- rownames(beta)
    
    # configuration 1 (turn off sparsity): alpha is estimated by Non-Negative Linear Least Squares 
    if(lambda_rate_alpha==0) {

        # estimate alpha by Non-Negative Linear Least Squares
        for(j in 1:n) {
            blas_set_num_threads(1)
            alpha[j,] <- nnls(t(beta),as.vector(x[j,]))$x
            blas_set_num_threads(blas_get_num_procs()) # reset to default
        }

    }

    # configuration 2 (sparsity on alpha): alpha is estimated by Non-Negative Lasso 
    if(lambda_rate_alpha>0) {

        # estimate alpha by Non-Negative Lasso
        for(j in 1:n) {

            # compute independently for each patient the counts to be approximted
            target <- x[j,]
            
            # estimate alpha by Non-Negative Lasso to ensure sparsity
            max_lambda_value <- max(abs(beta %*% target))
            lambda_values_alpha <- max_lambda_value * lambda_rate_alpha
            alpha[j,] <- as.vector(nnlasso(x = t(beta), 
                                            y = target, 
                                            family = "normal", 
                                            lambda = lambda_values_alpha, 
                                            intercept = FALSE, 
                                            normalize = FALSE, 
                                            tol = 1e-05, 
                                            maxiter = max_iterations_lasso, 
                                            path = FALSE)$coef[2,])
            
        }

    }
    
    # save the results
    results <- list(alpha=alpha,beta=beta)
    
    return(results)
    
}

#' Plot trinucleotides counts for a set of given patients.
#'
#' @examples
#' data(patients)
#' patients.plot(trinucleotides_counts=patients,samples=c("PD10010a","PD10011a","PD10014a"))
#'
#' @title patients.plot
#' @param trinucleotides_counts trinucleotides counts matrix.
#' @param samples name of the samples. This should match a rownames in trinucleotides_counts.
#' @param freq boolean value; shall I display rates instead of counts?
#' @param xlabels boolean value; shall I display x labels?
#' @return A ggplot2 object.
#' @export patients.plot
#' @import ggplot2
#' @import gridExtra
#' @importFrom reshape2 melt
#' @importFrom data.table as.data.table :=
#'
"patients.plot" <- function( trinucleotides_counts, samples = rownames(trinucleotides_counts), freq = FALSE, xlabels = FALSE ) {

    # make samples data
    trinucleotides_counts <- trinucleotides_counts[samples,,drop=FALSE]
    if(freq) {
        trinucleotides_counts <- trinucleotides_counts / rowSums(trinucleotides_counts)
    }

    # separate context and alteration
    x <- as.data.table(reshape2::melt(as.matrix(trinucleotides_counts),varnames=c("patient","cat")))
    x[,Context:=paste0(substr(cat,1,1),".",substr(cat,7,7))]
    x[,alt:=paste0(substr(cat,3,3),">",substr(cat,5,5))]

    # make the ggplot2 object
    glist <- list()
    for(i in 1:nrow(trinucleotides_counts)) {

        plt <- ggplot(x[patient==rownames(trinucleotides_counts)[i]]) + 
            geom_bar(aes(x=Context,y=value,fill=alt),stat="identity",position="identity") + 
            facet_wrap(~alt,nrow=1,scales="free_x") + 
            theme(axis.text.x=element_text(angle=90,hjust=1),panel.background=element_blank(),axis.line=element_line(colour="black")) + 
            ggtitle(rownames(trinucleotides_counts)[i]) + theme(legend.position="none") + ylab("Number of mutations")

        if(freq) {
            plt <- plt + ylab("Frequency of mutations")
        }

        if(!xlabels) {
            plt <- plt + theme(axis.text.x=element_blank(),axis.ticks.x=element_blank())
        }

        glist[[i]] <- plt

    }

    # make the final plot
    grid.arrange(grobs=glist,ncol=ceiling(nrow(trinucleotides_counts)/3))

}


#' Plot the inferred mutational signatures.
#'
#' @examples
#' data(nmf_LassoK_example)
#' signatures.plot(beta=nmf_LassoK_example$beta)
#'
#' @title signatures.plot
#' @param beta matrix with the inferred mutational signatures.
#' @param useRowNames boolean value; shall I use the rownames from beta as names for the signatures?
#' @param xlabels boolean value; shall I display x labels?
#' @return A ggplot2 object.
#' @export signatures.plot
#' @import ggplot2
#' @import gridExtra
#' @importFrom data.table as.data.table :=
#'
"signatures.plot" <- function( beta, useRowNames = FALSE, xlabels = FALSE ) {

    # set names of the signatures
    if(!useRowNames) {
        rownames(beta) <- paste0("Signature ",1:nrow(beta))
    }

    # separate context and alteration
    x <- as.data.table(reshape2::melt(as.matrix(beta),varnames=c("signature","cat")))
    x[,Context:=paste0(substr(cat,1,1),".",substr(cat,7,7))]
    x[,alt:=paste0(substr(cat,3,3),">",substr(cat,5,5))]

    # make the ggplot2 object
    glist <- list()
    for(i in 1:nrow(beta)) {

        plt <- ggplot(x[signature==rownames(beta)[i]]) + 
            geom_bar(aes(x=Context,y=value,fill=alt),stat="identity",position="identity") + 
            facet_wrap(~alt,nrow=1,scales="free_x") + 
            theme(axis.text.x=element_text(angle=90,hjust=1),panel.background=element_blank(),axis.line=element_line(colour="black")) + 
            ggtitle(rownames(beta)[i]) + theme(legend.position="none") + ylab("Frequency of mutations")

        if(!xlabels) {
            plt <- plt + theme(axis.text.x=element_blank(),axis.ticks.x=element_blank())
        }

        glist[[i]] <- plt

    }

    # make the final plot
    grid.arrange(grobs=glist,ncol=ceiling(nrow(beta)/3))

}

#' Make trinucleotides counts matrix from input data for a given reference genome.
#'
#' @examples
#' data(ssm560_reduced)
#' library("BSgenome.Hsapiens.1000genomes.hs37d5")
#' trinucleotides_counts = import.trinucleotides.counts(data=ssm560_reduced, 
#'      reference=BSgenome.Hsapiens.1000genomes.hs37d5)
#' 
#'
#' @title import.trinucleotides.counts
#' @param data a data.frame with variants having 6 columns: sample name, chromosome, start position, end position, ref, alt.
#' @param reference a BSgenome object with the reference genome to be used to retrieve flanking bases.
#' @return A matrix with trinucleotides counts per patient.
#' @export import.trinucleotides.counts
#' @importFrom data.table data.table dcast .N
#' @importFrom Biostrings DNAStringSet complement reverseComplement subseq
#' @import GenomicRanges
#' @import IRanges
#' @import GenomeInfoDb
#' @importFrom BSgenome getSeq
#'
"import.trinucleotides.counts" <- function( data, reference = NULL ) {

    # check that reference is a BSgenome object
    if(is.null(reference)|class(reference)!="BSgenome") {
        stop("The reference genome provided as input needs to be a BSgenome object.")
    }

    # preprocessing input data
    data <- as.data.frame(data)
    colnames(data) <- c("sample","chrom","start","end","ref","alt")


    # consider only single nucleotide variants involving (A,C,G,T) bases
    data <- data[which(data[,"start"]==data[,"end"]),,drop=FALSE]
    data <- data[which(as.matrix(data[,"ref"])%in%c("A","C","G","T")),,drop=FALSE]
    data <- data[which(as.matrix(data[,"alt"])%in%c("A","C","G","T")),,drop=FALSE]
    data <- data[,c("sample","chrom","start","ref","alt"),drop=FALSE]
    colnames(data) <- c("sample","chrom","pos","ref","alt")
    data <- unique(data)
    data <- data[order(data[,"sample"],data[,"chrom"],data[,"pos"]),,drop=FALSE]

    
    # convert data to GRanges
    data <- GRanges(data$chrom,IRanges(start=(data$pos-1),width=3),ref=DNAStringSet(data$ref),alt=DNAStringSet(data$alt),sample=data$sample)

    # check that all chromosomes match reference
    if(length(setdiff(seqnames(data),GenomeInfoDb::seqnames(reference)))>0) {
        warning("Check chromosome names, not all match reference genome.")
    }

    # find context for each mutation
    data$context <- getSeq(reference,data)

    # check for any mismatch with BSgenome context
    if(any(subseq(data$context,2,2)!=data$ref)) {
        warning("Check reference bases, not all match context.")
    }

    # get complements and reverse complements
    data$cref <- complement(data$ref)
    data$calt <- complement(data$alt)
    data$rccontext <- reverseComplement(data$context)

    # identify trinucleotides motif
    data$cat <- ifelse(as.vector(data$ref) %in% c("C","T"),paste0(subseq(data$context,1,1),"[",data$ref,">",data$alt,"]",subseq(data$context,3,3)), 
                       paste0(subseq(data$rccontext,1,1),"[",data$cref,">",data$calt,"]",subseq(data$rccontext,3,3)))

    # create 96 trinucleotides mutation categories
    categories_context <- NULL
    categories_alt <- rep(c(rep("C>A",4),rep("C>G",4),rep("C>T",4),rep("T>A",4),rep("T>C",4),rep("T>G",4)),4)
    categories_cat <- NULL
    cont <- 0
    for(i in c("A","C","G","T")) {
        for(j in 1:6) {
            for(k in c("A","C","G","T")) {
                cont <- cont + 1
                categories_context <- c(categories_context,paste0(k,":",i))
                categories_cat <- c(categories_cat,paste0(k,"[",categories_alt[cont],"]",i))
            }
        }
    }
    mutation_categories <- data.table::data.table(context=categories_context,alt=categories_alt,cat=categories_cat)
    
    # count number of mutations per sample for each category
    input1 <- data.table::data.table(mutation_categories[,"cat"])
    input2 <- data.table::data.table(sample=data$sample,cat=data$cat)
    input2 <- input2[,.N,by=.(sample,cat)]
    data <- merge(input1,input2,by="cat",all=TRUE)
    data <- data.table::dcast(data,sample~cat,value.var="N")
    data <- data[!is.na(sample),drop=FALSE]
    data[is.na(data)] <- 0

    # make trinucleotides counts matrix
    samples_names <- data$sample
    data <- as.matrix(data[,2:ncol(data),drop=FALSE])
    rownames(data) <- samples_names
    data <- data[sort(rownames(data)),,drop=FALSE]
    data <- data[,sort(colnames(data)),drop=FALSE]
    trinucleotides_counts <- array(0,c(nrow(data),96))
    rownames(trinucleotides_counts) <- rownames(data)
    colnames(trinucleotides_counts) <- sort(as.character(mutation_categories$cat))
    rows_contexts <- rownames(data)
    cols_contexts <- colnames(trinucleotides_counts)[which(colnames(trinucleotides_counts)%in%colnames(data))]
    trinucleotides_counts[rows_contexts,cols_contexts] <- data[rows_contexts,cols_contexts]

    # return trinucleotides counts matrix
    return(trinucleotides_counts)

}

# Return the mean squared error between the observed counts and the predicted ones for all the configurations considered 
# during cross-validation. This function expects an input object as the one generated by the function nmf.LassoCV.
#
# @examples
# data(cv_example)
# res = as.mean.squared.error(cv_example)
#
# @title as.mean.squared.error
# @param nmf.LassoCV.result results of cross validation into an object as the one generated by the function nmf.LassoCV
# @return A list with distributions and median values of the mean squared error values computed between the observed counts 
# and the predicted ones at each iteration of cross validation
# @export as.mean.squared.error
#
#"as.mean.squared.error" <- function( nmf.LassoCV.result ) {
#
#    mean_squared_error_distribution <- array(list(),dim(nmf.LassoCV.result[[1]][["mean_squared_error"]][[1]]))
#    rownames(mean_squared_error_distribution) <- rownames(nmf.LassoCV.result[[1]][["mean_squared_error"]][[1]])
#    colnames(mean_squared_error_distribution) <- colnames(nmf.LassoCV.result[[1]][["mean_squared_error"]][[1]])
#
#    for(i in 1:length(nmf.LassoCV.result)) {
#        curr_mse <- nmf.LassoCV.result[[i]][["mean_squared_error"]]
#        curr_mse_best <- curr_mse[[length(curr_mse)]]
#        invalid <- which(is.na(curr_mse_best),arr.ind=TRUE)
#        if(dim(invalid)[1]>0) {
#            for(j in 1:dim(invalid)[1]) {
#                curr_conf_k <- invalid[j,"row"]
#                curr_conf_l <- invalid[j,"col"]
#                for(j in 1:(length(curr_mse)-1)) {
#                    if(!is.na(curr_mse[[j]][curr_conf_k,curr_conf_l])) {
#                        curr_mse_best[curr_conf_k,curr_conf_l] <- curr_mse[[j]][curr_conf_k,curr_conf_l]
#                    }
#                }
#            }
#        }
#        for(a in 1:dim(mean_squared_error_distribution)[1]) {
#            for(b in 1:dim(mean_squared_error_distribution)[2]) {
#                mean_squared_error_distribution[[a,b]] <- c(unlist(mean_squared_error_distribution[[a,b]]),curr_mse_best[a,b])
#            }
#        }
#    }
#    
#    mean_squared_error_median <- array(NA,dim(nmf.LassoCV.result[[1]][["mean_squared_error"]][[1]]))
#    rownames(mean_squared_error_median) <- rownames(nmf.LassoCV.result[[1]][["mean_squared_error"]][[1]])
#    colnames(mean_squared_error_median) <- colnames(nmf.LassoCV.result[[1]][["mean_squared_error"]][[1]])
#    for(a in 1:dim(mean_squared_error_distribution)[1]) {
#        for(b in 1:dim(mean_squared_error_distribution)[2]) {
#            mean_squared_error_median[a,b] <- median(mean_squared_error_distribution[[a,b]],na.rm=TRUE)
#        }
#    }
#    
#    mean_squared_error_results <- list(distribution=mean_squared_error_distribution,median=mean_squared_error_median)
#
#    return(mean_squared_error_results)
#    
#}
#
# Return the discovered alpha values for a given configuration from an input object as the one generated by the function evaluate.lambda.range.
#
# @examples
# data(lambda_range_example)
# res = as.alpha.in.range(lambda_range_example,lambda_value=0.10)
#
# @title as.alpha.in.range
# @param lambda.range.result discovered signatures into an object as the one generated by the function evaluate.lambda.range
# @param lambda_value value of LASSO used for the estimation whose configuration has to be returned
# @return A matrix with alpha values
# @export as.alpha.in.range
#
#"as.alpha.in.range" <- function( lambda.range.result, lambda_value = 0.20 ) {
#
#    nmf.LassoK.result <- lambda.range.result[[1,paste0(as.character(lambda_value),"_lambda")]]
#    return(nmf.LassoK.result$alpha)
#    
#}
#
# Return the discovered signatures (beta values) for a given configuration from an input object as the one generated by the function evaluate.lambda.range.
#
# @examples
# data(lambda_range_example)
# res = as.beta.in.range(lambda_range_example,lambda_value=0.10)
#
# @title as.beta.in.range
# @param lambda.range.result discovered signatures into an object as the one generated by the function evaluate.lambda.range
# @param lambda_value value of LASSO used for the estimation whose configuration has to be returned
# @return A matrix with beta values
# @export as.beta.in.range
#
#"as.beta.in.range" <- function( lambda.range.result, lambda_value = 0.20 ) {
#
#    nmf.LassoK.result <- lambda.range.result[[1,paste0(as.character(lambda_value),"_lambda")]]
#    return(nmf.LassoK.result$beta)
#    
#}
#
# Return the initial values of beta used for the estimation of the signatures for a given configuration from an input object as the one generated by the 
# function evaluate.lambda.range.
#
# @examples
# data(lambda_range_example)
# res = as.starting.beta.in.range(lambda_range_example,lambda_value=0.10)
#
# @title as.starting.beta.in.range
# @param lambda.range.result discovered signatures into an object as the one generated by the function nmf.LassoK
# @param lambda_value value of LASSO used for the estimation whose configuration has to be returned
# @return A matrix with the initial values of beta used for the estimation of the signatures
# @export as.starting.beta.in.range
#
#"as.starting.beta.in.range" <- function( lambda.range.result, lambda_value = 0.20 ) {
#
#    nmf.LassoK.result <- lambda.range.result[[1,paste0(as.character(lambda_value),"_lambda")]]
#    return(nmf.LassoK.result$starting_beta)
#    
#}
#
# Return the log-likelihood values for a given configuration for each iteration during the estimation of the signatures from an input object as the one 
# generated by the function evaluate.lambda.range.
#
# @examples
# data(lambda_range_example)
# res = as.loglik.progression.in.range(lambda_range_example,lambda_value=0.10)
#
# @title as.loglik.progression.in.range
# @param lambda.range.result discovered signatures into an object as the one generated by the function evaluate.lambda.range
# @param lambda_value value of LASSO used for the estimation whose configuration has to be returned
# @return A vector with log-likelihood values for each iteration during the estimation of the signatures
# @export as.loglik.progression.in.range
#
#"as.loglik.progression.in.range" <- function( lambda.range.result, lambda_value = 0.20 ) {
#
#    nmf.LassoK.result <- lambda.range.result[[1,paste0(as.character(lambda_value),"_lambda")]]
#    return(nmf.LassoK.result$loglik_progression)
#    
#}
#
# Return the discovered alpha values from an input object as the one generated by the function nmf.LassoK.
#
# @examples
# data(nmf_LassoK_example)
# res = as.alpha(nmf_LassoK_example)
#
# @title as.alpha
# @param nmf.LassoK.result discovered signatures into an object as the one generated by the function nmf.LassoK
# @return A matrix with alpha values
# @export as.alpha
#
#"as.alpha" <- function( nmf.LassoK.result ) {
#    
#    return(nmf.LassoK.result$alpha)
#    
#}
#
# Return the discovered signatures (beta values) from an input object as the one generated by the function nmf.LassoK.
#
# @examples
# data(nmf_LassoK_example)
# res = as.beta(nmf_LassoK_example)
#
# @title as.beta
# @param nmf.LassoK.result discovered signatures into an object as the one generated by the function nmf.LassoK
# @return A matrix with beta values
# @export as.beta
#
#"as.beta" <- function( nmf.LassoK.result ) {
#    
#    return(nmf.LassoK.result$beta)
#    
#}
#
# Return the initial values of beta used for the estimation of the signatures from an input object as the one generated by the function nmf.LassoK.
#
# @examples
# data(nmf_LassoK_example)
# res = as.starting.beta(nmf_LassoK_example)
#
# @title as.starting.beta
# @param nmf.LassoK.result discovered signatures into an object as the one generated by the function nmf.LassoK
# @return A matrix with the initial values of beta used for the estimation of the signatures
# @export as.starting.beta
#
#"as.starting.beta" <- function( nmf.LassoK.result ) {
#    
#    return(nmf.LassoK.result$starting_beta)
#    
#}
#
# Return the log-likelihood values for each iteration during the estimation of the signatures from an input object as the one generated by 
# the function nmf.LassoK.
#
# @examples
# data(nmf_LassoK_example)
# res = as.loglik.progression(nmf_LassoK_example)
#
# @title as.loglik.progression
# @param nmf.LassoK.result discovered signatures into an object as the one generated by the function nmf.LassoK
# @return A vector with log-likelihood values for each iteration during the estimation of the signatures
# @export as.loglik.progression
#
#"as.loglik.progression" <- function( nmf.LassoK.result ) {
#    
#    return(nmf.LassoK.result$loglik_progression)
#    
#}

#' @name patients 
#' @title point mutations for 560 breast tumors 
#' @description dataset of counts of the point mutations detected in 560 breast tumors published in Nik-Zainal, Serena, et al. (2016). 
#' @docType data 
#' @usage data(patients) 
#' @format counts of the point mutations 
#' @source Nik-Zainal, Serena, et al. "Landscape of somatic mutations in 560 breast cancer whole-genome sequences." Nature 534.7605 (2016): 47. 
#' @return counts of point mutations for 560 tumors and 96 trinucleotides 
NULL

#' @name background 
#' @title germline replication error 
#' @description germline replication error estimated in Rahbari, Raheleh, et al. (2016). 
#' @docType data 
#' @usage data(background) 
#' @format vector of rates 
#' @source Rahbari, Raheleh, et al. "Timing, rates and spectra of human germline mutation." Nature genetics 48.2 (2016): 126. 
#' @return vector of rates for the 96 trinucleotides 
NULL

#' @name background2 
#' @title COSMIC replication error 
#' @description background replication error signature derived from COSMIC SBS5. 
#' @docType data 
#' @usage data(background2) 
#' @format vector of rates 
#' @source COSMIC database (https://cancer.sanger.ac.uk/cosmic/signatures) v3. 
#' @return vector of rates for the 96 trinucleotides 
NULL

#' @name mutation_categories 
#' @title trinucleotides mutation categories 
#' @description 96 trinucleotides mutation categories 
#' @docType data 
#' @usage data(mutation_categories) 
#' @format matrix of 96 trinucleotides mutation categories 
#' @return matrix of 96 trinucleotides mutation categories 
NULL

#' @name ssm560_reduced 
#' @title a reduced version of the point mutations for 560 breast tumors in the format compatible with the import function 
#' @description reduced versione of the dataset of counts of the point mutations detected in 560 breast tumors published in Nik-Zainal, Serena, et al. (2016). 
#' @docType data 
#' @usage data(ssm560_reduced) 
#' @format reduced versione of the counts of the point mutations in the format compatible with the import function 
#' @source Nik-Zainal, Serena, et al. "Landscape of somatic mutations in 560 breast cancer whole-genome sequences." Nature 534.7605 (2016): 47. 
#' @return reduced versione of the counts of point mutations for 560 tumors and 96 trinucleotides in the format compatible with the import function 
NULL

#' @name starting_betas_example 
#' @title example of results obtained with the function starting.betas.estimation on the counts input from Nik-Zainal, Serena, et al. (2016). 
#' @description example of results obtained with the function starting.betas.estimation on the counts input from Nik-Zainal, Serena, et al. (2016). 
#' @docType data 
#' @usage data(starting_betas_example) 
#' @format results obtained with the function starting.betas.estimation on the counts input from Nik-Zainal, Serena, et al. (2016) 
#' @return results obtained with the function starting.betas.estimation on the counts input from Nik-Zainal, Serena, et al. (2016) 
NULL

#' @name lambda_range_example 
#' @title example of results obtained with the function evaluate.lambda.range on the counts input from Nik-Zainal, Serena, et al. (2016). 
#' @description example of results obtained with the function evaluate.lambda.range on the counts input from Nik-Zainal, Serena, et al. (2016). 
#' @docType data 
#' @usage data(lambda_range_example) 
#' @format results obtained with the function evaluate.lambda.range on the counts input from Nik-Zainal, Serena, et al. (2016) 
#' @return results obtained with the function evaluate.lambda.range on the counts input from Nik-Zainal, Serena, et al. (2016) 
NULL

#' @name cv_example 
#' @title example of results obtained with the function nmf.LassoCV on the counts input from Nik-Zainal, Serena, et al. (2016). 
#' @description example of results obtained with the function nmf.LassoCV on the counts input from Nik-Zainal, Serena, et al. (2016). 
#' @docType data 
#' @usage data(cv_example) 
#' @format results obtained with the function nmf.LassoCV on the counts input from Nik-Zainal, Serena, et al. (2016) 
#' @return results obtained with the function nmf.LassoCV on the counts input from Nik-Zainal, Serena, et al. (2016) 
NULL

#' @name nmf_LassoK_example 
#' @title example of results obtained with the function nmf.LassoK on the counts input from Nik-Zainal, Serena, et al. (2016). 
#' @description example of results obtained with the function nmf.LassoK on the counts input from Nik-Zainal, Serena, et al. (2016). 
#' @docType data 
#' @usage data(nmf_LassoK_example) 
#' @format results obtained with the function nmf.LassoK on the counts input from Nik-Zainal, Serena, et al. (2016) 
#' @return results obtained with the function nmf.LassoK on the counts input from Nik-Zainal, Serena, et al. (2016) 
NULL

#' @name imported_data 
#' @title example of imported data from Nik-Zainal, Serena, et al. (2016). 
#' @description example of imported data from Nik-Zainal, Serena, et al. (2016). 
#' @docType data 
#' @usage data(imported_data) 
#' @format results obtained with the function import.counts.data on the data from Nik-Zainal, Serena, et al. (2016) 
#' @return results obtained with the function import.counts.data on the data from Nik-Zainal, Serena, et al. (2016) 
NULL

# Import point mutations data to build the count matrix to extract mutatational signatues.
#
### @examples
### data(ssm560_reduced)
### library("BSgenome.Hsapiens.1000genomes.hs37d5")
### bsg = BSgenome.Hsapiens.1000genomes.hs37d5
### data(mutation_categories)
### imported_data = import.counts.data(input=ssm560_reduced,bsg=bsg,mutation_categories=mutation_categories)
### data(imported_data)
### head(imported_data)
#
# @title import.counts.data
# @param input either a data.frame/data.table object or a file with 5 columns: sample name, chromosome, position, ref, alt.
# @param bsg a BSgenome object for the reference genome. Chromosome names have to match the input table.
# @param mutation_categories array with the 96 mutational categories to be considered. It is provided along with the package 
# by data(mutation_categories).
# @return A count matrix to extract mutatational signatues
# @export import.counts.data
# @importFrom data.table data.table as.data.table fread dcast .N
# @importFrom Biostrings DNAStringSet complement reverseComplement subseq
# @import GenomicRanges
# @import IRanges
# @import GenomeInfoDb
# @import BSgenome.Hsapiens.1000genomes.hs37d5
# @importFrom BSgenome getSeq
#
"import.counts.data" <- function(input, bsg, mutation_categories) {

    .Deprecated("import.trinucleotides.counts")

#  # check that input is a data frame or data table
#  if (!("data.frame" %in% class(input))) {
#    if (file.exists(input)) {
#      input <- fread(input, header = TRUE, check.names = FALSE)
#    } else {
#      stop("Input is neither a file nor a loaded data frame.")
#    }
#  }
#  
#  # set column names
#  colnames(input) <- c("sample", "chrom", "pos", "ref", "alt")
#  
#  # convert input to GRanges
#  inp <- GRanges(input$chrom, IRanges(start=input$pos-1, width=3), ref=DNAStringSet(input$ref), alt=DNAStringSet(input$alt), sample=input$sample)
#
#  # filter input: recognized bases only
#  bad <- which(!(inp$ref %in% c("A", "T", "C", "G") & inp$alt %in% c("A", "T", "C", "G")))
#  if(length(bad)>0){
#    warning(paste0("Removing ", length(bad), " entries containing nonstandard bases. \n"))
#    inp <- inp[inp$ref %in% c("A", "T", "C", "G") & inp$alt %in% c("A", "T", "C", "G")]
#  }
#  
#  # check that bsg is a bsgenome object
#  if(is.null(bsg)|class(bsg) != "BSgenome") {
#      stop("The bsg parameter needs to be a BSgenome object.")
#  }
#  
#  # check that all chromosomes match bsg
#  if(length(setdiff(seqnames(inp), GenomeInfoDb::seqnames(bsg)))>0) {
#        warning(paste0("Check chromosome names -- not all match ", bsg, " object.\n"))
#        inp[seqnames(inp) %in% GenomeInfoDb::seqnames(bsg)]
#  }
#
#  # find context for each mutation
#  inp$context <- getSeq(bsg, inp)
#
#  # check for mismatches with BSgenome context
#  if(any(subseq(inp$context,2,2)!=inp$ref)) {
#    warning("Check ref bases -- not all match context.\n ")
#  }
#  
#  # get complements and reverse complements
#  inp$cref <- complement(inp$ref)
#  inp$calt <- complement(inp$alt)
#  inp$rccontext <- reverseComplement(inp$context)
#  
#  # identify motif
#  inp$cat <- ifelse(inp$ref %in% c("C", "T"), 
#                     paste0(subseq(inp$context,1,1), "[", inp$ref, ">", inp$alt, "]", subseq(inp$context, 3, 3)),
#                     paste0(subseq(inp$rccontext,1,1), "[", inp$cref, ">", inp$calt, "]", subseq(inp$rccontext, 3, 3)))
#  
#  # count number of mutations per sample, category
#  counts <- merge(mutation_categories[, .(cat)], data.table(sample=inp$sample, cat=inp$cat)[, .N, by=.(sample, cat)], by="cat", all=TRUE)
#  counts <- dcast(counts, sample~cat, value.var = "N")
#  counts <- counts[!is.na(sample)]
#  counts[is.na(counts)] <- 0
#  
#  # make count marix
#  countMatrix <- as.matrix(counts[, 2:ncol(counts)])
#  rownames(countMatrix) <- counts$sample
#  
#  # order matrix columns alphabetically
#  countMatrix <- countMatrix[,order(colnames(countMatrix))]
#  
#  # identify samples with <100 mutations
#  if(!is.null(nrow(countMatrix))){
#    bad <- names(rowSums(countMatrix) <= 100)
#  }else{
#    bad <- (sum(countMatrix) <= 100)
#  }
#  if (length(bad) > 0) {
#    warning(paste("Some samples have fewer than 100 mutations:\n ", paste(bad, collapse = ", "), sep = " "))
#  }
#  
#  # return matrix
#  return(countMatrix)
#

}

context("SparseSignatures")

data("nmf_LassoK_example")
test_that("SparseSignatures produces correct output", {
    expect_equal(names(nmf_LassoK_example),c("alpha","beta","starting_beta","best_loglik","loglik_progression"))
})

---
title: "Using the package"
author: 
  - Daniele Ramazzotti
  - Avantika Lal
  - Keli Liu
  - Luca De Sano
  - Robert Tibshirani
  - Arend Sidow
date: "`r format(Sys.time(), '%B %d, %Y')`"
graphics: yes
package: SparseSignatures
output:
  BiocStyle::html_document:
    toc_float: true
vignette: >
  %\VignetteIndexEntry{Using the package}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
  %\VignetteDepends{SparseSignatures,BiocStyle}
---

We now present the main features of the package. To start, we show how to load data and transform them to a count matrix to perform the signatures discovery; first we load some example data provided in the package. 


The SparseSignatures package can be installed from Bioconductor as follow.

```{r, eval=FALSE}
if (!require("BiocManager", quietly = TRUE))
    install.packages("BiocManager")

BiocManager::install("SparseSignature")
```

```{r}
library("SparseSignatures")
data(ssm560_reduced)
head(ssm560_reduced)
```

These data are a reduced version with only 3 patients of the 560 breast tumors provided by Nik-Zainal, Serena, et al. (2016). We can transform such input data to a count matrix to perform the signatures discovery with the function import.counts.data. To do so, we also need to specify the reference genome as a BSgenome object and the format of the 96 nucleotides to be considered. This can be done as follows, where in the example we use hs37d5 as our reference genome. 

```{r}
library("BSgenome.Hsapiens.1000genomes.hs37d5")
bsg = BSgenome.Hsapiens.1000genomes.hs37d5
data(mutation_categories)
head(mutation_categories)
```

```{r}
imported_data = import.trinucleotides.counts(data=ssm560_reduced,reference=bsg)
head(imported_data)
```

The function import.counts.data can also take a text file as input with the same format as the one shown above. Now, we show an example of a visualization feature provided by the package, and we show the counts for the first patient PD10010a in the following plot. 

```{r fig.width=7, fig.height=7, fig.cap="Visualization of the counts from patient PD10010a from the dataset published in Nik-Zainal, Serena, et al."}
patients.plot(trinucleotides_counts=imported_data,samples="PD10010a")
```

After the data are loaded, signatures can be discovered. To do so, we need to define a set of parameters on which to perform the estimation. 

First of all, we need to specify the ranges for the number of signatures (variable K) and the LASSO penalty value (variable lambda rate) to be considered. The latter is more complicated to estimate, as it requires that the values in the range not to be too small in order to avoid dense signatures, but also should not be to high in order to still perform a good fit of the observed counts. 

Besides these parameters, we also need to estimate the initial values of beta to be used during the estimation. We now show how to do this on the set of counts from 560 tumors provided in Nik-Zainal, Serena, et al. (2016). 

```{r}
data(patients)
head(patients)
```

First, we can estimate the initial values of beta as follows. 

```{r eval=FALSE}
starting_betas = startingBetaEstimation(x=patients,K=3:12,background_signature=background)
```

Then, we also need to explore the search space of values for the LASSO penalty in order to make a good choice. To do so, we can use the function lambdaRangeBetaEvaluation to test different values to sparsify beta as follows. Notice that the package also provides the option to sparsify alpha and, in this case, we may use the function lambdaRangeAlphaEvaluation to explore the search space of values. 

```{r eval=FALSE}
lambda_range = lambdaRangeBetaEvaluation(x=patients,K=10,beta=starting_betas[[8,1]],
                                         lambda_values=c(0.05,0.10))
```

As the executions of these functions can be very time-consuming, we also provide as examples together with the package a set of pre-computed results by the two functions startingBetaEstimation and lambdaRangeBetaEvaluation obtained with the commands above. 

```{r}
data(starting_betas_example)
data(lambda_range_example)
```

Now that we have evaluated all the required parameters, we need to decide which configuration of number of signatures and lambda value is the best. To do so, we rely on cross-validation. 

```{r eval=FALSE}
cv = nmfLassoCV(x=patients,K=3:10)
```

We notice that the computations for this task can be very time consuming, expecially when many iterations of cross validations are specified (see manual) and a large set of configurations of the parameters are tested. To speed up the execution, we suggest using the parallel execution options. Also, to reduce the memory requirements, we advise splitting the cross validation in different runs, e.g., if one wants to perform 100 iterations, we would suggest making 10 independent runs of 10 iterations each. Also in this case, we provide as examples together with the package a set of pre-computed results obtained with the above command and the following settings: K = 3:10, cross validation entries = 0.10, lambda values = c(0.05,0.10,0.15), number of iterations of cross-validation = 2. 

```{r}
data(cv_example)
```

Finally, we can compute the signatures for the best configuration, i.e., K = 5. 

```{r}
beta = starting_betas_example[["5_signatures","Value"]]
res = nmfLasso(x = patients, K = 5, beta = beta, background_signature = background, seed = 12345)
```


We conclude this vignette by plotting the discovered signatures. 

```{r fig.width=7, fig.height=7, fig.cap="Visualization of the discovered signatures."}
data(nmf_LassoK_example)
signatures = nmf_LassoK_example$beta
signatures.plot(beta=signatures, xlabels=FALSE)
```

---
title: "Introduction"
author: 
  - Daniele Ramazzotti
  - Avantika Lal
  - Keli Liu
  - Luca De Sano
  - Robert Tibshirani
  - Arend Sidow
date: "`r format(Sys.time(), '%B %d, %Y')`"
graphics: yes
package: SparseSignatures
output:
  BiocStyle::html_document:
    toc_float: true
vignette: >
  %\VignetteIndexEntry{Introduction}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
  %\VignetteDepends{SparseSignatures,BiocStyle}
---

Point mutations occurring in a genome can be divided into 96 categories based on the base being mutated, the base it is mutated into and its two flanking bases. Therefore, for any patient, it is possible to represent all the point mutations occurring in that patient’s tumor as a vector of length 96, where each element represents the count of mutations for a given category in the patient. 

A mutational signature represents the pattern of mutations produced by a mutagen or mutagenic process inside the cell. Each signature can also be represented by a vector of length 96, where each element represents the probability that this particular mutagenic process generates a mutation of the 96 above mentioned categories. In this R package, we provide a set of functions to extract and visualize the mutational signatures that best explain the mutation counts of a large number of patients. 

## Changelog


- *2.9.3* New vignette. 
- *2.0.0* Migration from Travis-CI to Github Actions and Major refactoring. 
- *1.0.4* Move NMF to Depends section. 
- *1.0.3* Issue with the basis function solved. 
- *1.0.0* package released on Bioconductor in May 2018. 

## Algorithms and useful links


Acronym | Extended name | Reference 
-------|---------------|---------------
SparseSignatures | De Novo Mutational Signature Discovery in Tumor Genomes using SparseSignatures | https://www.biorxiv.org/content/10.1101/384834v3 

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{background2}
\alias{background2}
\title{COSMIC replication error}
\format{
vector of rates
}
\source{
COSMIC database (https://cancer.sanger.ac.uk/cosmic/signatures) v3.
}
\usage{
data(background2)
}
\value{
vector of rates for the 96 trinucleotides
}
\description{
background replication error signature derived from COSMIC SBS5.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/visualization.R
\name{patients.plot}
\alias{patients.plot}
\title{patients.plot}
\usage{
patients.plot(
  trinucleotides_counts,
  samples = rownames(trinucleotides_counts),
  freq = FALSE,
  xlabels = FALSE
)
}
\arguments{
\item{trinucleotides_counts}{trinucleotides counts matrix.}

\item{samples}{name of the samples. This should match a rownames in trinucleotides_counts.}

\item{freq}{boolean value; shall I display rates instead of counts?}

\item{xlabels}{boolean value; shall I display x labels?}
}
\value{
A ggplot2 object.
}
\description{
Plot trinucleotides counts for a set of given patients.
}
\examples{
data(patients)
patients.plot(trinucleotides_counts=patients,samples=c("PD10010a","PD10011a","PD10014a"))

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/signatures.discovery.lasso.R
\name{lambdaRangeBetaEvaluation}
\alias{lambdaRangeBetaEvaluation}
\title{lambdaRangeBetaEvaluation}
\usage{
lambdaRangeBetaEvaluation(
  x,
  K = 5,
  beta = NULL,
  background_signature = NULL,
  normalize_counts = TRUE,
  nmf_runs = 10,
  lambda_values = c(0.01, 0.05, 0.1, 0.2),
  iterations = 30,
  max_iterations_lasso = 10000,
  num_processes = Inf,
  seed = NULL,
  verbose = TRUE,
  log_file = ""
)
}
\arguments{
\item{x}{count matrix for a set of n patients and 96 trinucleotides.}

\item{K}{numeric value (minimum 2) indicating the number of signatures to be discovered.}

\item{beta}{starting beta for the estimation. If it is NULL, starting beta is estimated by NMF.}

\item{background_signature}{background signature to be used. If not provided, a warning is thrown and an initial value for it is 
estimated by NMF. If beta is not NULL, this parameter is ignored.}

\item{normalize_counts}{if true, the input count matrix x is normalize such that the patients have the same number of mutation.}

\item{nmf_runs}{number of iteration (minimum 1) of NMF to be performed for a robust estimation of starting beta. If beta is not NULL, 
this parameter is ignored.}

\item{lambda_values}{value of LASSO to be used for beta between 0 and 1. This value should be greater than 0. 1 is the value of LASSO 
that would shrink all the signatures to 0 within one step. The higher lambda_values is, the sparser are the resulting signatures, 
but too large values may result in a reduced fit of the observed counts.}

\item{iterations}{Number of iterations to be performed. Each iteration corresponds to a first step where beta is fitted 
and a second step where alpha is fitted.}

\item{max_iterations_lasso}{Number of maximum iterations to be performed during the sparsification via Lasso.}

\item{num_processes}{Number of processes to be used during parallel execution. To execute in single process mode, 
this parameter needs to be set to either NA or NULL.}

\item{seed}{Seed for reproducibility.}

\item{verbose}{boolean; Shall I print all messages?}

\item{log_file}{log file where to print outputs when using parallel. If parallel execution is disabled, this parameter is ignored.}
}
\value{
A list corresponding to results of the function nmfLasso for each value of lambda to be tested. This function allows 
to test a good range of lambda values for beta to be considered. One should keep in mind that too small values generate dense solution, 
while too high ones leads to poor fit. This behavior is resampled in the values of loglik_progression, which should be increasing: 
too small values of lambda results in unstable log-likelihood through the iterations, while too large values make log-likelihood 
drop.
}
\description{
Estimate the range of lambda values for beta to be considered in the signature inference. Note that too small values of lambda 
result in dense signatures, but too large values lead to bad fit of the counts.
}
\examples{
data(background)
data(patients)
res = lambdaRangeBetaEvaluation(x=patients[1:100,],
     K=5,
     background_signature=background,
     nmf_runs=1,
     lambda_values=c(0.01,0.05),
     num_processes=NA,
     seed=12345)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/signatures.assignment.lasso.R
\name{sigAssignmentCV}
\alias{sigAssignmentCV}
\title{sigAssignmentCV}
\usage{
sigAssignmentCV(
  x,
  beta,
  normalize_counts = TRUE,
  lambda_values_alpha = c(0, 0.01, 0.05, 0.1),
  cross_validation_entries = 0.01,
  cross_validation_iterations = 5,
  cross_validation_repetitions = 50,
  max_iterations_lasso = 10000,
  num_processes = Inf,
  seed = NULL,
  verbose = TRUE,
  log_file = ""
)
}
\arguments{
\item{x}{count matrix for a set of n patients and 96 trinucleotides.}

\item{beta}{beta to be fixed during the estimation of alpha.}

\item{normalize_counts}{if true, the input count matrix x is normalize such that the patients have the same number of mutation.}

\item{lambda_values_alpha}{value of LASSO to be used for alpha between 0 and 1. This value should be greater than 0. 1 is the value of LASSO 
that would shrink all the signatures to 0 within one step. The higher lambda_rate_alpha is, the sparser are the resulting exposures, 
but too large values may result in a reduced fit of the observed counts.}

\item{cross_validation_entries}{Percentage of cells in the count matrix to be replaced by 0s during cross validation.}

\item{cross_validation_iterations}{For each configuration, the first time the signatures are discovered form a matrix with a 
percentage of values replaced by 0s. This may result in poor fit/results. Then, we perform predictions of these entries and replace them with 
such predicted values. This parameter is the number of restarts to be performed to improve this estimate and obtain more stable solutions.}

\item{cross_validation_repetitions}{Number of time cross-validation should be repeated. Higher values result in better estimate, but are computationally more expensive.}

\item{max_iterations_lasso}{Number of maximum iterations to be performed during the sparsification via Lasso.}

\item{num_processes}{Number of processes to be used during parallel execution. To execute in single process mode, 
this parameter needs to be set to either NA or NULL.}

\item{seed}{Seed for reproducibility.}

\item{verbose}{boolean; Shall I print all messages?}

\item{log_file}{log file where to print outputs when using parallel. If parallel execution is disabled, this parameter is ignored.}
}
\value{
A list of 2 elements: grid_search_mse and and grid_search_loglik. Here, grid_search_mse reports the mean squared error for each configuration of performed 
cross validation; grid_search_loglik reports for each configuration the number of times the algorithm converged.
}
\description{
Perform the assessment of different sigAssignmentLasso solutions by cross validation for a given set of somatic mutational signatures and observations x. The estimation can slow down because of 
memory usage and intensive computations, when a big number of cross validation repetitions is asked and when the grid search is performed for a lot of configurations. In this case, 
an advice may be to split the computation into multiple smaller sets.
}
\examples{
data(patients)
data(starting_betas_example)
beta = starting_betas_example[["5_signatures","Value"]]
res = sigAssignmentCV(x=patients[1:100,],
     beta=beta,
     lambda_values_alpha=c(0.00),
     cross_validation_repetitions=2,
     num_processes=NA,
     seed=12345)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{patients}
\alias{patients}
\title{point mutations for 560 breast tumors}
\format{
counts of the point mutations
}
\source{
Nik-Zainal, Serena, et al. "Landscape of somatic mutations in 560 breast cancer whole-genome sequences." Nature 534.7605 (2016): 47.
}
\usage{
data(patients)
}
\value{
counts of point mutations for 560 tumors and 96 trinucleotides
}
\description{
dataset of counts of the point mutations detected in 560 breast tumors published in Nik-Zainal, Serena, et al. (2016).
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/signatures.discovery.lasso.R
\name{nmfLassoBootstrap}
\alias{nmfLassoBootstrap}
\title{nmfLassoBootstrap}
\usage{
nmfLassoBootstrap(
  x,
  K = 3:10,
  starting_beta = NULL,
  background_signature = NULL,
  normalize_counts = TRUE,
  nmf_runs = 10,
  bootstrap_repetitions = 50,
  iterations = 30,
  max_iterations_lasso = 10000,
  num_processes = Inf,
  seed = NULL,
  verbose = TRUE,
  log_file = ""
)
}
\arguments{
\item{x}{count matrix for a set of n patients and 96 trinucleotides.}

\item{K}{a range of numeric value (each of them greater than 1) indicating the number of signatures to be discovered.}

\item{starting_beta}{a list of starting beta value for each configuration of K. If it is NULL, starting betas are estimated by 
NMF.}

\item{background_signature}{background signature to be used. If not provided, a warning is thrown and an initial value for it is 
estimated by NMF. If beta is not NULL, this parameter is ignored.}

\item{normalize_counts}{if true, the input count matrix x is normalize such that the patients have the same number of mutation.}

\item{nmf_runs}{number of iteration (minimum 1) of NMF to be performed for a robust estimation of starting beta. If beta is not NULL, 
this parameter is ignored.}

\item{bootstrap_repetitions}{Number of time bootstrap should be repeated. Higher values result in better estimate, but are computationally more expensive.}

\item{iterations}{Number of iterations to be performed. Each iteration corresponds to a first step where beta is fitted 
and a second step where alpha is fitted.}

\item{max_iterations_lasso}{Number of maximum iterations to be performed during the sparsification via Lasso.}

\item{num_processes}{Number of processes to be used during parallel execution. To execute in single process mode, 
this parameter needs to be set to either NA or NULL.}

\item{seed}{Seed for reproducibility.}

\item{verbose}{boolean; Shall I print all messages?}

\item{log_file}{log file where to print outputs when using parallel. If parallel execution is disabled, this parameter is ignored.}
}
\value{
A list of 3 elements: stability, RSS and evar. Here, stability reports the estimared cosine similarity for alpha and beta at each bootstrap 
repetition; RSS reports for each configuration the estimated residual sum of squares; finally, evar reports the explained variance.
}
\description{
Perform the evaluation of different nmfLasso solutions by bootstrap for K (unknown) somatic mutational signatures given a set of observations x. The estimation can slow down because of 
memory usage and intensive computations, when a big number of bootstrap repetitions is asked and when the analysis is performed for a big range of signatures (K). In this case, 
an advice may be to split the computation into multiple smaller sets.
}
\examples{
data(background)
data(patients)
res = nmfLassoBootstrap(x=patients[1:100,],
     K=3:5,
     background_signature=background,
     nmf_runs=1,
     bootstrap_repetitions=2,
     num_processes=NA,
     seed=12345)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.import.R
\name{import.trinucleotides.counts}
\alias{import.trinucleotides.counts}
\title{import.trinucleotides.counts}
\usage{
import.trinucleotides.counts(data, reference = NULL)
}
\arguments{
\item{data}{a data.frame with variants having 6 columns: sample name, chromosome, start position, end position, ref, alt.}

\item{reference}{a BSgenome object with the reference genome to be used to retrieve flanking bases.}
}
\value{
A matrix with trinucleotides counts per patient.
}
\description{
Make trinucleotides counts matrix from input data for a given reference genome.
}
\examples{
data(ssm560_reduced)
library("BSgenome.Hsapiens.1000genomes.hs37d5")
trinucleotides_counts = import.trinucleotides.counts(data=ssm560_reduced, 
     reference=BSgenome.Hsapiens.1000genomes.hs37d5)


}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{background}
\alias{background}
\title{germline replication error}
\format{
vector of rates
}
\source{
Rahbari, Raheleh, et al. "Timing, rates and spectra of human germline mutation." Nature genetics 48.2 (2016): 126.
}
\usage{
data(background)
}
\value{
vector of rates for the 96 trinucleotides
}
\description{
germline replication error estimated in Rahbari, Raheleh, et al. (2016).
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{cv_example}
\alias{cv_example}
\title{example of results obtained with the function nmf.LassoCV on the counts input from Nik-Zainal, Serena, et al. (2016).}
\format{
results obtained with the function nmf.LassoCV on the counts input from Nik-Zainal, Serena, et al. (2016)
}
\usage{
data(cv_example)
}
\value{
results obtained with the function nmf.LassoCV on the counts input from Nik-Zainal, Serena, et al. (2016)
}
\description{
example of results obtained with the function nmf.LassoCV on the counts input from Nik-Zainal, Serena, et al. (2016).
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/signatures.assignment.lasso.R
\name{sigAssignmentLasso}
\alias{sigAssignmentLasso}
\title{sigAssignmentLasso}
\usage{
sigAssignmentLasso(
  x,
  beta,
  normalize_counts = TRUE,
  lambda_rate_alpha = 0.05,
  max_iterations_lasso = 10000,
  seed = NULL,
  verbose = TRUE
)
}
\arguments{
\item{x}{count matrix for a set of n patients and 96 trinucleotides.}

\item{beta}{beta to be fixed during the estimation of alpha.}

\item{normalize_counts}{if true, the input count matrix x is normalize such that the patients have the same number of mutation.}

\item{lambda_rate_alpha}{value of LASSO to be used for alpha between 0 and 1. This value should be greater than 0. 1 is the value of LASSO 
that would shrink all the exposure values to 0 within one step. The higher lambda_rate_alpha is, the sparser are the resulting exposure values, 
but too large values may result in a reduced fit of the observed counts.}

\item{max_iterations_lasso}{Number of maximum iterations to be performed during the sparsification via Lasso.}

\item{seed}{Seed for reproducibility.}

\item{verbose}{boolean; Shall I print all messages?}
}
\value{
A list with the discovered signatures and their assignment to patients. It includes 2 elements: 
             alpha: matrix of the assigned exposure values
             beta: matrix of the discovered signatures
}
\description{
Perform the assignment of somatic mutational signatures to patients given a set of observed counts x and signatures beta.
}
\examples{
data(patients)
data(starting_betas_example)
beta = starting_betas_example[["5_signatures","Value"]]
res = sigAssignmentLasso(x=patients[1:100,],beta=beta,lambda_rate_alpha=0.05,seed=12345)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/visualization.R
\name{signatures.plot}
\alias{signatures.plot}
\title{signatures.plot}
\usage{
signatures.plot(beta, useRowNames = FALSE, xlabels = FALSE)
}
\arguments{
\item{beta}{matrix with the inferred mutational signatures.}

\item{useRowNames}{boolean value; shall I use the rownames from beta as names for the signatures?}

\item{xlabels}{boolean value; shall I display x labels?}
}
\value{
A ggplot2 object.
}
\description{
Plot the inferred mutational signatures.
}
\examples{
data(nmf_LassoK_example)
signatures.plot(beta=nmf_LassoK_example$beta)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/signatures.discovery.lasso.R
\name{startingBetaEstimation}
\alias{startingBetaEstimation}
\title{startingBetaEstimation}
\usage{
startingBetaEstimation(
  x,
  K = 3:10,
  background_signature = NULL,
  normalize_counts = TRUE,
  nmf_runs = 10,
  seed = NULL,
  verbose = TRUE
)
}
\arguments{
\item{x}{count matrix for a set of n patients and 96 trinucleotides.}

\item{K}{numeric value (minimum 2) indicating the number of signatures to be discovered.}

\item{background_signature}{background signature to be used. If not provided, a warning is thrown and an initial value for it is 
estimated by NMF. If beta is not NULL, this parameter is ignored.}

\item{normalize_counts}{if true, the input count matrix x is normalize such that the patients have the same number of mutation.}

\item{nmf_runs}{number of iteration (minimum 1) of NMF to be performed for a robust estimation of starting beta. If beta is not NULL, 
this parameter is ignored.}

\item{seed}{Seed for reproducibility.}

\item{verbose}{boolean; Shall I print all messages?}
}
\value{
A list containing the starting beta values for each configuration of K.
}
\description{
Perform a robust estimation of the starting betas for the nmfLasso method
}
\examples{
data(background)
data(patients)
res = startingBetaEstimation(x=patients[1:100,],
     K=3:5,
     background_signature=background,
     nmf_runs=1,
     seed=12345)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/signatures.discovery.lasso.R
\name{nmfLasso}
\alias{nmfLasso}
\title{nmfLasso}
\usage{
nmfLasso(
  x,
  K,
  beta = NULL,
  background_signature = NULL,
  normalize_counts = TRUE,
  nmf_runs = 10,
  lambda_rate_alpha = 0.05,
  lambda_rate_beta = 0.05,
  iterations = 30,
  max_iterations_lasso = 10000,
  seed = NULL,
  verbose = TRUE
)
}
\arguments{
\item{x}{count matrix for a set of n patients and 96 trinucleotides.}

\item{K}{numeric value (minimum 2) indicating the number of signatures to be discovered.}

\item{beta}{starting beta for the estimation. If it is NULL, starting beta is estimated by NMF.}

\item{background_signature}{background signature to be used. If not provided, a warning is thrown and an initial value for it is 
estimated by NMF. If beta is not NULL, this parameter is ignored.}

\item{normalize_counts}{if true, the input count matrix x is normalize such that the patients have the same number of mutation.}

\item{nmf_runs}{number of iteration (minimum 1) of NMF to be performed for a robust estimation of starting beta. If beta is not NULL, 
this parameter is ignored.}

\item{lambda_rate_alpha}{value of LASSO to be used for alpha between 0 and 1. This value should be greater than 0. 1 is the value of LASSO 
that would shrink all the exposure values to 0 within one step. The higher lambda_rate_alpha is, the sparser are the resulting exposure values, 
but too large values may result in a reduced fit of the observed counts.}

\item{lambda_rate_beta}{value of LASSO to be used for beta between 0 and 1. This value should be greater than 0. 1 is the value of LASSO 
that would shrink all the signatures to 0 within one step. The higher lambda_rate_beta is, the sparser are the resulting signatures, 
but too large values may result in a reduced fit of the observed counts.}

\item{iterations}{Number of iterations to be performed. Each iteration corresponds to a first step where beta is fitted 
and a second step where alpha is fitted.}

\item{max_iterations_lasso}{Number of maximum iterations to be performed during the sparsification via Lasso.}

\item{seed}{Seed for reproducibility.}

\item{verbose}{boolean; Shall I print all messages?}
}
\value{
A list with the discovered signatures. It includes 6 elements: 
             alpha: matrix of the discovered exposure values
             beta: matrix of the discovered signatures
             starting_alpha: initial alpha on which the method has been applied
             starting_beta: initial beta on which the method has been applied
             loglik_progression: log-likelihood values during the iterations. This values should be increasing, if not the selected value of lambda is too high
             best_loglik: log-likelihood of the best signatures configuration
}
\description{
Perform the discovery of K somatic mutational signatures given a set of observed counts x.
}
\examples{
data(patients)
data(starting_betas_example)
beta = starting_betas_example[["5_signatures","Value"]]
res = nmfLasso(x=patients[1:100,],
     K=5,
     beta=beta,
     lambda_rate_alpha=0.05,
     lambda_rate_beta=0.05,
     iterations=5,
     seed=12345)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{mutation_categories}
\alias{mutation_categories}
\title{trinucleotides mutation categories}
\format{
matrix of 96 trinucleotides mutation categories
}
\usage{
data(mutation_categories)
}
\value{
matrix of 96 trinucleotides mutation categories
}
\description{
96 trinucleotides mutation categories
}
